package digsec.set.core.crypto;

import digsec.set.core.pkcs.*;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.security.*;

/**
  *	<pre>
  *	2830 DetachedDigest ::= DigestedData                          -- No parameter
  *	2831    (WITH COMPONENTS {..., contentInfo (WITH COMPONENTS
  *	2832                     {..., content ABSENT}) })
  * 	</pre>
  */

public class DetachedDigest extends DigestedData implements DerEncoder{

	public DetachedDigest(DerInputStream din) throws Exception {
		super(din);
	}

	public DetachedDigest(ContentInfo cInfo) throws Exception {
		super();
		byte[] der = cInfo.getContent().toByteArray();
		
		MessageDigest md = MessageDigest.getInstance("SHA");
		md.update(der,0,der.length);
		byte[] digest=md.digest();
	
		ContentInfo newContent = new ContentInfo(cInfo.getContentType(),
							null);
		ddVersion = new BigInt(0);
		digestAlgorithm=new AlgorithmId(AlgorithmId.SHA_oid);
		contentInfo=newContent;
		this.digest=digest;
	
	}

	public boolean verifyDigest(ContentInfo cInfo) throws Exception {
		
		byte[] der = cInfo.getContent().toByteArray();
	
		MessageDigest md = MessageDigest.getInstance("SHA");
		md.update(der,0,der.length);
		byte[] digest=md.digest();

		return MessageDigest.isEqual(this.digest,digest);
	}

	public static void main(String[] args) {
	   try{
		ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						new DerValue(DerValue.tag_UniversalString, "tahi kami sudah bulat".getBytes()));
		DetachedDigest dd = new DetachedDigest(cInfo);

		DerOutputStream dout = new DerOutputStream();
		dd.encode(dout);
		HexDumpEncoder hde = new HexDumpEncoder();
		System.out.println(hde.encodeBuffer(dout.toByteArray()));

		dd = new DetachedDigest(new DerInputStream(dout.toByteArray()));
		System.out.println(dd);
		System.out.println("Verify Digest: " + dd.verifyDigest(cInfo));
	   }
	   catch(Exception e) {
		e.printStackTrace();
	   }
	}

}
	