package digsec.set.core.crypto;
import digsec.set.core.pkcs.*;
import java.io.*;
import sun.security.util.*;
import sun.security.x509.X500Name;
import sun.misc.HexDumpEncoder;
import java.security.Security;
import java.security.SecureRandom;
import java.security.cert.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.crypto.provider.*;

/**
 *	<pre> 
 *	2937 EK { KeyData, ToBeEnveloped } ::= EncryptedData
 *	2938    (CONSTRAINED BY { ToBeEnveloped, -- encrypted with -- KeyData})
 *	2939    (WITH COMPONENTS { ..., encryptedContentInfo
 *	2940                    (WITH COMPONENTS { ..., encryptedContent PRESENT}) })
 *	</pre>
 */

public class EK implements DerEncoder
{
	private DESKey key;
	private byte[] IVData;
	private AlgorithmId parameter;
	private ContentInfo toBeEnveloped;
	private BigInt version;
	private EncryptedContentInfo encContentInfo;
	private EncryptedData encryptedData;
	private static int[] id_desCBC = {1,3,14,3,2,7};
	public static ObjectIdentifier DES_CBC_oid = new ObjectIdentifier(id_desCBC);

	/**
	 * Constructor for EK
	 *
	 */
	public EK(DESKey k, ContentInfo ci) throws Exception {
		key = k;
		toBeEnveloped = ci;
		encContentInfo = makeEncryptedContentInfo(key,toBeEnveloped);	
		encryptedData = new EncryptedData(encContentInfo);
	}

	public EK(DESKey k, DerInputStream din) throws Exception {
		key = k;
		//ContentInfo contentInfo = new ContentInfo(din);
		encryptedData = new EncryptedData(din);

		//ambil content
		//DerValue dv = contentInfo.getContent();
		//DerInputStream din1 = new DerInputStream(dv.toByteArray());

		//ambil dua DerValue
		//adv[0] berisi version dari EncryptedData
		//adv[1] berisi EncryptedContentInfo
		//DerValue[] adv = din1.getSequence(2);
		//isi version
		//version = new BigInt(adv[0].getInteger().toInt());
		//isi encryptedContentInfo
		//encContentInfo = new EncryptedContentInfo(new DerInputStream(adv[1].toByteArray()));	
		encContentInfo = encryptedData.getEncryptedContentInfo();

		//ambil parameter yang berisi IVData
		parameter = encContentInfo.getContentEncryptionAlgorithm();

		//ambil IVData
		byte[] param = parameter.getEncodedParams();
		DerValue derIv = new DerValue(param);
		IVData = derIv.getOctetString();

		//ambil encryptedContent untuk mengambil toBeEnveloped
		byte[] encryptedContent = encContentInfo.getEncryptedContent();
		byte[] plain = decryptDES(encryptedContent, key);

		//make toBeEnveloped
		toBeEnveloped = new ContentInfo(encContentInfo.getContentType(),
				 	              new DerValue(plain));
	}

	private byte[] makeEncryptedContent(DESKey k, ContentInfo ci) throws Exception {
		//ambil content, dan jadikan array of byte
		byte[] data = ci.getContent().toByteArray();
		//encrypt menggunakan key, lalu IVData terisi
		byte[] cipher = encryptDES(data,k);
		return cipher;
	}

	private EncryptedContentInfo makeEncryptedContentInfo(DESKey k, ContentInfo ci) throws Exception {
		//make encryptedContent for EncryptedContentInfo
		byte[] encryptedContent = makeEncryptedContent(key,ci);

		//Create algorithm identifier including the IV parameter
		parameter = new AlgorithmId(DES_CBC_oid, 
						    new DerValue(DerValue.tag_OctetString,IVData));

		//Create EncryptedContentInfo
		return new EncryptedContentInfo(toBeEnveloped.getContentType(),parameter,encryptedContent);			
	}
		
	private byte[] encryptDES (byte[] plain, DESKey key) throws Exception {
		int i=0;
		Cipher c = Cipher.getInstance("DES/CBC/PKCS5Padding", "ABA");

		//inisialisasi untuk enkrip menggunakan private key 
		c.init(c.ENCRYPT_MODE, key);
			
		i = c.getOutputSize(plain.length);
		IVData = c.getIV();
		byte[] cipher = new byte[i];
		i = c.doFinal(plain,0,plain.length,cipher);

		return cipher;
	}

	private byte[] decryptDES (byte[] cipher, DESKey key) throws Exception {
		int i=0;
		Cipher c = Cipher.getInstance("DES/CBC/PKCS5Padding");
		IvParameterSpec iv = new IvParameterSpec(IVData);

		//inisialisasi untuk dekrip menggunakan public key 
		c.init(c.DECRYPT_MODE, key, iv);
			
		i = c.getOutputSize(cipher.length);
		byte[] tempPlain = new byte[i];
		i = c.doFinal(cipher,0,cipher.length,tempPlain);
		byte[] plain = new byte[i];
		System.arraycopy(tempPlain,0,plain,0,i);

		return plain;
	}

	public void derEncode(OutputStream out) throws IOException {
		encryptedData.derEncode(out);
	}	

	public void encode(DerOutputStream dout) throws IOException {
		encryptedData.encode(dout);
	}	

	public DESKey getKey() {
		return key;
	}
	
	public ContentInfo getContentInfo() {
		return toBeEnveloped;
	}

	public EncryptedContentInfo getEncryptedContentInfo() throws Exception {
		return encContentInfo;
	}

	public BigInt getVersion() throws Exception {
		return version;
	}

	public String toString() {
		String s = "Object EK\n";
        	if(toBeEnveloped != null)
        		s = s + "EK :: toBeEnveloped: \n" + toBeEnveloped + "\n";
        	if(encryptedData != null)
        		s = s + "EK :: EncryptedData: \n" + encryptedData;
	
        return s;
	}

	public static void main(String[] args) {
		try {
			Security.addProvider(new ABAProvider());

			//make ContentInfo
			String strParam = "Peringatan pemerintah, merokok membahayakan keuangan anda, ";
			//strParam = strParam + "terutama jika rokoknya adalah rokok-rokok luar yang mahal";
			byte[] param = strParam.getBytes();
                  
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,param));		

			//buat DESKey
			byte[] seedBytes = {2,5,3,4};
			SecureRandom myRandom = new SecureRandom(seedBytes);
			KeyGenerator desKeyGen = KeyGenerator.getInstance("DES", "ABA");
			desKeyGen.init(myRandom);
			DESKey myKey = (DESKey)desKeyGen.generateKey();

			EK ek = new EK(myKey, cInfo);
			System.out.print(ek.toString());
		
			System.out.println("*******reconstruct********");
			DerOutputStream dout = new DerOutputStream();
			ek.encode(dout);

			EK ek2 = new EK(myKey, new DerInputStream(dout.toByteArray()));
			System.out.print(ek2.toString());

			//encode lagi, terus recontstruct lagi
			System.out.println("*******reconstruct********");
			dout.reset();
			ek2.encode(dout);
			EK ek3 = new EK(myKey, new DerInputStream(dout.toByteArray()));
			System.out.print(ek3.toString());

		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}


}