package digsec.set.core.crypto;
import digsec.set.core.pkcs.*;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.io.*;

/**
 *	<pre> 
 * 	2821 L { T1, T2 } ::= SEQUENCE {                     -- Linkage from t1 to t2
 *	2822    t1  	T1,
 * 	2823    t2  	DD { T2 }                                  -- PKCS#7 DigestedData
 *	2824 }
 *	</pre>
 */

public class Linkage implements DerEncoder {

	public Linkage(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);
		tuple1 = new ContentInfo(new DerInputStream(adv[0].toByteArray()));
		DD_tuple2 = new DetachedDigest(new DerInputStream(adv[1].toByteArray()));
	}

	public Linkage(ContentInfo tuple1, ContentInfo tuple2) throws Exception {
		this.tuple1 = tuple1;
		DD_tuple2 = new DetachedDigest(tuple2);
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream dout1 = new DerOutputStream();
		tuple1.encode(dout1);
		DD_tuple2.encode(dout1);
		dout.putDerValue(new DerValue((byte)48, dout1.toByteArray()));
		byte[] output = dout.toByteArray();
		out.write(output,0,output.length);
	}

	public void encode(DerOutputStream dout) throws Exception {
		derEncode(dout);
	}

	public String toString() {
		String s = "";
		s += "L(tuple1,tuple2) ::= SEQUENCE\n";
        		if(tuple1!=null && DD_tuple2!=null) {
			s += "tuple1\t\t: " + "\n" + tuple1 +  "\n";
			s += "DD(tuple2)\t: " + "\n" + DD_tuple2 + "\n";
		}				
		return s;
	}

	public ContentInfo getTuple1() {
		return tuple1;
	}

	public DetachedDigest getDDTuple2() {
		return DD_tuple2;
	}

	private ContentInfo  tuple1;
	private DetachedDigest DD_tuple2;

	public static void main(String[] args) {
	   try{
		ContentInfo cInfo1 = new ContentInfo(ContentInfo.DATA_OID,
		new DerValue(DerValue.tag_UniversalString,"tekad kami sudah bulat".getBytes()));
		ContentInfo cInfo2 = new ContentInfo(ContentInfo.DATA_OID,
		new DerValue(DerValue.tag_UniversalString,"tahi kami sudah bulat".getBytes()));

		Linkage link = new Linkage(cInfo1,cInfo2);
		DerOutputStream dout = new DerOutputStream();
		link.encode(dout);
		HexDumpEncoder hde = new HexDumpEncoder();
		System.out.println(hde.encodeBuffer(dout.toByteArray()));	
	
		link = new Linkage(new DerInputStream(dout.toByteArray()));
		System.out.println(link);

		DetachedDigest dd = link.getDDTuple2();
		System.out.println("Verify DD(tuple2): " + dd.verifyDigest(cInfo2));
	   }
	   catch(Exception e) {
		e.printStackTrace();
	   }
	}
}