package digsec.set.core.encapsulation;
import digsec.set.core.crypto.*;
import digsec.set.core.pkcs.*;
import java.io.*;
import java.util.*;
import sun.security.util.*;
import java.security.cert.*;
import sun.security.x509.*;
import java.security.*;
import au.net.aba.crypto.provider.*;
import javax.crypto.*;
import sun.misc.HexDumpEncoder;

/**
 * 	<pre> 
 *	2869 Enc { SIGNER, RECIPIENT, T } ::= E {
 *	2870    RECIPIENT,
 *	2871    S { SIGNER, T }
 *	2872 }
 *	</pre>
 *	@author Priharsanta - DSEC-FASILKOM UI	
 */

public class Enc implements DerEncoder {

	//Field
	//Sender
	private X509Certificate[] s_certificates;

	//Recipient
	private X509Certificate r_certificate;
	
	//Content to be encapsulated
	private ContentInfo T;

	//Element of Encapsulation S and E
	private S Sst;
	private E ErSst;


	//Constructor

	/**
	 *	@param s_certificates - sender's certificates (max 2)
	 *	@param s_privs - sender's private-keys (max 2)
	 *	@param r_certificate - recipient's certificate
	 *	@param T - to be encapsulated
       */
	public Enc (X509Certificate[] s_certificates, 			
			PrivateKey[] s_privs, 
			X509Certificate r_certificate, 
			ContentInfo T) throws Exception {

		this.s_certificates = s_certificates;
		this.r_certificate = r_certificate;
		this.T = T;

		make_Sst(s_privs);
		make_ErSst();
	}

	public Enc (X509Certificate s_certificate, 			
			PrivateKey s_priv, 
			X509Certificate r_certificate, 
			ContentInfo T) throws Exception {

		this.s_certificates = new X509Certificate[1];
		this.s_certificates[0] = s_certificate;
		this.r_certificate = r_certificate;
		PrivateKey[] s_privs = new PrivateKey[1]; 
		s_privs[0] = s_priv;
		this.T = T;

		make_Sst(s_privs);
		make_ErSst();
	}


	/**
	 *	Sender's public key is obtained from Sender's Certificate which
	 *	contained in SignerInfo
	 *	@param r_priv - recipient's private-key
	 *	@param din - DerInputStream which contains DerValue of Enc
	 */	
	public Enc (PrivateKey r_priv,
			DerInputStream din) throws Exception {

		open_ErSst(r_priv,din);
		open_Sst();
	}


	//Methods

	//open the E(RECIPIENT, S(SIGNER,T))
	private void open_ErSst(PrivateKey r_priv, DerInputStream din) throws Exception {
		ErSst = new E(r_priv,din);
	}

	//Open the S(SIGNER,T)
	private void open_Sst() throws Exception {
		ContentInfo s = ErSst.getContent();
		//convert from content info to S
		DerOutputStream dout = new DerOutputStream();
		s.encode(dout);
		Sst = new S(new DerInputStream(dout.toByteArray()));
		T = Sst.getContentInfo();
		s_certificates = Sst.getCertificates();
	}

	//make S(SIGNER,T)	
	private void make_Sst(PrivateKey[] s_privs) throws Exception {
		
		if(s_certificates!=null && s_certificates.length==2)
			Sst = new S(s_privs,s_certificates,T);
		else if(s_certificates!=null && s_certificates.length==1){
			//System.out.println(s_privs[0].toString() + s_certificates[0].toString() + T.toString());
			Sst = new S(s_privs[0],s_certificates[0],T);
		}
		else 
			throw new IllegalArgumentException("Sender's Certificates not exist");	
	}
	
	//make E(RECIPIENT,S(SIGNER,T))
	private void make_ErSst() throws Exception {
		DerOutputStream dout = new DerOutputStream();
		Sst.encode(dout);		

		if(r_certificate!=null)
			ErSst = new E(r_certificate,
				  		new ContentInfo(
							new DerInputStream(dout.toByteArray())));
		else
			throw new IllegalArgumentException("Recipient's Certificate not exist");
	}

	public void derEncode(OutputStream out) throws IOException {
		ErSst.derEncode(out);
	}

	public void encode(DerOutputStream dout) throws Exception {
		ErSst.encode(dout);
	}

	public ContentInfo getContentInfo() {
		return T;
	}

	public X509Certificate[] getSenderCertificates() {
		return s_certificates;
	}

	public X509Certificate getSenderCertificate() {
		return s_certificates[0];
	}
	
	public String toString() {
		String s = "Simple Encapsulation\n";
		s+=ErSst;
		s+=Sst;
		s+="To Be Enveloped:\n" + T;
		s+="Sender's Certificates:\n";
		for(int i=0; i<s_certificates.length; i++)
			s+= s_certificates[i] + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			Security.addProvider(new ABAProvider());

			ContentInfo toBeEncapsulated = new ContentInfo(new ObjectIdentifier("1.2.3.4"),
										     new DerValue("Dodol Garut Kamu"));   		

			//ContentInfo toBeEncapsulated1 = new ContentInfo(new ObjectIdentifier("1.2.3.4"),
						//				     new DerValue("Dodol Garuk"));   		

						
			//make sender's certificate
			X509Certificate[] s_certs = new X509Certificate[2];

			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();

			s_certs[0] = s_cert;
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
			cf = CertificateFactory.getInstance("X.509");
  			s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			s_certs[1] = s_cert;

			//make recipient's certificate
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
 			cf = CertificateFactory.getInstance("X.509");
  			X509Certificate r_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get sender's PrivateKey
			PrivateKey[] s_privs = new PrivateKey[2];
			
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.keystore");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"priharsanta".toCharArray());
			PrivateKey s_priv = (PrivateKey) ks.getKey("priharsanta", "priharsanta".toCharArray());			
			s_privs[0] = s_priv;		
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			s_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			
			s_privs[1] = s_priv;


			Enc enc = new Enc(s_certs,s_privs,r_cert,toBeEncapsulated);
			//Enc enc1 = new Enc(s_cert,s_priv,r_cert,toBeEncapsulated);

			//System.out.println(enc);	
			
			DerOutputStream dout = new DerOutputStream();
			enc.encode(dout);
			byte[] DerEnc = dout.toByteArray();
			//DerEnc[167] = (byte) 0x00;	

			HexDumpEncoder hde = new HexDumpEncoder();
			//System.out.println(hde.encodeBuffer(dout.toByteArray()));

			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			PrivateKey r_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			


			enc = new Enc(r_priv, new DerInputStream(DerEnc));
			dout.reset();
			enc.encode(dout);

			enc = new Enc(r_priv, new DerInputStream(dout.toByteArray()));			
		
			System.out.println(enc);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}	
}