package digsec.set.core.encapsulation;
import digsec.set.core.crypto.*;
import digsec.set.core.pkcs.*;
import java.io.*;
import java.util.*;
import sun.security.util.*;
import java.security.cert.*;
import sun.security.x509.*;
import java.security.*;
import au.net.aba.crypto.provider.*;
import javax.crypto.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.*;

/**
 *	<pre>
 *	2897 EncB { SIGNER, RECIPIENT, T, Baggage } ::= SEQUENCE {
 *	2898    enc      Enc { SIGNER, RECIPIENT, L { T, Baggage } },
 *	2899    baggage  Baggage
 *	2900 }
 *	</pre>
 */

public class EncB implements DerEncoder {

	//Fields

	//Sender
	private X509Certificate[] s_certificates;
	
	//Recipient
	private X509Certificate r_certificate;
	
	//Content to be encapsulated
	private ContentInfo T;
	private ContentInfo Baggage;

	//Enc and L
	private Linkage L;
	private Enc enc;

	//Constructor
	/**
	 *	@param s_certs - sender's certificates
	 *	@param r_cert  - recipient's  certificate
	 *	@param T	   - to be encapsulated
	 *	@param param   - extra parameter implements OAEPEncoder
 	 */
	public EncB(X509Certificate[] s_certs, 
			PrivateKey[] s_privs,
		      X509Certificate r_cert, 
		      ContentInfo T, 
			ContentInfo B) throws Exception {
			
		s_certificates=s_certs;
		r_certificate=r_cert;
		this.T=T;
		Baggage=B;
		L = new Linkage(T,B);
		makeEnc(s_privs);	
	}

	public EncB(X509Certificate s_cert, 
			PrivateKey s_priv,
		      X509Certificate r_cert, 
		      ContentInfo T, 
			ContentInfo B) throws Exception {
			
		s_certificates = new X509Certificate[1];
		s_certificates[0] = s_cert;
		r_certificate=r_cert;
		this.T=T;
		Baggage=B;
		L = new Linkage(T,B);
		PrivateKey[] s_privs = new PrivateKey[1];
		s_privs[0]=s_priv;
		makeEnc(s_privs);	
	}

	public EncB(PrivateKey r_priv, DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);
		enc = new Enc(r_priv, new DerInputStream(adv[0].toByteArray()));
		Baggage = new ContentInfo(new DerInputStream(adv[1].toByteArray()));	
		openEnc();
		if(!verify())
			throw new FalseVerifiedException("EncB");		
	}

	private void openEnc() throws Exception {
		ContentInfo ci = enc.getContentInfo();
		DerValue dv = ci.getContent();
		L = new Linkage(new DerInputStream(dv.toByteArray()));
		T = L.getTuple1();
		s_certificates = enc.getSenderCertificates();
	}

	private boolean verify() throws Exception {
		Linkage link = new Linkage(T,Baggage);
		DetachedDigest dd1 = link.getDDTuple2();		
		DetachedDigest dd2 = L.getDDTuple2();
		return dd1.equals(dd2);
	}
	
	private void makeEnc(PrivateKey[] s_privs) throws Exception {
		//convert link to content info
		DerOutputStream dout = new DerOutputStream();
		L.encode(dout);
		ContentInfo ci = new ContentInfo(T.getContentType(),
						         new DerValue(dout.toByteArray()));
		//make Enc(SENDER,RECIPIENT,L(T,B))
		enc = new Enc(s_certificates,s_privs,r_certificate,ci);
	}	

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		enc.derEncode(dout);
		Baggage.encode(dout);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		derEncode(dout);
	}

	public ContentInfo getContentInfo() {
		return T;
	}

	public ContentInfo getBaggage() {
		return Baggage;
	}

	public X509Certificate[] getSenderCertificates() {
		return s_certificates;
	}

	public X509Certificate getSenderCertificate() {
		return s_certificates[0];
	}

	public String toString() {
		String s = "EncB\n";
		s += enc;
		s += T;
		s += Baggage;
		return s;
	}

	public static void main(String[] args) {
		try {
			Security.addProvider(new ABAProvider());

			ContentInfo toBeEncapsulated = new ContentInfo(new ObjectIdentifier("1.2.3.4"),
										     new DerValue("Dodol Garut Kamu"));   		

			ContentInfo bag = new ContentInfo(new ObjectIdentifier("1.3.5.7"),
								    new DerValue("Cewek Kampret"));
			//byte[] nonce = {1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9,0};
			//PANOnly panOnly = new PANOnly("Priharsanta",nonce); 
			
			//make sender's certificate
			X509Certificate[] s_certs = new X509Certificate[2];

			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();

			s_certs[0] = s_cert;
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
			cf = CertificateFactory.getInstance("X.509");
  			s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			s_certs[1] = s_cert;

			//make recipient's certificate
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
 			cf = CertificateFactory.getInstance("X.509");
  			X509Certificate r_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get sender's PrivateKey
			PrivateKey[] s_privs = new PrivateKey[2];
			
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.keystore");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"priharsanta".toCharArray());
			PrivateKey s_priv = (PrivateKey) ks.getKey("priharsanta", "priharsanta".toCharArray());			
			s_privs[0] = s_priv;		
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			s_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			
			s_privs[1] = s_priv;


			EncB encB = new EncB(s_certs,s_privs,r_cert,toBeEncapsulated,bag);
			

			System.out.println(encB);	
			
			DerOutputStream dout = new DerOutputStream();
			encB.encode(dout);
			byte[] DerEncB = dout.toByteArray();
			//DerEnc[167] = (byte) 0x00;	

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(DerEncB));

			System.out.println("---------------Reconstructing---------------");

			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			PrivateKey r_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			


			encB = new EncB(r_priv, new DerInputStream(DerEncB));
			System.out.println(encB);


			dout.reset();
			encB.encode(dout);

			System.out.println("---------------Reconstructing Again ---------------");
		
			encB = new EncB(r_priv, new DerInputStream(dout.toByteArray()));			
		
			System.out.println(encB);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
		
}
