package digsec.set.core.encapsulation;
import digsec.set.core.crypto.*;
import digsec.set.core.pkcs.*;
import java.io.*;
import java.util.*;
import java.security.Security;
import java.security.SecureRandom;
import java.security.PrivateKey;
import java.security.cert.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import sun.misc.HexDumpEncoder;
import sun.security.util.*;
import sun.security.x509.*;
import au.net.aba.crypto.provider.*;

/**
 *	<pre>
 *	2877 EncK { KeyData, SIGNER, T } ::= EK{
 *	2878       KeyData,
 *	2879       S { SIGNER, T }
 *	2880 }
 *	</pre>
 */

public class EncK implements DerEncoder {

	//fields
	//KeyData
	private DESKey dek;
	
	//Signer
	private X509Certificate[] s_certificates;

	//toBeEncK
	private ContentInfo T;

	//Bonus
	private S signature;
	private EK energiKinetik;

	//Constructor
	/**
	 *	<pre>
	 *	@param dek - DES Key
	 *	@param s_certs - Sender's certificates
	 *	@param T - to be EncK
	 *	</pre>
	 */
	public EncK(DESKey dek,
			PrivateKey[] s_privs, 
			X509Certificate[] s_certs, 
			ContentInfo T) throws Exception {
 
		this.T = T;
		s_certificates = s_certs;
		this.dek = dek;
		makeS(s_privs);
		makeEK();
	}

	public EncK(DESKey dek, 
			PrivateKey s_priv,
			X509Certificate s_cert, 
			ContentInfo T) throws Exception {
 		
		this.T = T;
		s_certificates = new X509Certificate[1];
		s_certificates[0] = s_cert;
		this.dek = dek;
		PrivateKey[] s_privs = new PrivateKey[1];
		s_privs[0] = s_priv;
		makeS(s_privs);
		makeEK();
	}

	public EncK(DESKey dek, DerInputStream din) throws Exception {
		this.dek=dek;
		energiKinetik = new EK(dek,din);
		getSignature();
		
	}

	private void getSignature() throws Exception {
		ContentInfo ci = energiKinetik.getContentInfo();
		DerOutputStream dout = new DerOutputStream();
		ci.encode(dout);
		signature = new S(new DerInputStream(dout.toByteArray()));
		T = signature.getContentInfo();
		s_certificates = signature.getCertificates();
	}

	private void makeS(PrivateKey[] s_privs) throws Exception {
		if(s_certificates.length==2 && s_privs.length==2)
			signature = new S(s_privs,s_certificates,T);
		else if(s_certificates.length==1 && s_privs.length==1)
			signature = new S(s_privs[0],s_certificates[0],T);
		else
			throw new IllegalArgumentException("EncK");
	}

	private void makeEK() throws Exception {
		DerOutputStream dout = new DerOutputStream();
		signature.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));
		energiKinetik = new EK(dek,ci);
	}
	
	public void derEncode(OutputStream out) throws IOException {
		energiKinetik.derEncode(out);
	}

	public void encode(DerOutputStream dout) throws Exception {
		energiKinetik.encode(dout);
	}

	public X509Certificate[] getSenderCertificates() {
		return s_certificates;
	}

	public X509Certificate getSenderCertificate() {
		return s_certificates[0];
	}

	public ContentInfo getContentInfo() {
		return T;
	}

	public String toString() {
		String s = "EncK\n";
		s += energiKinetik;
		s += T;
		for(int i=0; i<s_certificates.length; i++) {
			s += s_certificates[i] + "\n";
		}
		return s;
	}

	public static void main(String[] args) {
		try {
			Security.addProvider(new ABAProvider());
			byte[] seedBytes = {2,5,3,4,7};
			SecureRandom myRandom = new SecureRandom(seedBytes);

			//make contentInfo
			String strParam = "Peringatan pemerintah, merokok membahayakan keuangan anda, ";
			strParam = strParam + "terutama jika rokoknya adalah rokok-rokok luar yang mahal";
			byte[] param = strParam.getBytes();
                  
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
							            new DerValue(strParam));

			//make sender's certificate
			X509Certificate[] s_certs = new X509Certificate[2];

			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();

			s_certs[0] = s_cert;
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.cert");
			cf = CertificateFactory.getInstance("X.509");
  			s_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			s_certs[1] = s_cert;
	
			//get sender's PrivateKey
			PrivateKey[] s_privs = new PrivateKey[2];
			
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\priharsanta.keystore");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"priharsanta".toCharArray());
			PrivateKey s_priv = (PrivateKey) ks.getKey("priharsanta", "priharsanta".toCharArray());			
			s_privs[0] = s_priv;		
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\joint\\iduy.keystore");
			ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"prayudi".toCharArray());
			s_priv = (PrivateKey) ks.getKey("iduy", "prayudi".toCharArray());			
			s_privs[1] = s_priv;


			//buat DESKey
			KeyGenerator desKeyGen = KeyGenerator.getInstance("DES", "ABA");
			desKeyGen.init(myRandom);
			DESKey myKey = (DESKey)desKeyGen.generateKey();


			//make EncK object
			EncK enck = new EncK(myKey, s_privs, s_certs, cInfo);
			System.out.println(enck);

/*			ContentInfo ci = enck.getToBeEnveloped();
			System.out.println(ci.toString());			
			S s = enck.getS();
			System.out.println(s.toString());			
*/			
			System.out.println("*******reconstruct********");
			DerOutputStream dout = new DerOutputStream();
			enck.encode(dout);
			EncK enck2 = new EncK(myKey, new DerInputStream(dout.toByteArray()));
			System.out.println(enck2);

		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}
}	