package digsec.set.core.pkcs;

import java.io.*;
import java.security.*;
import java.util.*;
import sun.security.util.*;

public class AlgorithmId
    implements Serializable, DerEncoder
{

    public AlgorithmId()
    {
    }

    public AlgorithmId(ObjectIdentifier objectidentifier)
    {
        algid = objectidentifier;
    }

    public AlgorithmId(ObjectIdentifier objectidentifier, DerValue dervalue)
        throws IOException
    {
        algid = objectidentifier;
        params = dervalue;
        if(params != null)
            decodeParams();
    }

    private String algName()
    {
        if(algid.equals(MD5_oid))
            return "MD5";
        if(algid.equals(MD2_oid))
            return "MD2";
        if(algid.equals(SHA_oid))
            return "SHA";
        if(algid.equals(RSAEncryption_oid) || algid.equals(RSA_oid))
            return "RSA";
        if(algid.equals(DH_oid) || algid.equals(DH_PKIX_oid))
            return "Diffie-Hellman";
        if(algid.equals(DSA_oid) || algid.equals(DSA_OIW_oid))
            return "DSA";
        if(algid.equals(md5WithRSAEncryption_oid))
            return "MD5withRSA";
        if(algid.equals(md2WithRSAEncryption_oid))
            return "MD2withRSA";
        if(algid.equals(sha1WithDSA_oid) || algid.equals(sha1WithDSA_OIW_oid) || algid.equals(shaWithDSA_OIW_oid))
            return "SHA1withDSA";
        if(algid.equals(sha1WithRSAEncryption_oid) || algid.equals(sha1WithRSAEncryption_OIW_oid))
            return "SHA1withRSA";
        else
            return algid.toString();
    }

    private static ObjectIdentifier algOID(String s)
    {
        if(s.indexOf(46) != -1)
            if(s.startsWith("OID."))
                return new ObjectIdentifier(s.substring("OID.".length()));
            else
                return new ObjectIdentifier(s);
        if(s.equalsIgnoreCase("MD5"))
            return MD5_oid;
        if(s.equalsIgnoreCase("MD2"))
            return MD2_oid;
        if(s.equalsIgnoreCase("SHA") || s.equalsIgnoreCase("SHA1") || s.equalsIgnoreCase("SHA-1"))
            return SHA_oid;
        if(s.equalsIgnoreCase("RSA"))
            return RSAEncryption_oid;
        if(s.equalsIgnoreCase("Diffie-Hellman") || s.equalsIgnoreCase("DH"))
            return DH_oid;
        if(s.equalsIgnoreCase("DSA"))
            return DSA_oid;
        if(s.equalsIgnoreCase("MD5withRSA") || s.equalsIgnoreCase("MD5/RSA"))
            return md5WithRSAEncryption_oid;
        if(s.equalsIgnoreCase("MD2withRSA") || s.equalsIgnoreCase("MD2/RSA"))
            return md2WithRSAEncryption_oid;
        if(s.equalsIgnoreCase("SHAwithDSA") || s.equalsIgnoreCase("SHA1withDSA") || s.equalsIgnoreCase("SHA/DSA") || s.equalsIgnoreCase("SHA1/DSA") || s.equalsIgnoreCase("DSAWithSHA1"))
            return sha1WithDSA_oid;
        if(s.equalsIgnoreCase("SHA1WithRSA") || s.equalsIgnoreCase("SHA1/RSA"))
            return sha1WithRSAEncryption_OIW_oid;
        if(!initOidTable)
        {
            java.security.Provider aprovider[] = Security.getProviders();
            for(int i = 0; i < aprovider.length; i++)
            {
                Enumeration enumeration = aprovider[i].keys();
                while(enumeration.hasMoreElements()) 
                {
                    String s2 = (String)enumeration.nextElement();
                    int j;
                    if(!s2.toUpperCase().startsWith("ALG.ALIAS") || (j = s2.toUpperCase().indexOf("OID.", 0)) == -1)
                        continue;
                    j += "OID.".length();
                    if(j == s2.length())
                        break;
                    if(oidTable == null)
                        oidTable = new Hashtable(11);
                    String s1 = s2.substring(j);
                    String s3 = aprovider[i].getProperty(s2).toUpperCase();
                    if(oidTable.get(s3) == null)
                        oidTable.put(s3, new ObjectIdentifier(s1));
                }

            }

            initOidTable = true;
        }
        return (ObjectIdentifier)oidTable.get(s.toUpperCase());
    }

    protected void decodeParams()
        throws IOException
    {
        try
        {
            algParams = AlgorithmParameters.getInstance(algid.toString());
        }
        catch(NoSuchAlgorithmException _ex)
        {
            algParams = null;
            return;
        }
        algParams.init(params.toByteArray());
    }

    public void derEncode(OutputStream outputstream)
        throws IOException
    {
        DerOutputStream deroutputstream = new DerOutputStream();
        DerOutputStream deroutputstream1 = new DerOutputStream();
        deroutputstream.putOID(algid);
        if(params == null)
            deroutputstream.putNull();
        else
            deroutputstream.putDerValue(params);
        deroutputstream1.write((byte)48, deroutputstream);
        outputstream.write(deroutputstream1.toByteArray());
    }

    public final byte[] encode()
        throws IOException
    {
        DerOutputStream deroutputstream = new DerOutputStream();
        DerOutputStream deroutputstream1 = new DerOutputStream();
        deroutputstream1.putOID(algid);
        if(params == null)
            deroutputstream1.putNull();
        else
            deroutputstream1.putDerValue(params);
        deroutputstream.write((byte)48, deroutputstream1);
        return deroutputstream.toByteArray();
    }

    public final void encode(DerOutputStream deroutputstream)
        throws IOException
    {
        derEncode(deroutputstream);
    }

    public boolean equals(Object obj)
    {
        if(obj instanceof AlgorithmId)
            return equals((AlgorithmId)obj);
        if(obj instanceof ObjectIdentifier)
            return equals((ObjectIdentifier)obj);
        else
            return false;
    }

    public final boolean equals(ObjectIdentifier objectidentifier)
    {
        return algid.equals(objectidentifier);
    }

    public boolean equals(AlgorithmId algorithmid)
    {
        if(!algid.equals(algorithmid.algid))
            return false;
        if(params == null && algorithmid.params == null)
            return true;
        if(params == null)
            return false;
        else
            return params.equals(algorithmid.params);
    }

    public static AlgorithmId get(String s)
        throws NoSuchAlgorithmException
    {
        ObjectIdentifier objectidentifier = algOID(s);
        if(objectidentifier == null)
            throw new NoSuchAlgorithmException("unrecognized algorithm name: " + s);
        else
            return new AlgorithmId(objectidentifier);
    }

    public static AlgorithmId getAlgorithmId(String s)
        throws NoSuchAlgorithmException
    {
        return get(s);
    }

    public byte[] getEncodedParams()
        throws IOException
    {
        if(params == null)
            return null;
        else
            return params.toByteArray();
    }

    public String getName()
    {
        return algName();
    }

    public final ObjectIdentifier getOID()
    {
        return algid;
    }

    public AlgorithmParameters getParameters()
    {
        return algParams;
    }

    protected String paramsToString()
    {
        if(params == null)
            return "";
        if(algParams != null)
            return algParams.toString();
        else
            return ", params unparsed";
    }

    public static AlgorithmId parse(DerValue dervalue)
        throws IOException
    {
        if(dervalue.tag != 48)
            throw new IOException("algid parse error, not a sequence");
        DerInputStream derinputstream = dervalue.toDerInputStream();
        ObjectIdentifier objectidentifier = derinputstream.getOID();
        DerValue dervalue1;
        if(derinputstream.available() == 0)
        {
            dervalue1 = null;
        }
        else
        {
            dervalue1 = derinputstream.getDerValue();
            if(dervalue1.tag == 5)
                dervalue1 = null;
        }
        AlgorithmId algorithmid = new AlgorithmId(objectidentifier, dervalue1);
        if(dervalue1 != null)
            algorithmid.decodeParams();
        return algorithmid;
    }

    public String toString()
    {
        return algName() + paramsToString();
    }

    private static final long serialVersionUID = 0x640067c6d62263e5L;
    private static boolean initOidTable = false;
    private static Hashtable oidTable;
    private ObjectIdentifier algid;
    private AlgorithmParameters algParams;
    protected DerValue params;
    private static final int MD2_data[] = {
        1, 2, 840, 0x1bb8d, 2, 2
    };
    private static final int MD5_data[] = {
        1, 2, 840, 0x1bb8d, 2, 5
    };
    private static final int SHA1_OIW_data[] = {
        1, 3, 14, 3, 2, 26
    };
    public static final ObjectIdentifier MD2_oid = new ObjectIdentifier(MD2_data);
    public static final ObjectIdentifier MD5_oid = new ObjectIdentifier(MD5_data);
    public static final ObjectIdentifier SHA_oid = new ObjectIdentifier(SHA1_OIW_data);
    private static final int DH_data[] = {
        1, 2, 840, 0x1bb8d, 1, 3, 1
    };
    private static final int DH_PKIX_data[] = {
        1, 2, 840, 10046, 2, 1
    };
    private static final int DSA_OIW_data[] = {
        1, 3, 14, 3, 2, 12
    };
    private static final int DSA_PKIX_data[] = {
        1, 2, 840, 10040, 4, 1
    };
    private static final int RSA_data[] = {
        1, 2, 5, 8, 1, 1
    };
    private static final int RSAEncryption_data[] = {
        1, 2, 840, 0x1bb8d, 1, 1, 1
    };
    public static final ObjectIdentifier DH_oid = new ObjectIdentifier(DH_data);
    public static final ObjectIdentifier DH_PKIX_oid = new ObjectIdentifier(DH_PKIX_data);
    public static final ObjectIdentifier DSA_OIW_oid = new ObjectIdentifier(DSA_OIW_data);
    public static final ObjectIdentifier DSA_oid = new ObjectIdentifier(DSA_PKIX_data);
    public static final ObjectIdentifier RSA_oid = new ObjectIdentifier(RSA_data);
    public static final ObjectIdentifier RSAEncryption_oid = new ObjectIdentifier(RSAEncryption_data);
    private static final int md2WithRSAEncryption_data[] = {
        1, 2, 840, 0x1bb8d, 1, 1, 2
    };
    private static final int md5WithRSAEncryption_data[] = {
        1, 2, 840, 0x1bb8d, 1, 1, 4
    };
    private static final int sha1WithRSAEncryption_data[] = {
        1, 2, 840, 0x1bb8d, 1, 1, 5
    };
    private static final int sha1WithRSAEncryption_OIW_data[] = {
        1, 3, 14, 3, 2, 29
    };
    private static final int shaWithDSA_OIW_data[] = {
        1, 3, 14, 3, 2, 13
    };
    private static final int sha1WithDSA_OIW_data[] = {
        1, 3, 14, 3, 2, 27
    };
    private static final int dsaWithSHA1_PKIX_data[] = {
        1, 2, 840, 10040, 4, 3
    };
    public static final ObjectIdentifier md2WithRSAEncryption_oid = new ObjectIdentifier(md2WithRSAEncryption_data);
    public static final ObjectIdentifier md5WithRSAEncryption_oid = new ObjectIdentifier(md5WithRSAEncryption_data);
    public static final ObjectIdentifier sha1WithRSAEncryption_oid = new ObjectIdentifier(sha1WithRSAEncryption_data);
    public static final ObjectIdentifier sha1WithRSAEncryption_OIW_oid = new ObjectIdentifier(sha1WithRSAEncryption_OIW_data);
    public static final ObjectIdentifier shaWithDSA_OIW_oid = new ObjectIdentifier(shaWithDSA_OIW_data);
    public static final ObjectIdentifier sha1WithDSA_OIW_oid = new ObjectIdentifier(sha1WithDSA_OIW_data);
    public static final ObjectIdentifier sha1WithDSA_oid = new ObjectIdentifier(dsaWithSHA1_PKIX_data);

}
