package digsec.set.core.pkcs;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;


public class ContentInfo implements DerEncoder
{

    public ContentInfo(DerInputStream derinputstream)
        throws IOException, ParsingException
    {
        this(derinputstream, false);
    }

    public ContentInfo(DerInputStream derinputstream, boolean flag)
        throws IOException, ParsingException
    {
        DerValue adervalue[] = derinputstream.getSequence(2);
        DerValue dervalue = adervalue[0];
        DerInputStream derinputstream1 = new DerInputStream(dervalue.toByteArray());
        contentType = derinputstream1.getOID();
        if(flag)
            content = adervalue[1];
        else
        if(adervalue.length > 1)
        {
            DerValue dervalue1 = adervalue[1];
            DerInputStream derinputstream2 = new DerInputStream(dervalue1.toByteArray());
            DerValue adervalue1[] = derinputstream2.getSet(1, true);
            content = adervalue1[0];
        }
    }

    public ContentInfo(ObjectIdentifier objectidentifier, DerValue dervalue)
    {
        contentType = objectidentifier;
        content = dervalue;
    }

    public ContentInfo(byte abyte0[])
    {
        DerValue dervalue = new DerValue((byte)4, abyte0);
        contentType = DATA_OID;
        content = dervalue;
    }

    public void derEncode(OutputStream out) throws IOException {
	  DerOutputStream deroutputstream = new DerOutputStream();	
	  DerOutputStream deroutputstream2 = new DerOutputStream();
        deroutputstream2.putOID(contentType);
        if(content != null)
        {
            DerValue dervalue = null;
            DerOutputStream deroutputstream1 = new DerOutputStream();
            content.encode(deroutputstream1);
            dervalue = new DerValue((byte)-96, deroutputstream1.toByteArray());
            deroutputstream2.putDerValue(dervalue);
        }
        deroutputstream.write((byte)48, deroutputstream2);
	  out.write(deroutputstream.toByteArray());	
    }	

    public void encode(DerOutputStream deroutputstream)
        throws IOException
    {
		derEncode(deroutputstream);
    }

    public DerValue getContent()
    {
        return content;
    }

    public byte[] getContentBytes()
        throws IOException
    {
        if(content == null)
        {
            return null;
        }
        else
        {
            DerInputStream derinputstream = new DerInputStream(content.toByteArray());
            return derinputstream.getOctetString();
        }
    }

    public byte[] getData()
        throws IOException
    {
        if(contentType.equals(DATA_OID) || contentType.equals(OLD_DATA_OID))
        {
            if(content == null)
                return null;
            else
                return content.getOctetString();
        }
        else
        {
            throw new IOException("content type is not DATA: " + contentType);
        }
    }

    public ObjectIdentifier getContentType() {
	  return contentType;
    }		

    public String toString () 
    {
	try {
	  HexDumpEncoder hde = new HexDumpEncoder();		
        String s = "";
        s = s + "Content Info Sequence\nContent type: " + contentType + "\n";
	  if(content!=null)	
        	s = s + "Content: \n" + hde.encodeBuffer(content.toByteArray());
    	  else
		s = s + "Content: <empty>\n";	
	  return s;
	}
	catch (Exception e) {
		e.printStackTrace();
		return null;
	}
    }

    private static int pkcs7[] = {
        1, 2, 840, 0x1bb8d, 1, 7
    };
    private static int data[] = {
        1, 2, 840, 0x1bb8d, 1, 7, 1
    };
    private static int sdata[] = {
        1, 2, 840, 0x1bb8d, 1, 7, 2
    };
    private static int edata[] = {
        1, 2, 840, 0x1bb8d, 1, 7, 3
    };
    private static int sedata[] = {
        1, 2, 840, 0x1bb8d, 1, 7, 4
    };
    private static int ddata[] = {
        1, 2, 840, 0x1bb8d, 1, 7, 5
    };
    private static int crdata[] = {
        1, 2, 840, 0x1bb8d, 1, 7, 6
    };
    private static int nsdata[] = {
        2, 16, 840, 1, 0x1bc42, 2, 5
    };
    public static final ObjectIdentifier PKCS7_OID;
    public static final ObjectIdentifier DATA_OID;
    public static final ObjectIdentifier SIGNED_DATA_OID;
    public static final ObjectIdentifier ENVELOPED_DATA_OID;
    public static final ObjectIdentifier SIGNED_AND_ENVELOPED_DATA_OID;
    public static final ObjectIdentifier DIGESTED_DATA_OID;
    public static final ObjectIdentifier ENCRYPTED_DATA_OID;
    private static final int OLD_SDATA[] = {
        1, 2, 840, 0x10fdcd, 1, 7, 2
    };
    public static final ObjectIdentifier OLD_SIGNED_DATA_OID = new ObjectIdentifier(OLD_SDATA);
    private static final int OLD_DATA[] = {
        1, 2, 840, 0x10fdcd, 1, 7, 1
    };
    public static final ObjectIdentifier OLD_DATA_OID = new ObjectIdentifier(OLD_DATA);
    public static final ObjectIdentifier NETSCAPE_CERT_SEQUENCE_OID;
    ObjectIdentifier contentType;
    DerValue content;

    static 
    {
        PKCS7_OID = new ObjectIdentifier(pkcs7);
        DATA_OID = new ObjectIdentifier(data);
        SIGNED_DATA_OID = new ObjectIdentifier(sdata);
        ENVELOPED_DATA_OID = new ObjectIdentifier(edata);
        SIGNED_AND_ENVELOPED_DATA_OID = new ObjectIdentifier(sedata);
        DIGESTED_DATA_OID = new ObjectIdentifier(ddata);
        ENCRYPTED_DATA_OID = new ObjectIdentifier(crdata);
        NETSCAPE_CERT_SEQUENCE_OID = new ObjectIdentifier(nsdata);
    }
}
