package digsec.set.core.pkcs;

import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.io.*;
import java.util.Arrays;

/**
 *	<pre>
 * 	2807 DigestedData ::= SEQUENCE {
 * 	2808    ddVersion        	INTEGER { ddVer0(0) } (ddVer0),
 * 	2809    digestAlgorithm  	AlgorithmIdentifier {{DigestAlgorithms}},
 *	2810    contentInfo    		ContentInfo,
 *	2811    digest           	Digest
 *	2812 }
 *
 *	2744 Digest ::= OCTET STRING (SIZE(1..20))
 *	</pre>
 */

public class DigestedData implements DerEncoder{

	protected DigestedData() {}

	public DigestedData(DerInputStream din) throws Exception {
		
		ContentInfo temp = new ContentInfo(din);
		DerValue dv = temp.getContent();

		DerInputStream din1 = new DerInputStream(dv.toByteArray());
		DerValue[] adv = din1.getSequence(4);
		
		ddVersion = new BigInt(adv[0].getInteger().toInt());
		digestAlgorithm = AlgorithmId.parse(adv[1]);
		contentInfo = new ContentInfo(new DerInputStream(adv[2].toByteArray()));
		digest = adv[3].getOctetString();		
	}

	public DigestedData(AlgorithmId digestAlg, ContentInfo cInfo, byte[] digest) {
		ddVersion = new BigInt(0);
		digestAlgorithm=digestAlg;
		contentInfo=cInfo;
		this.digest=digest;
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		DerOutputStream dout1 = new DerOutputStream();
		dout1.putInteger(ddVersion);
		digestAlgorithm.encode(dout1);
		contentInfo.encode(dout1);
		dout1.putOctetString(digest);
		
		ContentInfo temp = new ContentInfo(ContentInfo.DIGESTED_DATA_OID,
					 	   	   new DerValue((byte)48,dout1.toByteArray()));
		temp.encode(dout);
		byte[] output = dout.toByteArray();
		out.write(output,0,output.length);

	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}
	
	public BigInt getVersion() {
		return ddVersion;
	}

	public ContentInfo getContentInfo() {
		return contentInfo;
	}

	public byte[] getDigest() {
		return digest;
	}

	public AlgorithmId getDigestAlgorithm() {
		return digestAlgorithm;
	}

	public String toString() {
       		String s = "";
		s += "DigestedData\n";
        		if(ddVersion != null)
            		s = s + "version\t\t: " + ddVersion + "\n";
		if(digestAlgorithm!=null)
			s = s + "digestAlgorithm\t: " + digestAlgorithm + "\n";  
      		if(contentInfo!=null)
			s = s + "contentInfo\t: " + "\n" + contentInfo + "\n";
		if(digest!=null) {
			HexDumpEncoder hde = new HexDumpEncoder();
			s = s + "digest\t\t: \n" + hde.encodeBuffer(digest);			
        		}
		return s;
 	}

	public boolean equals(DigestedData dd) {
		byte[] digest1 = dd.getDigest();
		return Arrays.equals(digest,digest1);
	}

	protected BigInt ddVersion; 
	protected ContentInfo contentInfo;
	protected AlgorithmId digestAlgorithm;
	protected byte[] digest;

	public static void main(String[] args) {
	   try {
		byte[] digest = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
		byte[] content = {0,1,2,3,4,5,6,7,8};
		ContentInfo cInfo = new ContentInfo(ContentInfo.SIGNED_DATA_OID,
				        new DerValue(DerValue.tag_OctetString,content));
		AlgorithmId alg = new AlgorithmId(AlgorithmId.SHA_oid);
		DigestedData dd = new DigestedData(alg,cInfo,digest);
		
		DerOutputStream dout = new DerOutputStream();
		dd.encode(dout);
		HexDumpEncoder hde = new HexDumpEncoder();
		
		System.out.println(hde.encodeBuffer(dout.toByteArray()));

		dd = new DigestedData(new DerInputStream(dout.toByteArray()));
		System.out.println(dd);
	   }
	   catch(Exception e) {
		e.printStackTrace();	
	   }		
	}
}