package digsec.set.core.pkcs;
import java.io.*;
import sun.security.util.*;

import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	2761 EncryptedData ::= SEQUENCE {
 *	2762         version		INTEGER { enVer0(0) } (enVer0),
 *	2763         encryptedContentInfo	EncryptedContentInfo
 *	2764 }
 *	</pre>
 */

public class EncryptedData implements DerEncoder
{
	private BigInt version;
	private EncryptedContentInfo encryptedContentInfo;

	/**
	 * Constructor for EncryptedData
	 *
	 */
	public EncryptedData(EncryptedContentInfo eci) {
		version = new BigInt(0);
		encryptedContentInfo = eci;		
	}

	/**
	 * Constructor for EncryptedData
	 *
	 */
	public EncryptedData(DerInputStream din) throws Exception {
		ContentInfo contentInfo = new ContentInfo(din);
		//ambil content
		DerValue dv = contentInfo.getContent();
		DerInputStream din1 = new DerInputStream(dv.toByteArray());
		DerValue[] adv = din1.getSequence(2);
		//isi version
		version = new BigInt(adv[0].getInteger().toInt());
		if (!version.equals(new BigInt(0)))
			throw new RuntimeException("Different version of EncryptedData");

		//isi encryptedContentInfo
		encryptedContentInfo = new EncryptedContentInfo(new DerInputStream(adv[1].toByteArray()));	
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger(version);
		encryptedContentInfo.encode(dout);
		DerValue dv = new DerValue((byte)48, dout.toByteArray());
		//jadiin ContentInfo
		ContentInfo cInfo = new ContentInfo(ContentInfo.ENCRYPTED_DATA_OID,dv);		
		DerOutputStream dout1 = new DerOutputStream();
		//ContentInfo ditulis ke deroutputstream lalu dijadiin ByteArray
		cInfo.encode(dout1);
		byte[] output = dout1.toByteArray();
		//tulis ke outputstream
		out.write(output,0,output.length);	
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}
	
	public EncryptedContentInfo getEncryptedContentInfo() throws Exception {
		return encryptedContentInfo;
	}

	public BigInt getVersion() throws Exception {
		return version;
	}
	
	public String toString() {
		String s = "Object EncryptedData\n";
        	if(version != null)
			s = s + "EncryptedData :: Version: " + version + "\n";
        	if(encryptedContentInfo != null)
        		s = s + "EncryptedData :: EncryptedContentInfo: \n" + encryptedContentInfo;
	
        return s;
	}

	public static void main(String[] args) {
		try {
			//make ContentInfo
			String strParam = "Peringatan pemerintah, merokok membahayakan keuangan anda, ";
			strParam = strParam + "terutama jika rokoknya adalah rokok-rokok luar yang mahal";
			byte[] param = strParam.getBytes();
                  
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,param));		

			//make EncryptedContentInfo
			int[] oid = {1,3,14,3,2,7};
			ObjectIdentifier id_desCBC = new ObjectIdentifier(oid);
			EncryptedContentInfo enc = 
				new EncryptedContentInfo(ContentInfo.ENCRYPTED_DATA_OID, 
							 	 new AlgorithmId(id_desCBC),
							 	 cInfo.getContentBytes());	

			EncryptedData ed = new EncryptedData(enc);
			System.out.println(ed.toString());
	
			DerOutputStream dout = new DerOutputStream();
			ed.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			EncryptedData ed2 = new EncryptedData(new DerInputStream(dout.toByteArray()));
			System.out.println(ed2);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}