package digsec.set.core.pkcs;

import sun.security.util.*;
import sun.security.x509.X500Name;
import sun.misc.HexDumpEncoder;
import java.io.*;

/**
 * <pre>
 * 	2766 EnvelopedData ::= SEQUENCE {
 * 	2767    edVersion             INTEGER { edVer1(1) } (edVer1),
 *	2768    recipientInfos        RecipientInfos,
 *	2769    encryptedContentInfo  EncryptedContentInfo
 *	2770 }
 * </pre>
 */
public class EnvelopedData implements DerEncoder {

	public EnvelopedData(DerInputStream din) throws Exception {
		ContentInfo contentInfo = new ContentInfo(din);
		DerValue dv = contentInfo.getContent();
	
		DerInputStream din1 = new DerInputStream(dv.toByteArray());
		DerValue[] adv = din1.getSequence(3);

		version = adv[0].getInteger();

		DerInputStream din2 = new DerInputStream(adv[1].toByteArray());
		DerValue[] recInfos = din2.getSequence(1);
		int len = recInfos.length;
		recipientInfos = new RecipientInfo[len];
		for(int i=0; i<len; i++) {
			recipientInfos[i] = new RecipientInfo(recInfos[i].toDerInputStream());
		}
		
		encryptedContentInfo = new EncryptedContentInfo(new DerInputStream(adv[2].toByteArray()));

		
      	}

	public EnvelopedData(RecipientInfo[] rInfos, 
                   			EncryptedContentInfo eCInfo) {
		recipientInfos = rInfos;
		encryptedContentInfo = eCInfo;
	}

	public void derEncode(OutputStream out) throws java.io.IOException {

		DerOutputStream dout = new DerOutputStream();
		HexDumpEncoder hde = new HexDumpEncoder();
		DerOutputStream dout1 = new DerOutputStream();
		dout1.putInteger(version);
		//System.out.println(hde.encodeBuffer(dout1.toByteArray()));
		dout1.putOrderedSetOf((byte)48,recipientInfos);
		//System.out.println(hde.encodeBuffer(dout1.toByteArray()));
		encryptedContentInfo.encode(dout1);
		//System.out.println(hde.encodeBuffer(dout1.toByteArray()));
		DerValue dv = new DerValue((byte)48,dout1.toByteArray());
		//System.out.println(hde.encodeBuffer(dv.toByteArray()));
		ContentInfo cInfo = new ContentInfo(ContentInfo.ENVELOPED_DATA_OID,dv);
		cInfo.encode(dout);
		byte[] output = dout.toByteArray();
		out.write(output,0,output.length);
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}	

	public RecipientInfo[] getRecipientInfos() {
		return recipientInfos;
	}

	public EncryptedContentInfo getEncryptedContentInfo() {
		return encryptedContentInfo;
	}
 
	public String toString()
    	{
        		String s = "";
        		s = s + "EnvelopedData \n";
        		if(version != null)
            		s = s + "EnvelopedData :: version: " + version + "\n";
        		if(recipientInfos != null)
        		{
            		s = s + "EnvelopedData :: recipientInfos: \n";
            		for(int i = 0; i < recipientInfos.length; i++)
                		s = s + "\t" + recipientInfos[i] + "\n";
        		}
		if(encryptedContentInfo!=null)
			s = s + "EnvelopedData :: encryptedContentInfo: \n" + 
		    		encryptedContentInfo;
			
        		return s;
    	}


	private RecipientInfo[] recipientInfos;
	private EncryptedContentInfo encryptedContentInfo;
	private BigInt version = new BigInt(1);	

	public static void main (String[] args) {
	   try {
		X500Name name = new X500Name("Arif", "DIGSEC", "Fasilkom", "Indonesia");
		X500Name name1 = new X500Name("Iduy", "DIGSEC", "Fasilkom", "Indonesia");
		AlgorithmId alg = new AlgorithmId(AlgorithmId.RSAEncryption_oid);
		BigInt serial = new BigInt(1234567890);
		RecipientInfo[] recInfos = new RecipientInfo[2];
		byte[] encrKey = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};

		recInfos[0] = new RecipientInfo(name,serial,alg,encrKey);
		recInfos[1] = new RecipientInfo(name1,serial,alg,encrKey);

		byte[] encContent = {0,1,2,3,4,5,6,7,8,9};
		int[] oid = {1,3,14,3,2,7};
		ObjectIdentifier id_desCBC = new ObjectIdentifier(oid);	
		EncryptedContentInfo enc = 
			new EncryptedContentInfo(ContentInfo.ENCRYPTED_DATA_OID, 
						 	 new AlgorithmId(id_desCBC),
						 	 encContent);	

		EnvelopedData env = new EnvelopedData(recInfos,enc);
		DerOutputStream dout = new DerOutputStream();
		env.encode(dout);
		HexDumpEncoder hde = new HexDumpEncoder();
		System.out.println(hde.encodeBuffer(dout.toByteArray()));

		env = new EnvelopedData(new DerInputStream(dout.toByteArray()));
		System.out.println(env.toString());
	   }
	   catch(Exception e) {
	   	e.printStackTrace();
	   }	
	}
}