package digsec.set.core.pkcs;

import java.io.*;
import java.security.cert.CertificateException;
import java.util.Date;
import java.util.Hashtable;
import sun.misc.CharacterEncoder;
import sun.misc.HexDumpEncoder;
import sun.security.util.*;
import sun.security.x509.*;

// Referenced classes of package sun.security.pkcs:
//            SignerInfo

public class PKCS9Attribute
    implements DerEncoder
{

    public PKCS9Attribute(String s, Object obj)
        throws IllegalArgumentException
    {
        ObjectIdentifier objectidentifier = getOID(s);
        if(objectidentifier == null)
        {
            throw new IllegalArgumentException("Unrecognized attribute name " + s + " constructing PKCS9Attribute.");
        }
        else
        {
            init(objectidentifier, obj);
            return;
        }
    }

    public PKCS9Attribute(DerValue dervalue)
        throws IOException
    {
        DerInputStream derinputstream = new DerInputStream(dervalue.toByteArray());
        DerValue adervalue[] = derinputstream.getSequence(2);
        if(derinputstream.available() != 0)
            throw new IOException("Excess data parsing PKCS9Attribute");
        if(adervalue.length != 2)
            throw new IOException("PKCS9Attribute doesn't have two components");
        ObjectIdentifier objectidentifier = adervalue[0].getOID();
        index = indexOf(objectidentifier, PKCS9_OIDS, 1);
        if(index == -1)
            throw new IOException("Invalid OID for PKCS9 attribute: " + objectidentifier);
        DerValue adervalue1[] = (new DerInputStream(adervalue[1].toByteArray())).getSet(1);
        if(SINGLE_VALUED[index] && adervalue1.length > 1)
            throwSingleValuedException();
        for(int i = 0; i < adervalue1.length; i++)
        {
            Byte byte1 = new Byte(adervalue1[i].tag);
            if(indexOf(byte1, PKCS9_VALUE_TAGS[index], 0) == -1)
                throwTagException(byte1);
        }

        switch(index)
        {
        default:
            break;

        case 1: /* '\001' */
        case 2: /* '\002' */
        case 8: /* '\b' */
            String as[] = new String[adervalue1.length];
            for(int j = 0; j < adervalue1.length; j++)
                as[j] = adervalue1[j].getAsString();

            value = as;
            break;

        case 3: /* '\003' */
            value = adervalue1[0].getOID();
            break;

        case 4: /* '\004' */
            value = adervalue1[0].getOctetString();
            break;

        case 5: /* '\005' */
            value = (new DerInputStream(adervalue1[0].toByteArray())).getUTCTime();
            break;

        case 6: /* '\006' */
            SignerInfo asignerinfo[] = new SignerInfo[adervalue1.length];
            for(int k = 0; k < adervalue1.length; k++)
                asignerinfo[k] = new SignerInfo(adervalue1[k].toDerInputStream());

            value = asignerinfo;
            break;

        case 7: /* '\007' */
            value = adervalue1[0].getAsString();
            break;

        case 9: /* '\t' */
            throw new IOException("PKCS9 extended-certificate attribute not supported.");

        case 10: /* '\n' */
            throw new IOException("PKCS9 IssuerAndSerialNumberattribute not supported.");

        case 11: /* '\013' */
        case 12: /* '\f' */
            throw new IOException("PKCS9 RSA DSI attributes11 and 12, not supported.");

        case 13: /* '\r' */
            throw new IOException("PKCS9 attribute #13 not supported.");

        case 14: /* '\016' */
            value = new CertificateExtensions(new DerInputStream(adervalue1[0].toByteArray()));
            break;

        case 15: /* '\017' */
            throw new IOException("PKCS9 SMIMECapability attribute not supported.");

        }
    }

    public PKCS9Attribute(ObjectIdentifier objectidentifier, Object obj)
        throws IllegalArgumentException
    {
        init(objectidentifier, obj);
    }

    public void derEncode(OutputStream outputstream)
        throws IOException
    {
        DerOutputStream deroutputstream = new DerOutputStream();
        deroutputstream.putOID(getOID());
        switch(index)
        {
        default:
            break;

        case 1: /* '\001' */
        case 2: /* '\002' */
            String as[] = (String[])value;
            DerOutputStream aderoutputstream[] = new DerOutputStream[as.length];
            for(int i = 0; i < as.length; i++)
            {
                aderoutputstream[i] = new DerOutputStream();
                aderoutputstream[i].putIA5String(as[i]);
            }

            deroutputstream.putOrderedSetOf((byte)49, aderoutputstream);
            break;

        case 3: /* '\003' */
            DerOutputStream deroutputstream1 = new DerOutputStream();
            deroutputstream1.putOID((ObjectIdentifier)value);
            deroutputstream.write((byte)49, deroutputstream1.toByteArray());
            break;

        case 4: /* '\004' */
            DerOutputStream deroutputstream2 = new DerOutputStream();
            deroutputstream2.putOctetString((byte[])value);
            deroutputstream.write((byte)49, deroutputstream2.toByteArray());
            break;

        case 5: /* '\005' */
            DerOutputStream deroutputstream3 = new DerOutputStream();
            deroutputstream3.putUTCTime((Date)value);
            deroutputstream.write((byte)49, deroutputstream3.toByteArray());
            break;

        case 6: /* '\006' */
            deroutputstream.putOrderedSetOf((byte)49, (DerEncoder[])value);
            break;

        case 7: /* '\007' */
            DerOutputStream deroutputstream4 = new DerOutputStream();
            deroutputstream4.putPrintableString((String)value);
            deroutputstream.write((byte)49, deroutputstream4.toByteArray());
            break;

        case 8: /* '\b' */
            String as1[] = (String[])value;
            DerOutputStream aderoutputstream1[] = new DerOutputStream[as1.length];
            for(int j = 0; j < as1.length; j++)
            {
                aderoutputstream1[j] = new DerOutputStream();
                aderoutputstream1[j].putPrintableString(as1[j]);
            }

            deroutputstream.putOrderedSetOf((byte)49, aderoutputstream1);
            break;

        case 9: /* '\t' */
            throw new IOException("PKCS9 extended-certificate attribute not supported.");

        case 10: /* '\n' */
            throw new IOException("PKCS9 IssuerAndSerialNumberattribute not supported.");

        case 11: /* '\013' */
        case 12: /* '\f' */
            throw new IOException("PKCS9 RSA DSI attributes11 and 12, not supported.");

        case 13: /* '\r' */
            throw new IOException("PKCS9 attribute #13 not supported.");

        case 14: /* '\016' */
            DerOutputStream deroutputstream5 = new DerOutputStream();
            CertificateExtensions certificateextensions = (CertificateExtensions)value;
            try
            {
                certificateextensions.encode(deroutputstream5, true);
            }
            catch(CertificateException certificateexception)
            {
                throw new IOException(certificateexception.toString());
            }
            deroutputstream.write((byte)49, deroutputstream5.toByteArray());
            break;

        case 15: /* '\017' */
            throw new IOException("PKCS9 attribute #15 not supported.");

        }
        DerOutputStream deroutputstream6 = new DerOutputStream();
        deroutputstream6.write((byte)48, deroutputstream.toByteArray());
        outputstream.write(deroutputstream6.toByteArray());
    }

    public String getName()
    {
        return (String)OID_NAME_TABLE.get(PKCS9_OIDS[index]);
    }

    public static String getName(ObjectIdentifier objectidentifier)
    {
        return (String)OID_NAME_TABLE.get(objectidentifier);
    }

    public ObjectIdentifier getOID()
    {
        return PKCS9_OIDS[index];
    }

    public static ObjectIdentifier getOID(String s)
    {
        return (ObjectIdentifier)NAME_OID_TABLE.get(s.toLowerCase());
    }

    public Object getValue()
    {
        return value;
    }

    static int indexOf(Object obj, Object aobj[], int i)
    {
        for(int j = i; j < aobj.length; j++)
            if(obj.equals(aobj[j]))
                return j;

        return -1;
    }

    private void init(ObjectIdentifier objectidentifier, Object obj)
        throws IllegalArgumentException
    {
        index = indexOf(objectidentifier, PKCS9_OIDS, 1);
        if(index == -1)
            throw new IllegalArgumentException("Unsupported OID " + objectidentifier + " constructing PKCS9Attribute.");
        if(!VALUE_CLASSES[index].isInstance(obj))
        {
            throw new IllegalArgumentException("Wrong value class  for attribute " + objectidentifier + " constructing PKCS9Attribute; was " + obj.getClass().toString() + ", should be " + VALUE_CLASSES[index].toString());
        }
        else
        {
            value = obj;
            return;
        }
    }

    public boolean isSingleValued()
    {
        return SINGLE_VALUED[index];
    }

    private void throwSingleValuedException()
        throws IOException
    {
        throw new IOException("Single-value attribute " + getOID() + " (" + getName() + ")" + " has multiple values.");
    }

    private void throwTagException(Byte byte1)
        throws IOException
    {
        Byte abyte[] = PKCS9_VALUE_TAGS[index];
        StringBuffer stringbuffer = new StringBuffer(100);
        stringbuffer.append("Value of attribute ");
        stringbuffer.append(getOID().toString());
        stringbuffer.append(" (");
        stringbuffer.append(getName());
        stringbuffer.append(") has wrong tag: ");
        stringbuffer.append(byte1.toString());
        stringbuffer.append(".  Expected tags: ");
        stringbuffer.append(abyte[0].toString());
        for(int i = 1; i < abyte.length; i++)
        {
            stringbuffer.append(", ");
            stringbuffer.append(abyte[i].toString());
        }

        stringbuffer.append(".");
        throw new IOException(stringbuffer.toString());
    }

    public String toString()
    {
        StringBuffer stringbuffer = new StringBuffer(100);
        stringbuffer.append("[");
        stringbuffer.append(OID_NAME_TABLE.get(PKCS9_OIDS[index]));
        stringbuffer.append(": ");
        if(SINGLE_VALUED[index])
        {
            if(value instanceof byte[])
            {
                HexDumpEncoder hexdumpencoder = new HexDumpEncoder();
                stringbuffer.append(hexdumpencoder.encodeBuffer((byte[])value));
            }
            else
            {
                stringbuffer.append(value.toString());
            }
            stringbuffer.append("]");
            return stringbuffer.toString();
        }
        boolean flag = true;
        Object aobj[] = (Object[])value;
        for(int i = 0; i < aobj.length; i++)
        {
            if(flag)
                flag = false;
            else
                stringbuffer.append(", ");
            stringbuffer.append(aobj[i].toString());
        }

        return stringbuffer.toString();
    }

    private static final String RSADSI_str = "1.2.840.113549";
    private static final String PKCS_str = "1.2.840.113549.1";
    private static final String PKCS9_str = "1.2.840.113549.1.9";
    static final ObjectIdentifier PKCS9_OIDS[];
    public static final ObjectIdentifier EMAIL_ADDRESS_OID;
    public static final ObjectIdentifier UNSTRUCTURED_NAME_OID;
    public static final ObjectIdentifier CONTENT_TYPE_OID;
    public static final ObjectIdentifier MESSAGE_DIGEST_OID;
    public static final ObjectIdentifier SIGNING_TIME_OID;
    public static final ObjectIdentifier COUNTERSIGNATURE_OID;
    public static final ObjectIdentifier CHALLENGE_PASSWORD_OID;
    public static final ObjectIdentifier UNSTRUCTURED_ADDRESS_OID;
    public static final ObjectIdentifier EXTENDED_CERTIFICATE_ATTRIBUTES_OID;
    public static final ObjectIdentifier ISSUER_SERIALNUMBER_OID;
    public static final ObjectIdentifier EXTENSION_REQUEST_OID;
    public static final ObjectIdentifier SMIME_CAPABILITY_OID;
    public static final String EMAIL_ADDRESS_STR = "EmailAddress";
    public static final String UNSTRUCTURED_NAME_STR = "UnstructuredName";
    public static final String CONTENT_TYPE_STR = "ContentType";
    public static final String MESSAGE_DIGEST_STR = "MessageDigest";
    public static final String SIGNING_TIME_STR = "SigningTime";
    public static final String COUNTERSIGNATURE_STR = "Countersignature";
    public static final String CHALLENGE_PASSWORD_STR = "ChallengePassword";
    public static final String UNSTRUCTURED_ADDRESS_STR = "UnstructuredAddress";
    public static final String EXTENDED_CERTIFICATE_ATTRIBUTES_STR = "ExtendedCertificateAttributes";
    public static final String ISSUER_SERIALNUMBER_STR = "IssuerAndSerialNumber";
    private static final String RSA_PROPRIETARY_STR = "RSAProprietary";
    private static final String SMIME_SIGNING_DESC_STR = "SMIMESigningDesc";
    public static final String EXTENSION_REQUEST_STR = "ExtensionRequest";
    public static final String SMIME_CAPABILITY_STR = "SMIMECapability";
    private static final Hashtable NAME_OID_TABLE;
    private static final Hashtable OID_NAME_TABLE;
    private static final Byte PKCS9_VALUE_TAGS[][] = {
        null, {
        new Byte((byte)22)
    }, {
        new Byte((byte)22)
    }, {
        new Byte((byte)6)
    }, {
        new Byte((byte)4)
    }, {
        new Byte((byte)23)
    }, {
        new Byte((byte)48)
    }, {
        new Byte((byte)19), new Byte((byte)20)
    }, {
        new Byte((byte)19), new Byte((byte)20)
    }, {
        new Byte((byte)49)
    }, 
        {
        new Byte((byte)48)
    }, null, null, null, {
        new Byte((byte)48)
    }, {
        new Byte((byte)48)
    }
    };
    private static final Class VALUE_CLASSES[];
    private static final boolean SINGLE_VALUED[] = {
        false, false, false, true, true, true, false, true, false, false, 
        true, false, false, false, true, true
    };
    private int index;
    private Object value;

    static 
    {
        PKCS9_OIDS = new ObjectIdentifier[16];
        for(int i = 1; i < PKCS9_OIDS.length; i++)
            PKCS9_OIDS[i] = new ObjectIdentifier("1.2.840.113549.1.9." + i);

        EMAIL_ADDRESS_OID = PKCS9_OIDS[1];
        UNSTRUCTURED_NAME_OID = PKCS9_OIDS[2];
        CONTENT_TYPE_OID = PKCS9_OIDS[3];
        MESSAGE_DIGEST_OID = PKCS9_OIDS[4];
        SIGNING_TIME_OID = PKCS9_OIDS[5];
        COUNTERSIGNATURE_OID = PKCS9_OIDS[6];
        CHALLENGE_PASSWORD_OID = PKCS9_OIDS[7];
        UNSTRUCTURED_ADDRESS_OID = PKCS9_OIDS[8];
        EXTENDED_CERTIFICATE_ATTRIBUTES_OID = PKCS9_OIDS[9];
        ISSUER_SERIALNUMBER_OID = PKCS9_OIDS[10];
        EXTENSION_REQUEST_OID = PKCS9_OIDS[14];
        SMIME_CAPABILITY_OID = PKCS9_OIDS[15];
        NAME_OID_TABLE = new Hashtable(16);
        NAME_OID_TABLE.put("emailaddress", PKCS9_OIDS[1]);
        NAME_OID_TABLE.put("unstructuredname", PKCS9_OIDS[2]);
        NAME_OID_TABLE.put("contenttype", PKCS9_OIDS[3]);
        NAME_OID_TABLE.put("messagedigest", PKCS9_OIDS[4]);
        NAME_OID_TABLE.put("signingtime", PKCS9_OIDS[5]);
        NAME_OID_TABLE.put("countersignature", PKCS9_OIDS[6]);
        NAME_OID_TABLE.put("challengepassword", PKCS9_OIDS[7]);
        NAME_OID_TABLE.put("unstructuredaddress", PKCS9_OIDS[8]);
        NAME_OID_TABLE.put("extendedcertificateattributes", PKCS9_OIDS[9]);
        NAME_OID_TABLE.put("issuerandserialnumber", PKCS9_OIDS[10]);
        NAME_OID_TABLE.put("rsaproprietary", PKCS9_OIDS[11]);
        NAME_OID_TABLE.put("rsaproprietary", PKCS9_OIDS[12]);
        NAME_OID_TABLE.put("signingdescription", PKCS9_OIDS[13]);
        NAME_OID_TABLE.put("extensionrequest", PKCS9_OIDS[14]);
        NAME_OID_TABLE.put("smimecapability", PKCS9_OIDS[15]);
        OID_NAME_TABLE = new Hashtable(16);
        OID_NAME_TABLE.put(PKCS9_OIDS[1], "EmailAddress");
        OID_NAME_TABLE.put(PKCS9_OIDS[2], "UnstructuredName");
        OID_NAME_TABLE.put(PKCS9_OIDS[3], "ContentType");
        OID_NAME_TABLE.put(PKCS9_OIDS[4], "MessageDigest");
        OID_NAME_TABLE.put(PKCS9_OIDS[5], "SigningTime");
        OID_NAME_TABLE.put(PKCS9_OIDS[6], "Countersignature");
        OID_NAME_TABLE.put(PKCS9_OIDS[7], "ChallengePassword");
        OID_NAME_TABLE.put(PKCS9_OIDS[8], "UnstructuredAddress");
        OID_NAME_TABLE.put(PKCS9_OIDS[9], "ExtendedCertificateAttributes");
        OID_NAME_TABLE.put(PKCS9_OIDS[10], "IssuerAndSerialNumber");
        OID_NAME_TABLE.put(PKCS9_OIDS[11], "RSAProprietary");
        OID_NAME_TABLE.put(PKCS9_OIDS[12], "RSAProprietary");
        OID_NAME_TABLE.put(PKCS9_OIDS[13], "SMIMESigningDesc");
        OID_NAME_TABLE.put(PKCS9_OIDS[14], "ExtensionRequest");
        OID_NAME_TABLE.put(PKCS9_OIDS[15], "SMIMECapability");
        VALUE_CLASSES = new Class[16];
        try
        {
            Class class1 = Class.forName("[Ljava.lang.String;");
            VALUE_CLASSES[0] = null;
            VALUE_CLASSES[1] = class1;
            VALUE_CLASSES[2] = class1;
            VALUE_CLASSES[3] = Class.forName("sun.security.util.ObjectIdentifier");
            VALUE_CLASSES[4] = Class.forName("[B");
            VALUE_CLASSES[5] = Class.forName("java.util.Date");
            VALUE_CLASSES[6] = Class.forName("[Lsun.security.pkcs.SignerInfo;");
            VALUE_CLASSES[7] = Class.forName("java.lang.String");
            VALUE_CLASSES[8] = class1;
            VALUE_CLASSES[9] = null;
            VALUE_CLASSES[10] = null;
            VALUE_CLASSES[11] = null;
            VALUE_CLASSES[12] = null;
            VALUE_CLASSES[13] = null;
            VALUE_CLASSES[14] = Class.forName("sun.security.x509.CertificateExtensions");
            VALUE_CLASSES[15] = null;
        }
        catch(ClassNotFoundException classnotfoundexception)
        {
            throw new ExceptionInInitializerError(classnotfoundexception.toString());
        }
    }
}
