package digsec.set.core.pkcs;

import java.io.*;
import java.security.*;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import sun.misc.CharacterEncoder;
import sun.misc.HexDumpEncoder;
import sun.security.util.*;
import sun.security.x509.*;


// Referenced classes of package sun.security.pkcs:
//            ContentInfo, PKCS7, PKCS9Attribute, PKCS9Attributes, 
//            ParsingException

public class SignerInfo
    implements DerEncoder
{

    public SignerInfo(DerInputStream derinputstream)
        throws IOException, ParsingException
    {
        this(derinputstream, false);
    }

    public SignerInfo(DerInputStream derinputstream, boolean flag)
        throws IOException, ParsingException
    {
        version = derinputstream.getInteger();
        DerValue adervalue[] = derinputstream.getSequence(2);
        byte abyte0[] = adervalue[0].toByteArray();
        issuerName = new X500Name(new DerValue((byte)48, abyte0));
        certificateSerialNumber = adervalue[1].getInteger();
        DerValue dervalue = derinputstream.getDerValue();
        digestAlgorithmId = AlgorithmId.parse(dervalue);
        if(flag)
            derinputstream.getSet(0);
        else
        if((byte)derinputstream.peekByte() == -96)
            authenticatedAttributes = new PKCS9Attributes(derinputstream);
        dervalue = derinputstream.getDerValue();
        digestEncryptionAlgorithmId = AlgorithmId.parse(dervalue);
        encryptedDigest = derinputstream.getOctetString();
        if(flag)
            derinputstream.getSet(0);
        else
        if(derinputstream.available() != 0 && (byte)derinputstream.peekByte() == -95)
            unauthenticatedAttributes = new PKCS9Attributes(derinputstream);
        if(derinputstream.available() != 0)
            throw new ParsingException("extra data at the end");
        else
            return;
    }

    public SignerInfo(X500Name x500name, BigInt bigint, AlgorithmId algorithmid, PKCS9Attributes pkcs9attributes, AlgorithmId algorithmid1, byte abyte0[], PKCS9Attributes pkcs9attributes1)
    {
        version = new BigInt(1);
        issuerName = x500name;
        certificateSerialNumber = bigint;
        digestAlgorithmId = algorithmid;
        authenticatedAttributes = pkcs9attributes;
        digestEncryptionAlgorithmId = algorithmid1;
        encryptedDigest = abyte0;
        unauthenticatedAttributes = pkcs9attributes1;
    }

    public SignerInfo(X500Name x500name, BigInt bigint, AlgorithmId algorithmid, AlgorithmId algorithmid1, byte abyte0[])
    {
        version = new BigInt(1);
        issuerName = x500name;
        certificateSerialNumber = bigint;
        digestAlgorithmId = algorithmid;
        digestEncryptionAlgorithmId = algorithmid1;
        encryptedDigest = abyte0;
    }

    public void derEncode(OutputStream outputstream)
        throws IOException
    {
        DerOutputStream deroutputstream = new DerOutputStream();
        deroutputstream.putInteger(version);
        DerOutputStream deroutputstream1 = new DerOutputStream();
        issuerName.encode(deroutputstream1);
        deroutputstream1.putInteger(certificateSerialNumber);
        deroutputstream.write((byte)48, deroutputstream1);
        digestAlgorithmId.encode(deroutputstream);
        if(authenticatedAttributes != null)
            authenticatedAttributes.encode((byte)-96, deroutputstream);
        digestEncryptionAlgorithmId.encode(deroutputstream);
        deroutputstream.putOctetString(encryptedDigest);
        if(unauthenticatedAttributes != null)
            unauthenticatedAttributes.encode((byte)-95, deroutputstream);
        DerOutputStream deroutputstream2 = new DerOutputStream();
        deroutputstream2.write((byte)48, deroutputstream);
        outputstream.write(deroutputstream2.toByteArray());
    }

    public void encode(DerOutputStream deroutputstream)
        throws IOException
    {
        derEncode(deroutputstream);
    }

    public PKCS9Attributes getAuthenticatedAttributes()
    {
        return authenticatedAttributes;
    }

    public X509Certificate getCertificate(PKCS7 pkcs7)
        throws IOException
    {
        return pkcs7.getCertificate(certificateSerialNumber, issuerName);
    }

    public ArrayList getCertificateChain(PKCS7 pkcs7)
        throws IOException
    {
        X509Certificate x509certificate = pkcs7.getCertificate(certificateSerialNumber, issuerName);
        if(x509certificate == null)
            return null;
        ArrayList arraylist = new ArrayList();
        arraylist.add(x509certificate);
        X509Certificate ax509certificate[] = pkcs7.getCertificates();
        if(ax509certificate == null || x509certificate.getSubjectDN().equals(x509certificate.getIssuerDN()))
            return arraylist;
        Principal principal = x509certificate.getIssuerDN();
        int i = 0;
        boolean flag;
        do
        {
            flag = false;
            for(int j = i; j < ax509certificate.length; j++)
            {
                if(!principal.equals(ax509certificate[j].getSubjectDN()))
                    continue;
                arraylist.add(ax509certificate[j]);
                if(ax509certificate[j].getSubjectDN().equals(ax509certificate[j].getIssuerDN()))
                {
                    i = ax509certificate.length;
                }
                else
                {
                    principal = ax509certificate[j].getIssuerDN();
                    X509Certificate x509certificate1 = ax509certificate[i];
                    ax509certificate[i] = ax509certificate[j];
                    ax509certificate[j] = x509certificate1;
                    i++;
                }
                flag = true;
                break;
            }

        }
        while(flag);
        return arraylist;
    }

    public BigInt getCertificateSerialNumber()
    {
        return certificateSerialNumber;
    }

    public AlgorithmId getDigestAlgorithmId()
    {
        return digestAlgorithmId;
    }

    public AlgorithmId getDigestEncryptionAlgorithmId()
    {
        return digestEncryptionAlgorithmId;
    }

    public byte[] getEncryptedDigest()
    {
        return encryptedDigest;
    }

    public X500Name getIssuerName()
    {
        return issuerName;
    }

    public PKCS9Attributes getUnauthenticatedAttributes()
    {
        return unauthenticatedAttributes;
    }

    public BigInt getVersion()
    {
        return version;
    }

    public String toString()
    {
        HexDumpEncoder hexdumpencoder = new HexDumpEncoder();
        String s = "";
        s = s + "Signer Info for (issuer): " + issuerName + "\n";
        s = s + "\tversion: " + version + "\n";
        s = s + "\tcertificateSerialNumber: " + certificateSerialNumber + "\n";
        s = s + "\tdigestAlgorithmId: " + digestAlgorithmId + "\n";
        if(authenticatedAttributes != null)
            s = s + "\tauthenticatedAttributes: " + authenticatedAttributes + "\n";
        s = s + "\tdigestEncryptionAlgorithmId: " + digestEncryptionAlgorithmId + "\n";
        s = s + "\tencryptedDigest: \n" + hexdumpencoder.encodeBuffer(encryptedDigest) + "\n";
        if(unauthenticatedAttributes != null)
            s = s + "\tunauthenticatedAttributes: " + unauthenticatedAttributes + "\n";
        return s;
    }

    SignerInfo verify(PKCS7 pkcs7)
        throws NoSuchAlgorithmException, SignatureException
    {
        return verify(pkcs7, null);
    }

    SignerInfo verify(PKCS7 pkcs7, byte abyte0[])
        throws NoSuchAlgorithmException, SignatureException
    {
        try
        {
            ContentInfo contentinfo = pkcs7.getContentInfo();
            if(abyte0 == null)
                abyte0 = contentinfo.getContentBytes();
            String s = getDigestAlgorithmId().getName();
            if(s.equalsIgnoreCase("SHA"))
                s = "SHA1";
            byte abyte1[];
            if(authenticatedAttributes == null)
            {
                abyte1 = abyte0;
            }
            else
            {
                ObjectIdentifier objectidentifier = (ObjectIdentifier)authenticatedAttributes.getAttributeValue(PKCS9Attribute.CONTENT_TYPE_OID);
                if(objectidentifier == null || !objectidentifier.equals(contentinfo.contentType))
                    return null;
                byte abyte2[] = (byte[])authenticatedAttributes.getAttributeValue(PKCS9Attribute.MESSAGE_DIGEST_OID);
                if(abyte2 == null)
                    return null;
                MessageDigest messagedigest = MessageDigest.getInstance(s);
                byte abyte3[] = messagedigest.digest(abyte0);
                if(abyte2.length != abyte3.length)
                    return null;
                for(int i = 0; i < abyte2.length; i++)
                    if(abyte2[i] != abyte3[i])
                        return null;

                abyte1 = authenticatedAttributes.getDerEncoding();
            }
            String s1 = getDigestEncryptionAlgorithmId().getName();
            if(s1.equalsIgnoreCase("SHA1withDSA"))
                s1 = "DSA";
            String s2 = s + "with" + s1;
            Signature signature = Signature.getInstance(s2);
            X509Certificate x509certificate = getCertificate(pkcs7);
            if(x509certificate == null)
                return null;
            if(x509certificate.hasUnsupportedCriticalExtension())
                throw new SignatureException("Certificate has unsupported critical extension(s)");
            byte abyte4[] = x509certificate.getExtensionValue(PKIXExtensions.KeyUsage_Id.toString());
            if(abyte4 != null)
            {
                KeyUsageExtension keyusageextension;
                try
                {
                    keyusageextension = new KeyUsageExtension(new Boolean(true), abyte4);
                }
                catch(IOException _ex)
                {
                    throw new SignatureException("Failed to parse keyUsage extension");
                }
                boolean flag = ((Boolean)keyusageextension.get("digital_signature")).booleanValue();
                if(!flag)
                    throw new SignatureException("Key usage restricted: cannot be used for digital signatures");
            }
            java.security.PublicKey publickey = x509certificate.getPublicKey();
            signature.initVerify(publickey);
            signature.update(abyte1);
            if(signature.verify(encryptedDigest))
                return this;
        }
        catch(IOException ioexception)
        {
            throw new SignatureException("IO error verifying signature:\n" + ioexception.getMessage());
        }
        catch(InvalidKeyException invalidkeyexception)
        {
            throw new SignatureException("InvalidKey: " + invalidkeyexception.getMessage());
        }
        return null;
    }

    BigInt version;
    X500Name issuerName;
    BigInt certificateSerialNumber;
    AlgorithmId digestAlgorithmId;
    AlgorithmId digestEncryptionAlgorithmId;
    byte encryptedDigest[];
    PKCS9Attributes authenticatedAttributes;
    PKCS9Attributes unauthenticatedAttributes;
}
