package digsec.set.payment.component;
import digsec.set.core.pkcs.*;
import digsec.set.core.crypto.OAEPEncoder;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.io.*;
import java.util.StringTokenizer;

/**
 *	<pre>
 *	397 AcctData ::= SEQUENCE {
 *	398    acctIdentification  AcctIdentification,
 *	399    exNonce             Nonce
 *	400 }
 *
 *	402 AcctIdentification ::= VisibleString (SIZE(ub-acctIdentification))
 *
 *	296 Nonce ::= OCTET STRING (SIZE(20))
 *
 *	</pre>
 */

public class AcctData implements DerEncoder, OAEPEncoder {

	protected String acctIdentification; // size(74)
	protected byte[] exNonce; // size(20)	

	public static ObjectIdentifier id_set_content = new ObjectIdentifier("2.23.42.0");

	public AcctData(String acctIdentification, byte[] nonce) throws Exception{
		if(acctIdentification.length()!=74 || nonce.length != 20)
			throw new RuntimeException("Invalid input of AcctData");
		this.acctIdentification=acctIdentification;
		exNonce = nonce;
	}
	
	public AcctData(DerInputStream din) throws Exception {
		ContentInfo cInfo = new ContentInfo(din);
		DerInputStream din1 = new DerInputStream(cInfo.getContent().toByteArray());

		DerValue[] adv = din1.getSequence(2);
		acctIdentification = adv[0].getPrintableString();
		exNonce = adv[1].getOctetString();
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		DerValue[] adv = new DerValue[2];
		adv[0] = new DerValue(acctIdentification); //18 is tag for NumericString 
		adv[1] = new DerValue((byte)4,exNonce); 	
		dout.putSequence(adv);
		ContentInfo cInfo = new ContentInfo(id_set_content, new DerValue(dout.toByteArray()));
		dout.reset();

		cInfo.encode(dout);

		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		
		derEncode(dout);
	}	

	public byte[] encodeOAEP () throws IOException{
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		bout.write(acctIdentification.getBytes());				
		bout.write(exNonce);		
		return bout.toByteArray();
	}

	public static AcctData parseOAEP(byte[] oaep) throws Exception {
		ByteArrayInputStream bin = new ByteArrayInputStream(oaep);
		byte[] b_acctIdentification = new byte[74];
		bin.read(b_acctIdentification,0,74);
			
		byte[] nonce = new byte[20];
		bin.read(nonce,0,20);
		return new AcctData(new String(b_acctIdentification), nonce);
	}

	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();
		String s = "";
		s += "AcctData\n";
		s += "acctIdentification: " + acctIdentification + "\n";
		s += "exNonce: " + "\n" + hde.encodeBuffer(exNonce) + "\n";
		return s;
	}

	public String getAcctId() {
		return acctIdentification;
	}

	public byte[] getNonce() {
		return exNonce;
	}

	public static void main (String[] args) {
		try { 
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			AcctData acct = new AcctData("1234567890" +
						 		"1234567890" +
								"1234567890" +
								"1234567890" +
								"1234567890" +
								"1234567890" +
								"1234567890" +
								"1234"	, ab);
			DerOutputStream dout = new DerOutputStream();
			acct.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
			acct = new AcctData(new DerInputStream(dout.toByteArray()));
			System.out.println(acct);
			byte[] temp = acct.encodeOAEP();
			System.out.println(hde.encodeBuffer(temp));
			acct = AcctData.parseOAEP(temp);
			System.out.println(acct);
			dout.reset();
			acct.encode(dout);
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}