package digsec.set.payment.component;
import sun.security.util.*;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import digsec.set.core.pkcs.*;
import digsec.set.core.encapsulation.*;
import digsec.set.core.crypto.*;
import java.util.*;

/**
 *	<pre>
 *	1787 AuthToken  ::= EncX { P1, P2, AuthTokenData, PANToken }
 *	</pre>
 */

public class AuthToken implements DerEncoder {

	//fields
	public AuthTokenData 	authTokenData;
	public PANToken		panToken;
	private EncX 		encX;

	//Constructor
	public AuthToken(	PrivateKey 		p1_priv,
				X509Certificate	p1_cert,
				X509Certificate	p2_cert,
				AuthTokenData 	authTokenData,
			     	PANToken	 	panToken) throws Exception {

		this.authTokenData = authTokenData;
		this.panToken	 = panToken;

		DerOutputStream dout = new DerOutputStream();
		authTokenData.encode(dout);
		ContentInfo cInfo	 = new ContentInfo(AuthTokenTBE.id_set_content_AuthTokenTBE,
								 new DerValue(dout.toByteArray()));
		encX = new EncX(p1_cert,p1_priv,p2_cert,cInfo,panToken);
	}

	public AuthToken(PrivateKey p_priv, DerInputStream din) throws Exception {
		encX = new EncX(p_priv, din);	
		panToken = (PANToken) encX.getParameter();
		ContentInfo cInfo = encX.getContentInfo();
		ObjectIdentifier oid = cInfo.getContentType();
		if(!oid.equals(AuthTokenTBE.id_set_content_AuthTokenTBE))
			throw new FalseVerifiedException("AuthToken - AuthTokenDataOID");
		DerValue dv = cInfo.getContent();
		authTokenData = new AuthTokenData(new DerInputStream(dv.toByteArray()));
	}

	public void derEncode(OutputStream out) throws IOException {
		encX.derEncode(out);
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthToken>\n";
		s += authTokenData + "\n";
		s += panToken + "\n";
		return s;	
	}

	public static void main(String[] args) {
		try {
			//get payment-gateway's certificate
			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate p_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get payment-gateway's private key
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.jks");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"PaymentGateway".toCharArray());
			PrivateKey p_priv = (PrivateKey) ks.getKey("PG", "PaymentGateway".toCharArray());

			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);
			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			AuthTokenData auth = new AuthTokenData(id,cur,"dodol812",install,
									   new BigInt(1), new Date(), 
									   null);
		
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);

			AuthToken authToken = new AuthToken(p_priv,p_cert,p_cert,auth,pToken);
			System.out.println(authToken);

			DerOutputStream dout = new DerOutputStream();
			authToken.encode(dout);
			System.out.println("-------------after encoding--------------");
			authToken = new AuthToken(p_priv,
							  new DerInputStream(dout.toByteArray()));	
			System.out.println(authToken);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}		
		 
		
	