package digsec.set.payment.component;
import sun.security.util.*;
import java.io.*;
import java.security.cert.*;
import java.security.*;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.crypto.*;
import java.util.*;

/**
 *	<pre>
 *	1790 AuthTokenTBE ::= SEQUENCE {
 *	1791    authTokenData  AuthTokenData,
 *	1792    s              SO { P1, AuthTokenTBS }
 *	1793 }
 *	</pre>
 */

public class AuthTokenTBE implements DerEncoder {

	//fields
	public static ObjectIdentifier id_set_content_AuthTokenTBE = new ObjectIdentifier("2.23.42.0.55");
	public AuthTokenData 	authTokenData;
	public SO		  	so;

	//Constructor      
	public AuthTokenTBE(PrivateKey p_priv,
				  X509Certificate p_cert,
				  AuthTokenData authTokenData, 
				  AuthTokenTBS authTokenTBS) throws Exception {
		this.authTokenData = authTokenData;
		DerOutputStream dout = new DerOutputStream();
		authTokenTBS.encode(dout);
		so = new SO(p_priv,p_cert,
				new ContentInfo(new DerInputStream(dout.toByteArray())));
	}

	public AuthTokenTBE(DerInputStream din, PANToken panToken) throws Exception {
		ContentInfo cInfo = new ContentInfo(din);
		ObjectIdentifier id = cInfo.getContentType();
		if(!id.equals(id_set_content_AuthTokenTBE))
			throw new FalseVerifiedException("id-set-content-AuthTokenTBE");
		DerValue content = cInfo.getContent();
		din = new DerInputStream(content.toByteArray());
		DerValue[] adv = din.getSequence(2);
		authTokenData = new AuthTokenData(new DerInputStream(adv[0].toByteArray()));	
		AuthTokenTBS authTokenTBS = new AuthTokenTBS(authTokenData, panToken);
		DerOutputStream dout = new DerOutputStream();
		authTokenTBS.encode(dout);
		so = new SO(new DerInputStream(adv[1].toByteArray()),
				new ContentInfo(new DerInputStream(dout.toByteArray())));
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		authTokenData.encode(dout);
		so.encode(dout);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48, dout);
		dout.reset();
		ContentInfo cInfo = new ContentInfo(id_set_content_AuthTokenTBE,
								new DerValue(dout1.toByteArray()));
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthTokenTBE>\n";
		s += authTokenData + "\n";
		s += so		 + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			//get payment-gateway's certificate
			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate p_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get payment-gateway's private key
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.jks");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"PaymentGateway".toCharArray());
			PrivateKey p_priv = (PrivateKey) ks.getKey("PG", "PaymentGateway".toCharArray());

			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);
			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			AuthTokenData auth = new AuthTokenData(id,cur,"dodol812",install,
									   new BigInt(1), new Date(), 
									   null);
		
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);

			AuthTokenTBS aTBS = new AuthTokenTBS(auth,pToken);

			AuthTokenTBE aTBE = new AuthTokenTBE(p_priv,p_cert,auth,aTBS);
			System.out.println(aTBE);
			DerOutputStream dout = new DerOutputStream();
			aTBE.encode(dout);
			aTBE = new AuthTokenTBE(new DerInputStream(dout.toByteArray()),
							pToken);
			System.out.println("-------------after encoding--------------");
			System.out.println(aTBE);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}	