package digsec.set.payment.component;
import digsec.set.core.crypto.FalseVerifiedException;
import digsec.set.core.pkcs.ContentInfo;
import sun.security.util.*;
import java.io.*;
import java.util.*;

/**
 *	<pre>
 *	1795 AuthTokenTBS ::= SEQUENCE {
 *	1796    authTokenData  AuthTokenData,
 *	1797    panToken       PANToken
 *	1798 }
 *	</pre>
 */

public class AuthTokenTBS implements DerEncoder {

	//fields
	public static ObjectIdentifier 
		id_set_content_AuthTokenTBS = new ObjectIdentifier("2.23.42.0.19");
	public AuthTokenData 	authTokenData;
	public PANToken 		panToken;

	//Constructor
	public AuthTokenTBS(AuthTokenData authTokenData,
				  PANToken	    panToken) throws Exception {

		this.authTokenData = authTokenData;
		this.panToken	 = panToken;
	}

	public AuthTokenTBS(DerInputStream din) throws Exception {
		ContentInfo cInfo = new ContentInfo(din);
		ObjectIdentifier id = cInfo.getContentType();
		if(!id.equals(id_set_content_AuthTokenTBS))
			throw new FalseVerifiedException("id-set-content-AuthTokenTBS");
		DerValue content = cInfo.getContent();
		din = new DerInputStream(content.toByteArray());
		DerValue[] adv = din.getSequence(2);
		authTokenData = new AuthTokenData(new DerInputStream(adv[0].toByteArray()));
		panToken	  = new PANToken(new DerInputStream(adv[1].toByteArray()));
	}
	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();	
		authTokenData.derEncode(dout);
		panToken.derEncode(dout);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		ContentInfo cInfo = new ContentInfo(id_set_content_AuthTokenTBS,
								new DerValue(dout1.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthTokenTBS>\n";
		s += authTokenData + "\n";
		s += panToken + "\n";
		return s;
	}

	public static void main (String[] args) {
		try {
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);
			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			AuthTokenData auth = new AuthTokenData(id,cur,"dodol812",install,
									   new BigInt(1), new Date(), 
									   null);
		
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);

			AuthTokenTBS aTBS = new AuthTokenTBS(auth,pToken);
			System.out.println(aTBS);
			DerOutputStream dout = new DerOutputStream();
			aTBS.encode(dout);
			aTBS = new AuthTokenTBS(new DerInputStream(dout.toByteArray()));
			System.out.println("-----------after encode--------------");
			System.out.println(aTBS);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}