package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.lang.Math;	
import sun.security.util.DerEncoder;
import sun.security.util.ObjectIdentifier;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import sun.security.util.DerValue;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.AlgorithmId;

/**
 * <pre>
 * 243 BackKeyData ::= SEQUENCE {
 * 244    backAlgID  ALGORITHM-IDENTIFIER.&id({ContentEncryptionAlgorithms}),
 * 245    backKey    BackKey
 * 246 }
 * 247
 * 248 BackKey ::= OCTET STRING (SIZE(1..24))                         -- Secret
 *
 * </pre>
 */
public class BackKeyData implements DerEncoder 
{
	private byte[] backKey;
	private AlgorithmId backAlgID;
	private ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
	private AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC);

	/**
	 * Constructor for BackKeyData
	 *
	 * @param backKey key
	 * @param backAlgID algorithm
	 */
	public BackKeyData(byte[] backKey, AlgorithmId backAlgID) throws Exception {
		this.backKey = backKey;
		this.backAlgID = backAlgID;
	}

	/**
	 * Constructor for BackKeyData
	 *
	 * @param din DER representation of BackKeyData
	 *
	 */
	public BackKeyData(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);

		//ambil backKey
		backKey = adv[0].getOctetString();

		//ambil purchase amount
		backAlgID = AlgorithmId.parse(adv[1]);
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putOctetString(backKey);
		backAlgID.encode(dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for getting backKey
	 *
	 * @return backKey
	 */
	public byte[] getBackKey() {
		return backKey;
	}

	/**
	 * Method for getting backAlgID
	 *
	 * @return Algorithm ID
	 */
	public AlgorithmId getBackAlgID() {
		return backAlgID;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "";
		s = s + "BackKeyData ::= backKey: " + "\n" + hde.encodeBuffer(backKey) + "\n";
		s = s + "BackKeyData ::= backAlgID: " + "\n" + backAlgID.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData backKeyData = new BackKeyData(backKey, id_desCBC);
			
			System.out.println(backKeyData);

			DerOutputStream dout = new DerOutputStream();
			backKeyData.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			backKeyData = new BackKeyData(new DerInputStream(dout.toByteArray()));
			System.out.println(backKeyData);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

