package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	261 CountryCode ::= INTEGER (1..999)   -- ISO-3166 country code
 *	</pre>
 */

public class CountryCode implements DerEncoder {
	
	private BigInt countryCode;

	/**
	 * Constructor for CountryCode
	 */
	public CountryCode(BigInt countryCode) throws Exception {

		if(countryCode.toInt()<1 || countryCode.toInt()>999)
			throw new RuntimeException("Invalid input of CountryCode");

		this.countryCode = countryCode;
	}


	/**
	 * Constructor for CountryCode
	 * @param din DER representation of CountryCode
	 */
	public CountryCode(DerInputStream din) throws IOException {
		countryCode = din.getInteger();
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger (countryCode);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the totalFreightShippingAmount
	 */
	public BigInt getCountryCode() {
		return countryCode;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "CountryCode ::= " + countryCode + "\n";
		return s;
	}

	static void main(String[] args) {
		try 
		{
			CountryCode countryCode = new CountryCode (new BigInt(10));
			System.out.println(countryCode);

			DerOutputStream dout = new DerOutputStream();
			countryCode.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			countryCode = new CountryCode (new DerInputStream(dout.toByteArray()));
			System.out.println(countryCode);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
