package digsec.set.payment.component;

import java.io.*;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.crypto.DetachedDigest;

/**
 * <pre>
 *
 * 870 HOD ::= DD { HODInput }
 * 
 * </pre>
 */
public class HOD implements DerEncoder 
{
	private HODInput hodInput;
	private DetachedDigest dd;

	/**
	 * Constructor for HOD
	 *
	 * @param hodInput HODInput
	 *
	 */
	public HOD(HODInput hodInput) throws Exception {
		this.hodInput = hodInput;

		//ubah HODInput menjadi ContentInfo untuk input DetachedDigest
		DerOutputStream dout = new DerOutputStream();
		hodInput.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));

		dd = new DetachedDigest(ci);
	}

	/**
	 * Constructor for HOD
	 *
	 * @param din DER representation of HOD
	 *
	 */
	public HOD(DerInputStream din) throws Exception {
		dd = new DetachedDigest(din);
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		dd.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "HOD : " + dd.toString();

		return s;
	}

	static void main(String[] args) {
		try 
		{
			FileInputStream fi = new FileInputStream("HODInput.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);

			HODInput hodi = new HODInput(new DerInputStream(input));  

			HOD hod = new HOD(hodi);

			System.out.println(hod);

			DerOutputStream dout = new DerOutputStream();
			hod.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			hod = new HOD(new DerInputStream(dout.toByteArray()));
			System.out.println(hod);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

