package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Date;
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.crypto.DetachedDigest;
import digsec.set.core.crypto.HMAC;

/**
 * <pre>
 *
 *   820 HOIData ::= DD { OIData }                         -- PKCS#7 DigestedData
 * 
 * </pre>
 */
public class HOIData implements DerEncoder 
{
	public DetachedDigest dd;

	/**
	 * Constructor for HOIData
	 *
	 * @param piData PIData 
	 *
	 */
	public HOIData(OIData oiData) throws Exception {

		//ubah OIData menjadi ContentInfo untuk input DetachedDigest
		DerOutputStream dout = new DerOutputStream();
		oiData.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));

		dd = new DetachedDigest(ci);
	}

	/**
	 * Constructor for HPIData
	 *
	 * @param din DER representation of HPIData
	 *
	 */
	public HOIData(DerInputStream din) throws Exception {
		dd = new DetachedDigest(din);
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		dd.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "HOIData ::= " + dd.toString();

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*******bikin transIDs*********
			byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			//*******ini odSalt*******
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod = new HOD(hodi);

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod,odSalt,challM,brandID,bin);

			//******baru deh bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);
			System.out.println(hoiData);

			DerOutputStream dout = new DerOutputStream();
			hoiData.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			hoiData = new HOIData(new DerInputStream(dout.toByteArray()));
			System.out.println(hoiData);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

