package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Date;
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.crypto.DetachedDigest;
import digsec.set.core.crypto.HMAC;

/**
 * <pre>
 *
 *  818 HPIData ::= DD { PIData }                         -- PKCS#7 DigestedData
 * 
 * </pre>
 */
public class HPIData implements DerEncoder 
{
	public DetachedDigest dd;

	/**
	 * Constructor for HPIData
	 *
	 * @param piData PIData 
	 *
	 */
	public HPIData(PIData piData) throws Exception {

		//ubah PIData menjadi ContentInfo untuk input DetachedDigest
		DerOutputStream dout = new DerOutputStream();
		piData.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));

		dd = new DetachedDigest(ci);
	}

	/**
	 * Constructor for HPIData
	 *
	 * @param din DER representation of HPIData
	 *
	 */
	public HPIData(DerInputStream din) throws Exception {
		dd = new DetachedDigest(din);
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		dd.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "HPIData ::= " + dd.toString();

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs transIDs = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			//*******bikin inputs*********
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			//*******ini installRecurData*******
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			Inputs inputs = new Inputs(hodi,cur);

			//*******bikin merchantID*********
			String merchantID = "Toko Bunga Selalu Sayang";

			//*******bikin transStain*********
			byte[] XID = transIDs.getXID();
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,install,transStain,swIdent,acqBackKeyData);

			//******bikin PANData*******
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANData panData = new PANData("1294", "199908", secret, ab);

			//******bikin PIData******
			PIData piData = new PIData(piHead, panData);

			//******baru deh bikin PIData******
			HPIData hpiData = new HPIData(piData);
			System.out.println(hpiData);

			DerOutputStream dout = new DerOutputStream();
			hpiData.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			hpiData = new HPIData(new DerInputStream(dout.toByteArray()));
			System.out.println(hpiData);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

