package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 * <pre>
 * 846 Inputs ::= SEQUENCE {
 * 847    hod       HOD,
 * 848    purchAmt  CurrencyAmount
 * 849 }
 *
 * 870 HOD ::= DD { HODInput }
 * 
 * </pre>
 */
public class Inputs implements DerEncoder 
{
	private HOD hod;
	private CurrencyAmount purchAmt;

	/**
	 * Constructor for Inputs
	 *
	 * @param hodInput HODInput
	 * @param pAmnt purchase amount
	 *
	 */
	public Inputs(HODInput hodInput, CurrencyAmount purchAmt) throws Exception {
		this.hod = new HOD(hodInput);
		this.purchAmt = purchAmt;
	}

	/**
	 * Constructor for Inputs
	 *
	 * @param din DER representation of HODInput
	 *
	 */
	public Inputs(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);

		//ambil HOD
		hod = new HOD(new DerInputStream(adv[0].toByteArray()));

		//ambil purchase amount
		purchAmt = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		hod.encode(dout);					//tulis HOD	
		purchAmt.encode(dout);				//tulis purchase amount	

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for getting HOD
	 *
	 * @return order description
	 */
	public HOD getHOD() {
		return hod;
	}

	/**
	 * Method for getting purchase amount
	 *
	 * @return purchase amount 
	 */
	public CurrencyAmount getPurchAmt() {
		return purchAmt;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "";
		s = s + "Inputs ::= HOD: " + "\n" + hod.toString() + "\n";
		s = s + "Inputs ::= Purchase Amount: " + "\n" + purchAmt.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			Inputs inputs = new Inputs(hodi,cur);
			System.out.println(inputs);

			DerOutputStream dout = new DerOutputStream();
			inputs.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			inputs = new Inputs(new DerInputStream(dout.toByteArray()));
			System.out.println(inputs);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

