package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	286 Location ::= SEQUENCE {
 *	287    countryCode    CountryCode,
 *	288    city           [0] EXPLICIT SETString { ub-cityName=50 }  OPTIONAL,
 *	289    stateProvince  [1] EXPLICIT SETString { ub-stateProvince=50 }  OPTIONAL,
 *	290    postalCode     [2] EXPLICIT SETString { ub-postalCode=14 }  OPTIONAL,
 *	291    locationID     [3] EXPLICIT SETString { ub-locationID=10 }  OPTIONAL
 *	292 }
 *	</pre>
 */

public class Location implements DerEncoder {
	
	private CountryCode countryCode;
	private String city;
	private String stateProvince;
	private String postalCode;
	private String locationID;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for Location
	 */
	public Location(CountryCode countryCode, String city, String stateProvince,
		String postalCode, String locationID) throws Exception {

		if(city.length()>50 || stateProvince.length()>50
               || postalCode.length() > 14 || locationID.length()>10)
			throw new RuntimeException("Invalid input of Location");

		this.countryCode = countryCode;
		this.city = city;
		this.stateProvince = stateProvince;
		this.postalCode = postalCode;
		this.locationID = locationID;
	}

	/**
	 * Constructor for Location
	 * @param din DER representation of Location
	 */
	public Location(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(5);

		//isi countryCode
		countryCode = new CountryCode(new DerInputStream(adv[0].toByteArray()));

		//isi city
		if (!adv[1].equals(derNull)) {
			city = adv[1].data.getDerValue().getPrintableString();
		}
		else
			city = null;

		// isi stateProvince
		if (!adv[2].equals(derNull)) {
			stateProvince = adv[2].data.getDerValue().getPrintableString();
		}
		else
			stateProvince = null;

		// isi postalCode
		if (!adv[3].equals(derNull)) {
			postalCode = adv[3].data.getDerValue().getPrintableString();
		}
		else
			postalCode = null;

		// isi locationID
		if (!adv[4].equals(derNull)) {
			locationID = adv[4].data.getDerValue().getPrintableString();
		}
		else
			locationID = null;
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout = new DerOutputStream();

		countryCode.encode (dout);

		//kalau city ada, tulis, kalau tidak, tulis null
		if (city != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (city);
			dout.write((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau stateProvince ada, tulis, kalau tidak, tulis null
		if (stateProvince != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (stateProvince);
			dout.write((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau postalCode ada, tulis, kalau tidak, tulis null
		if (postalCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (postalCode);
			dout.write((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau locationID ada, tulis, kalau tidak, tulis null
		if (locationID != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (locationID);
			dout.write((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the countryCode
	 */
	public CountryCode getCountryCode() {
		return countryCode;
	}

	/**
	 * Method for get the city
	 */
	public String getCity() {
		return city;
	}

	/**
	 * Method for get the stateProvince
	 */
	public String getStateProvince() {
		return stateProvince;
	}

	/**
	 * Method for get the postalCode
	 */
	public String getPostalCode() {
		return postalCode;
	}

	/**
	 * Method for get the locationID
	 */
	public String getLocationID() {
		return locationID;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of Location object
	 */
	public String toString() {
		String s = "";

		s = s + "Location ::= CountryCode : " + countryCode + "\n";

		if(city != null)
			s = s + "Location ::= City : " + city + "\n";

		if(stateProvince != null)
			s = s + "Location ::= StateProvince : " + stateProvince + "\n";

		if(postalCode != null)
			s = s + "Location ::= PostalCode : " + postalCode + "\n";

		if(locationID != null)
			s = s + "Location ::= LocationID : " + locationID + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			
			Location location = new Location (countryCode, city, stateProvince,
				postalCode, locationID);
			System.out.println(location);

			DerOutputStream dout = new DerOutputStream();
			location.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			location = new Location (new DerInputStream(dout.toByteArray()));
			System.out.println(location);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
