package digsec.set.payment.component;
import java.io.*;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	1906 MerTermIDs ::= SEQUENCE {
 *	1907    merchantID  MerchantID,
 *	1908    terminalID  VisibleString (SIZE(1..ub-terminalID=48)) OPTIONAL,
 *	1909    agentNum    INTEGER (0..MAX)  OPTIONAL,
 *	1910    chainNum    [0] INTEGER (0..MAX)  OPTIONAL,
 *	1911    storeNum    [1] INTEGER (0..MAX)  OPTIONAL
 *	1912 }
 *
 *	294 MerchantID ::= SETString { ub-MerchantID=30 }
 *	</pre>
 */

public class MerTermIDs implements DerEncoder {

	//fields
	public String merchantID;
	public String terminalID;
	public BigInt agentNum;
	public BigInt chainNum;
	public BigInt storeNum;

	//constructors
	public MerTermIDs(String merchantID,
				String terminalID,
				BigInt agentNum,
				BigInt chainNum,
				BigInt storeNum) throws Exception {

		if(merchantID.length()<1 || merchantID.length()>30)
			throw new IllegalArgumentException("MerTermIDs::merchantID");
		this.merchantID=merchantID;
		if(terminalID!=null && (terminalID.length()<1 || terminalID.length()>48))
			throw new IllegalArgumentException("MerTermIDs::terminalID");
		this.terminalID=terminalID;
		this.agentNum=agentNum;
		this.chainNum=chainNum;
		this.storeNum=storeNum;
	} 

	public MerTermIDs(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(5);
		merchantID = adv[0].getPrintableString();
		try {
			terminalID = adv[1].getPrintableString();
		}
		catch(Exception e) {
			terminalID = null;
		}
		try {
			agentNum = adv[2].getInteger();
		}
		catch(Exception e) {
			agentNum = null;
		}
/*		if((new DerInputStream(adv[3].toByteArray())).peekByte() == -128) {
			adv[3].resetTag((byte)0x02);
			chainNum=adv[3].getInteger();
		}*/
		if(adv[3].tag == (byte)0x80) {
			adv[3].resetTag((byte)0x02);
			chainNum=adv[3].getInteger();
		}
		else
			chainNum=null;
		if((new DerInputStream(adv[4].toByteArray())).peekByte() == -127) {
			adv[4].resetTag((byte)0x02);
			storeNum=adv[4].getInteger();
		}
		else
			storeNum=null;

	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putPrintableString(merchantID);
		if(terminalID!=null)
			dout.putPrintableString(terminalID);
		else
			dout.putNull();
		if(agentNum!=null)
			dout.putInteger(agentNum);
		else
			dout.putNull();
		if(chainNum!=null) {
			DerOutputStream dout1 = new DerOutputStream();
			dout1.putInteger(chainNum);
			DerValue dv = new DerValue(dout1.toByteArray());
			dv.resetTag((byte)-128);
			dout.putDerValue(dv);
		}
		else
			dout.putNull();
		if(storeNum!=null) {
			DerOutputStream dout1 = new DerOutputStream();
			dout1.putInteger(storeNum);
			DerValue dv = new DerValue(dout1.toByteArray());
			dv.resetTag((byte)-127);
			dout.putDerValue(dv);
		}
		else
			dout.putNull();		

		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)0x30,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		derEncode(dout);
	}

	public String toString() {
		String s="MerTermIDs\n";
		s += "merchantID: " + merchantID + "\n";
		if(terminalID!=null)
		s += "terminalID: " + terminalID + "\n";
		if(agentNum!=null)
		s += "agentNum: " + agentNum + "\n";
		if(chainNum!=null)
		s += "chainNum: " + chainNum + "\n";
		if(storeNum!=null)
		s += "storeNum: " + storeNum + "\n";
		return s;
	}
	
	public static void main(String[] args) {
		try {
			MerTermIDs mti = new MerTermIDs("Dhany's Butique","812-317",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));
								
			DerOutputStream dout = new DerOutputStream();
			mti.encode(dout);
			FileOutputStream fo = new FileOutputStream("MerTermIDs.dat");
			fo.write(dout.toByteArray());
			fo.close();
		
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			FileInputStream fi = new FileInputStream("MerTermIDs.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);
			mti = new MerTermIDs(new DerInputStream(input));
			System.out.println(mti);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}