package digsec.set.payment.component;
import digsec.set.core.pkcs.*;
import digsec.set.core.crypto.OAEPEncoder;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.io.*;
import java.util.StringTokenizer;

/**
 *	<pre>
 *	552 PANOnly ::= SEQUENCE {
 *	553    pan      PAN,
 *	554    exNonce  Nonce
 *	555 }
 *	
 *	298 PAN ::= NumericString (SIZE(1..19))
 *	
 *	296 Nonce ::= OCTET STRING (SIZE(20))
 *	</pre>
 */

public class PANOnly implements DerEncoder, OAEPEncoder {

	protected String pan; // size(1..19)
	protected byte[] exNonce; // size(20)	

	public static ObjectIdentifier id_set_content_PANOnly = new ObjectIdentifier("2.23.42.0.2");

	public PANOnly(String pan, byte[] nonce) throws Exception{
		if(pan.length()<1 || pan.length()>19 || nonce.length != 20)
			throw new RuntimeException("Invalid input of PANOnly");
		this.pan=pan;
		exNonce = nonce;
	}
	
	public PANOnly(DerInputStream din) throws Exception {
		ContentInfo cInfo = new ContentInfo(din);
		DerInputStream din1 = new DerInputStream(cInfo.getContent().toByteArray());

		DerValue[] adv = din1.getSequence(2);
		pan = adv[0].getPrintableString();
		exNonce = adv[1].getOctetString();
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		DerValue[] adv = new DerValue[2];
		adv[0] = new DerValue(pan); //18 is tag for NumericString 
		adv[1] = new DerValue((byte)4,exNonce); 	
		dout.putSequence(adv);
		ContentInfo cInfo = new ContentInfo(id_set_content_PANOnly, new DerValue(dout.toByteArray()));
		dout.reset();
		cInfo.encode(dout);

		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		
		derEncode(dout);
	}	

	public byte[] encodeOAEP () throws IOException{
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		
		StringBuffer strbuf = new StringBuffer(pan);
		if(pan.length() < 19) {
			for(int i=pan.length(); i<19; i++)
				strbuf=strbuf.append((char)0x20); 
		}
		bout.write(strbuf.toString().getBytes());
		bout.write(exNonce);		
		return bout.toByteArray();
	}

	public static PANOnly parseOAEP(byte[] oaep) throws Exception {
		ByteArrayInputStream bin = new ByteArrayInputStream(oaep);
		byte[] b_pan = new byte[19];
		bin.read(b_pan,0,19);
		
		StringTokenizer strtok = new StringTokenizer(new String(b_pan));		
		String newPan = strtok.nextToken(" ");		

		byte[] nonce = new byte[20];
		bin.read(nonce,0,20);
		return new PANOnly(newPan,nonce);
	}

	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();
		String s = "";
		s += "PANOnly\n";
		s += "pan: " + pan + "\n";
		s += "exNonce: " + "\n" + hde.encodeBuffer(exNonce) + "\n";
		return s;
	}

	public String getPAN() {
		return pan;
	}

	public byte[] getNonce() {
		return exNonce;
	}

	public static void main (String[] args) {
		try { 
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANOnly pOnly = new PANOnly("1294", ab);
			DerOutputStream dout = new DerOutputStream();
			pOnly.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
			pOnly = new PANOnly(new DerInputStream(dout.toByteArray()));
			System.out.println(pOnly);
			byte[] temp = pOnly.encodeOAEP();
			System.out.println(hde.encodeBuffer(temp));
			pOnly = PANOnly.parseOAEP(temp);
			System.out.println(pOnly);
			dout.reset();
			pOnly.encode(dout);
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}