package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.crypto.HMAC;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;

/**
 * <pre>
 * 828 PIData ::= SEQUENCE {
 * 829    piHead   PIHead,
 * 830    panData  PANData
 * 831 }
 *
 * 
 * </pre>
 */
public class PIData implements DerEncoder 
{
	public PIHead piHead;
	public PANData panData;
	public static ObjectIdentifier id_set_content_PIData = new ObjectIdentifier("2.23.42.0.5");

	/**
	 * Constructor for PIData
	 *
	 * @param piHead
	 * @param panData
	 *
	 */
	public PIData(PIHead piHead,PANData panData) throws Exception {
		this.piHead = piHead;
		this.panData = panData;
	}

	/**
	 * Constructor for PIData
	 *
	 * @param din DER representation of PIData
	 *
	 */
	public PIData(DerInputStream din) throws Exception {
		//jadiin Content Info
		ContentInfo contentInfo = new ContentInfo(din);
		//ambil content
		DerValue dv = contentInfo.getContent();
		DerInputStream din1 = new DerInputStream(dv.toByteArray());

		DerValue[] adv = din1.getSequence(2);

		this.piHead = new PIHead(new DerInputStream(adv[0].toByteArray()));
		this.panData= new PANData(new DerInputStream(adv[1].toByteArray()));
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		piHead.derEncode(dout);					
		panData.derEncode(dout);				

		//tambahin tag sequence
		DerValue dv = new DerValue(DerValue.tag_Sequence, dout.toByteArray());
	
		//jadiin ContentInfo
		ContentInfo cInfo = new ContentInfo(id_set_content_PIData,dv);		
		cInfo.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "";
		s = s + "PIData ::= PIHead: " + "\n" + piHead.toString() + "\n";
		s = s + "PIData ::= PANData: " + "\n" + panData.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs transIDs = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			//*******bikin inputs*********
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			//*******ini installRecurData*******
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			Inputs inputs = new Inputs(hodi,cur);

			//*******bikin merchantID*********
			String merchantID = "Toko Bunga Selalu Sayang";

			//*******bikin transStain*********
			byte[] XID = transIDs.getXID();
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,install,transStain,swIdent,acqBackKeyData);

			//******bikin PANData*******
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANData panData = new PANData("1294", "199908", secret, ab);

			//******baru deh bikin PIHead*******
			PIData piData = new PIData(piHead, panData);

			System.out.println(piData);

			DerOutputStream dout = new DerOutputStream();
			piData.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			piData = new PIData(new DerInputStream(dout.toByteArray()));
			System.out.println(piData);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

