package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Calendar;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import sun.security.x509.X500Name;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.crypto.DetachedDigest;
import digsec.set.core.crypto.EX;
import digsec.set.core.crypto.HMAC;

import java.security.*;
import java.security.cert.*;
import java.security.interfaces.*;
import sun.security.util.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.crypto.provider.*;

/**
 * <pre>
 *
 * 799 PIDualSigned ::= SEQUENCE {
 * 800    piSignature  PISignature,
 * 801    exPIData     EX { P, PI-OILink, PANData }
 * 802 }
 * 
 * </pre>
 */
public class PIDualSigned 
{
	public PISignature piSignature;
	public EX ex;
	public static ObjectIdentifier id_set_contentType = new ObjectIdentifier("2.23.42.0");

	/**
	 * Constructor for PIDualSigned 
	 *
	 * @param keys PrivateKey of the signer 
	 * @param certs X509Certificate of the signer, optional 
	 * @param pitbs PI-TBS
	 *
	 */
	public PIDualSigned(X509Certificate cert, PIOILink pLink, PANData pData) throws Exception {

		//ubah pLink menjadi ContentInfo untuk input EX
		DerOutputStream dout = new DerOutputStream();
		pLink.encode(dout);

		ContentInfo ci= new ContentInfo(id_set_contentType,new DerValue(dout.toByteArray()));

		ex = new EX(cert,ci,pData);
	}

	/**
	 * Constructor for PIDualSigned
	 *
	 * @privKey private key of the recipient
	 * @param din DER representation of PIOILink
	 *
	 */
	public PIDualSigned(PrivateKey privKey, DerInputStream din) throws Exception {
		ex = new EX(privKey, din);	
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		ex.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "PIDualSigned : " + ex.toString();

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*****ambil sertifikat dan private key*****
			Security.addProvider(new ABAProvider());

			String alias = "yudi";
			String password = "kunciku";
			String pkpassword = "kunciku";
			InputStream inStream = new FileInputStream(args[0]);
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,password.toCharArray());

			X509Certificate cert = (X509Certificate)ks.getCertificate(alias);
			PrivateKey pk = (PrivateKey)ks.getKey(alias,pkpassword.toCharArray());
			inStream.close();		

			//******bikin PANData*******
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANData panData = new PANData("1294", "199908", secret, ab);

			//*******bikin transIDs*********
			byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			//*******ini odSalt*******
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod = new HOD(hodi);

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//*******bikin Inputs*********
			Inputs inputs = new Inputs(hodi,cur);

			//*******bikin merchantID*********
			String merchantID = "Toko Bunga Selalu Sayang";

			//*******bikin transStain*********
			byte[] XID = transIDs.getXID();
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			//byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******bikin OIData*******
			OIData oiData = new OIData(transIDs,rrpid,challC,hod,odSalt,challM,brandID,bin);

			//******bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,install,transStain,swIdent,acqBackKeyData);

			//******bikin PIOILink*******
			PIOILink pioiLink = new PIOILink(piHead, oiData);

			//******bikin PIDualSigned*******
			PIDualSigned pds = new PIDualSigned(cert, pioiLink, panData);
			System.out.println(pds);

			DerOutputStream dout = new DerOutputStream();
			pds.encode(dout);

			System.out.println("*******der********");
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("\n*******reconstruct********");
			pds = new PIDualSigned(pk, new DerInputStream(dout.toByteArray()));
			System.out.println(pds);
			
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

