package digsec.set.payment.component;

import java.io.*;
import java.util.Date;
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.crypto.DetachedDigest;
import digsec.set.core.crypto.HMAC;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;

/**
 * <pre>
 *
 * 807 PI-OILink ::= L { PIHead, OIData }
 * 
 * </pre>
 */
public class PIOILink implements DerEncoder 
{
	public PIHead piHead;
	public DetachedDigest dd;

	/**
	 * Constructor for PIOILink
	 *
	 * @param piHead PIHead
	 * @param oiData OIData
	 *
	 */
	public PIOILink(PIHead piHead, OIData oiData) throws Exception {
		this.piHead = piHead;

		//ubah oiData menjadi ContentInfo untuk input DetachedDigest
		DerOutputStream dout = new DerOutputStream();
		oiData.encode(dout);
		ContentInfo ci= new ContentInfo(new DerInputStream(dout.toByteArray()));

		dd = new DetachedDigest(ci);
	}

	/**
	 * Constructor for PIOILink
	 *
	 * @param din DER representation of PIOILink
	 *
	 */
	public PIOILink(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);

		piHead = new PIHead(new DerInputStream(adv[0].toByteArray()));
		dd = new DetachedDigest(new DerInputStream(adv[1].toByteArray()));	
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		piHead.encode(dout);
		dd.encode(dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "PIOILink: PIHead: " + "\n" + piHead.toString() + "\n";
		s = s + "PIOILink: DetachedDigest: " + "\n" + dd.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			FileInputStream fi = new FileInputStream("OIData.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);

			OIData oiData = new OIData(new DerInputStream(input));
			
			fi = new FileInputStream("PIHead.dat");
			input = new byte[fi.available()];
			fi.read(input);
			
			PIHead piHead = new PIHead(new DerInputStream(input));
			
			//******baru deh bikin PIHead*******
			PIOILink pioiLink = new PIOILink(piHead, oiData);

			System.out.println(pioiLink.toString());

			DerOutputStream dout = new DerOutputStream();
			pioiLink.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			pioiLink = new PIOILink(new DerInputStream(dout.toByteArray()));
			System.out.println(pioiLink);
	
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

