package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.math.BigInteger;
import java.lang.Math;	
import java.util.Calendar;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import sun.security.x509.X500Name;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.crypto.DetachedDigest;
import digsec.set.core.crypto.SO;
import digsec.set.core.crypto.HMAC;

import java.security.*;
import java.security.cert.*;
import java.security.interfaces.*;
import sun.security.util.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.crypto.provider.*;

/**
 * <pre>
 *
 *  811 PISignature ::= SO { C, PI-TBS }
 * 
 * </pre>
 */
public class PISignature
{
	public SO so;

	/**
	 * Constructor for PISignature
	 *
	 * @param keys PrivateKey of the signer 
	 * @param certs X509Certificate of the signer, optional 
	 * @param pitbs PI-TBS
	 *
	 */
	public PISignature(PrivateKey[] keys, X509Certificate[] certs, PITBS pitbs) throws Exception {

		//ubah PI-TBS menjadi ContentInfo untuk input SO
		DerOutputStream dout = new DerOutputStream();
		pitbs.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));

		so = new SO(keys,certs,ci);
	}

	/**
	 * Constructor for PISignature
	 *
	 * @param keys PrivateKey of the signer 
	 * @param certs X509Certificate of the signer, optional 
	 * @param pitbs PI-TBS
	 *
	 */
	public PISignature(PrivateKey keys, X509Certificate certs, PITBS pitbs) throws Exception {

		//ubah PI-TBS menjadi ContentInfo untuk input SO
		DerOutputStream dout = new DerOutputStream();
		pitbs.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));

		so = new SO(keys,certs,ci);
	}

	/**
	 * Constructor for PISignature
	 *
	 * @param din DER representation of PISignature
	 *
	 */
	public PISignature(DerInputStream din, PITBS pitbs) throws Exception {

		//ubah PI-TBS menjadi ContentInfo untuk input SO
		DerOutputStream dout = new DerOutputStream();
		pitbs.encode(dout);
		ContentInfo ci = new ContentInfo(new DerInputStream(dout.toByteArray()));

		so = new SO(din,ci);
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		so.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		String s = "";
		s = s + "PISignature : " + so.toString();

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*****ambil sertifikat dan private key*****
			Security.addProvider(new ABAProvider());

			String alias = "yudi";
			String password = "kunciku";
			String pkpassword = "kunciku";
			InputStream inStream = new FileInputStream(args[0]);
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,password.toCharArray());

			X509Certificate cert = (X509Certificate)ks.getCertificate(alias);
			PrivateKey pk = (PrivateKey)ks.getKey(alias,pkpassword.toCharArray());
			inStream.close();		

			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs transIDs = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			//*******bikin inputs*********
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			//*******ini installRecurData*******
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			Inputs inputs = new Inputs(hodi,cur);

			//*******bikin merchantID*********
			String merchantID = "Toko Bunga Selalu Sayang";

			//*******bikin transStain*********
			byte[] XID = transIDs.getXID();
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,install,transStain,swIdent,acqBackKeyData);

			//******bikin PANData*******
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANData panData = new PANData("1294", "199908", secret, ab);

			//******bikin PIData******
			PIData piData = new PIData(piHead, panData);

			//******bikin HPIData******
			HPIData hpiData = new HPIData(piData);

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			HOD hod = new HOD(hodi);

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod,odSalt,challM,brandID,bin);

			//******bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);

			//******bikin PI-TBS******
			PITBS pitbs = new PITBS(hpiData,hoiData);
			
			//******baru deh bikin PISignature******
			PISignature piSign = new PISignature(pk, cert, pitbs);
			System.out.println(piSign.toString());

			DerOutputStream dout = new DerOutputStream();
			piSign.encode(dout);

			System.out.println("*******der********");
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encode(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			piSign = new PISignature(new DerInputStream(dout.toByteArray()), pitbs);
			System.out.println(piSign.toString());
			
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

