package digsec.set.payment.component;

import java.io.OutputStream;
import java.io.IOException;
import java.util.Date;
import java.util.Calendar;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.core.crypto.HMAC;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.pkcs.ContentInfo;

/**
 * <pre>
 * 813 PI-TBS ::= SEQUENCE {
 * 814    hPIData  HPIData,
 * 815    hOIData  HOIData
 * 816 }
 *
 * 
 * </pre>
 */
public class PITBS implements DerEncoder 
{
	public HPIData hpiData;
	public HOIData hoiData;
	public static ObjectIdentifier id_set_content_PITBS = new ObjectIdentifier("2.23.42.0.13");

	/**
	 * Constructor for PITBS
	 *
	 * @param hpiData
	 * @param hoiData
	 *
	 */
	public PITBS(HPIData hpiData, HOIData hoiData) throws Exception {
		this.hpiData = hpiData;
		this.hoiData = hoiData;
	}

	/**
	 * Constructor for PIData
	 *
	 * @param din DER representation of PIData
	 *
	 */
	public PITBS(DerInputStream din) throws Exception {
		//jadiin Content Info
		ContentInfo contentInfo = new ContentInfo(din);
		//ambil content
		DerValue dv = contentInfo.getContent();
		DerInputStream din1 = new DerInputStream(dv.toByteArray());

		DerValue[] adv = din1.getSequence(2);

		this.hpiData = new HPIData(new DerInputStream(adv[0].toByteArray()));
		this.hoiData = new HOIData(new DerInputStream(adv[1].toByteArray()));
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		hpiData.derEncode(dout);					
		hoiData.derEncode(dout);				

		//tambahin tag sequence
		DerValue dv = new DerValue(DerValue.tag_Sequence, dout.toByteArray());
	
		//jadiin ContentInfo
		ContentInfo cInfo = new ContentInfo(id_set_content_PITBS,dv);		
		cInfo.derEncode(out);
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MessageHeader object
	 */
	public String toString() {
		HexDumpEncoder hde = new HexDumpEncoder();

		String s = "";
		s = s + "HPIData ::= HPIData: " + "\n" + hpiData.toString() + "\n";
		s = s + "HOIData ::= HOIData: " + "\n" + hoiData.toString() + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs transIDs = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			//*******bikin inputs*********
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			//*******ini installRecurData*******
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			Inputs inputs = new Inputs(hodi,cur);

			//*******bikin merchantID*********
			String merchantID = "Toko Bunga Selalu Sayang";

			//*******bikin transStain*********
			byte[] XID = transIDs.getXID();
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,install,transStain,swIdent,acqBackKeyData);

			//******bikin PANData*******
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANData panData = new PANData("1294", "199908", secret, ab);

			//******bikin PIData******
			PIData piData = new PIData(piHead, panData);

			//******bikin HPIData******
			HPIData hpiData = new HPIData(piData);

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			HOD hod = new HOD(hodi);

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod,odSalt,challM,brandID,bin);

			//******bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);

			//******baru deh bikin HPIData******
			PITBS pitbs = new PITBS(hpiData,hoiData);
			System.out.println(pitbs);

			DerOutputStream dout = new DerOutputStream();
			pitbs.encode(dout);

			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			pitbs = new PITBS(new DerInputStream(dout.toByteArray()));
			System.out.println(pitbs);
		}
		catch(Exception e) 
		{
			e.printStackTrace();
		}
	}
}

