package digsec.set.payment.component;
import sun.security.util.*;
import java.util.Date;
import sun.misc.HexDumpEncoder;
import java.io.*;
import java.math.*;

/**
 *	<pre>
 *	1914 RRTags ::= SEQUENCE {
 *	1915    rrpid        RRPID,
 *	1916    merTermIDs   MerTermIDs,
 *	1917    currentDate  Date
 *	1918 }
 *	</pre>
 */

public class RRTags implements DerEncoder {

	//fields
	private byte[] RRPID; //20byte
	private MerTermIDs merTermIDs;
	private Date currentDate;

	//constructors
	public RRTags(byte[] RRPID, MerTermIDs mti) throws Exception {
		if(RRPID.length!=20)
			throw new IllegalArgumentException("RRTags::RRPID");
		this.RRPID=RRPID;
		merTermIDs=mti;
		currentDate = new Date();
	}

	public RRTags(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(3);
		RRPID = adv[0].getOctetString();
		merTermIDs = new MerTermIDs(new DerInputStream(adv[1].toByteArray()));
		currentDate = (new DerInputStream(adv[2].toByteArray())).getGeneralizedTime();
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putOctetString(RRPID);
		merTermIDs.derEncode(dout);
		dout.putGeneralizedTime(currentDate);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)0x30,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws Exception {
		derEncode(dout);
	}

	public byte[] getRRPID() {
		return RRPID;
	}

	public MerTermIDs getMerTermIDs() {
		return merTermIDs;
	}

	public Date getDate() {
		return currentDate;
	}

	public String toString() {
		String s = "RRTags\n";
		s += "RRPID: " + (new BigInteger(RRPID)).toString(16) + "\n";
		s += merTermIDs;
		s += currentDate;
		return s;
	}

	public static void main(String[] args) {

		try{
			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);
			DerOutputStream dout = new DerOutputStream();
			rr.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));
			rr = new RRTags(new DerInputStream(dout.toByteArray()));
			System.out.println(rr);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}