package digsec.set.payment.component;
import sun.security.util.*;
import java.io.*;
import java.util.*;
import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	1957 Recurring ::= SEQUENCE {
 *	1958    recurringFrequency  INTEGER (1..ub-recurringFrequency=366),
 *	1959    recurringExpiry     Date
 *	1960 }
 *	</pre>
 */
public class Recurring extends InstallRecurInd implements DerEncoder {

	public BigInt recurringFrequency;
	public Date	recurringExpiryDate;

	/**
	 *	@param freq the minimum days between authorizations
	 *	@param date the expiry date
       */
	public Recurring(BigInt freq, Date date) {
		recurringFrequency=freq;
		recurringExpiryDate=date;
	}

	public Recurring(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSet(2,true);
		recurringFrequency=adv[0].getInteger();
		recurringExpiryDate=(new DerInputStream(adv[1].toByteArray())).getGeneralizedTime();
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger(recurringFrequency);
		dout.putGeneralizedTime(recurringExpiryDate);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)-95,dout);
		out.write(dout1.toByteArray());	
	}

	public void encode(DerOutputStream dout) throws Exception {
		derEncode(dout);
	}

	public String toString() {
		String s = "Recurring\n";
		s += "recurringFrequency : " + recurringFrequency + "\n";
		s += "recurringExpiryDate: " + recurringExpiryDate + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			DerOutputStream dout = new DerOutputStream();
			rec.encode(dout);

			byte[] der = dout.toByteArray();
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(der));

			rec = new Recurring(new DerInputStream(der));
			System.out.println(rec);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}