package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
/**
 *	<pre>
 *	1217 AuthCharInd ::= ENUMERATED {
 *	1218    directMarketing      (0),
 *	1219    recurringPayment     (1),
 *	1220    addressVerification  (2),
 *	1221    preferredCustomer    (3),
 *	1222    incrementalAuth      (4)
 *	1223 }
 *	</pre>
 */

public class AuthCharInd implements DerEncoder {
	
	public static BigInt directMarketing = new BigInt(0);
	public static BigInt recurringPayment = new BigInt(1);
	public static BigInt addressVerification = new BigInt(2);
	public static BigInt preferredCustomer = new BigInt(3);
	public static BigInt incrementalAuth = new BigInt(4);
	private BigInt value;

	/**
	 * Constructor for AuthCharInd
	 */
	public AuthCharInd(BigInt value) throws Exception {
		if(value.toInt()<0 || value.toInt()>4)
			throw new RuntimeException("Invalid input of AuthCharInd");

		this.value = value;
	}

	/**
	 * Constructor for AuthCharInd
	 * @param din DER representation of AuthCharInd
	 */
	public AuthCharInd(DerInputStream din) throws IOException {
		this.value = din.getEnumerated();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putEnumerated (value);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public BigInt getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if (value.equals(directMarketing))
			s = s + "AuthCharInd ::= directMarketing \n";
		else if (value.equals(recurringPayment))
			s = s + "AuthCharInd ::= recurringPayment \n";
		else if (value.equals(addressVerification))
			s = s + "AuthCharInd ::= addressVerification \n";
		else if (value.equals(preferredCustomer))
			s = s + "AuthCharInd ::= preferredCustomer \n";
		else
			s = s + "AuthCharInd ::= incrementalAuth \n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt value = new BigInt(1);
			AuthCharInd authCharInd = new AuthCharInd (value);
			System.out.println(authCharInd);

			DerOutputStream dout = new DerOutputStream();
			authCharInd.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			authCharInd = new AuthCharInd (new DerInputStream(dout.toByteArray()));
			System.out.println(authCharInd);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
