package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
/**
 *	<pre>
 *	3238 AutoApplicableRate ::= CHOICE {
 *	3239    dailyRentalRate   [0] CurrencyAmount,
 *	3240    weeklyRentalRate  [1] CurrencyAmount
 *	3241 }
 *	</pre>
 */

public class AutoApplicableRate implements DerEncoder {
	private CurrencyAmount value;
	private int flag;

	/**
	 * Constructor for AutoApplicableRate
	 */
	public AutoApplicableRate(int flag, CurrencyAmount value) throws Exception {		
		if(flag<0 || flag>1)
			throw new RuntimeException("Invalid input of AutoApplicableRate");
		this.value = value;
	}

	/**
	 * Constructor for AutoApplicableRate
	 * @param din DER representation of AutoApplicableRate
	 */
	public AutoApplicableRate(DerInputStream din) throws IOException {
		DerValue adv = din.getDerValue();

		if (adv.tag==(byte)0xA0) flag=0;
		else flag=1;
		adv.resetTag(DerValue.tag_Sequence);
		value = new CurrencyAmount(new DerInputStream(adv.toByteArray()));
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout = new DerOutputStream();

		// tulis sequence untuk objek ini
		value.encode (midout);
		if (flag==0) dout.writeImplicit((byte)0xA0, midout);
		else dout.writeImplicit((byte)0xA1, midout);
		out.write(dout.toByteArray());
	}

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public CurrencyAmount getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of AutoApplicableRate object
	 */
	public String toString() {
		String s = "";

		if (flag==0)
			s = s + "AutoApplicableRate ::= " + "dailyRentalRate\n" + value + "\n";
		else
			s = s + "AutoApplicableRate ::= " + "weeklyRentalRate\n" + value + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);
			System.out.println(autoApplicableRate);

			DerOutputStream dout = new DerOutputStream();
			autoApplicableRate.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			autoApplicableRate = new AutoApplicableRate (new DerInputStream(dout.toByteArray()));
			System.out.println(autoApplicableRate);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

