package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3224 AutoNoShow ::= ENUMERATED {
 *	3225    normalVehicle   (0),
 *	3226    specialVehicle  (1)
 *	3227 }
 *	</pre>
 */

public class AutoNoShow implements DerEncoder {
	
	public static BigInt normalVehicle = new BigInt(0);
	public static BigInt specialVehicle = new BigInt(1);
	private BigInt value;

	/**
	 * Constructor for AutoNoShow
	 */
	public AutoNoShow(BigInt value) throws Exception {
		if(value.toInt()<0 || value.toInt()>1)
			throw new RuntimeException("Invalid input of AutoNoShow");

		this.value = value;
	}

	/**
	 * Constructor for AutoNoShow
	 * @param din DER representation of AutoNoShow
	 */
	public AutoNoShow(DerInputStream din) throws IOException {
		this.value = din.getEnumerated();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putEnumerated (value);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public BigInt getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if (value.equals(normalVehicle))
			s = s + "AutoNoShow ::= normalVehicle \n";
		else
			s = s + "AutoNoShow ::= specialVehicle \n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt value = new BigInt(1);
			AutoNoShow autoNoShow = new AutoNoShow (value);
			System.out.println(autoNoShow);

			DerOutputStream dout = new DerOutputStream();
			autoNoShow.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			autoNoShow = new AutoNoShow (new DerInputStream(dout.toByteArray()));
			System.out.println(autoNoShow);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

