package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3229 AutoRateInfo ::= SEQUENCE {
 *	3230    autoApplicableRate    AutoApplicableRate,
 *	3231    lateReturnHourlyRate  [0] CurrencyAmount  OPTIONAL,
 *	3232    distanceRate          [1] CurrencyAmount  OPTIONAL,
 *	3233    freeDistance          [2] Distance  OPTIONAL,
 *	3234    vehicleClassCode      [3] EXPLICIT SETString { ub-vehicleClass=2 }  OPTIONAL,
 *	3235    corporateID           [4] EXPLICIT SETString { ub-corpID=12 }  OPTIONAL
 *	3236 }
 *	</pre>
 */

public class AutoRateInfo implements DerEncoder {

	private AutoApplicableRate autoApplicableRate;
	private CurrencyAmount lateReturnHourlyRate;
	private CurrencyAmount distanceRate;
	private Distance freeDistance;
	private String vehicleClassCode;
	private String corporateID;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for AutoRateInfo
	 */
	public AutoRateInfo(AutoApplicableRate autoApplicableRate,
		CurrencyAmount lateReturnHourlyRate, CurrencyAmount distanceRate,
		Distance freeDistance, String vehicleClassCode, String corporateID) throws Exception {

		if(vehicleClassCode.length()>2 || corporateID.length()>12)
			throw new RuntimeException("Invalid input of AutoRateInfo");

		this.autoApplicableRate = autoApplicableRate;
		this.lateReturnHourlyRate = lateReturnHourlyRate;
		this.distanceRate = distanceRate;
		this.freeDistance = freeDistance;
		this.vehicleClassCode = vehicleClassCode;
		this.corporateID = corporateID;
	}

	/**
	 * Constructor for AutoRateInfo
	 * @param din DER representation of AutoRateInfo
	 */
	public AutoRateInfo(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(6);

		autoApplicableRate = new AutoApplicableRate(new DerInputStream(adv[0].toByteArray()));

		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_Sequence);
			lateReturnHourlyRate = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));
		}
		else
			lateReturnHourlyRate = null;

		if (!adv[2].equals(derNull)) {
			adv[2].resetTag(DerValue.tag_Sequence);
			distanceRate = new CurrencyAmount(new DerInputStream(adv[2].toByteArray()));
		}
		else
			distanceRate = null;

		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_Sequence);
			freeDistance = new Distance(new DerInputStream(adv[3].toByteArray()));
		}
		else
			freeDistance = null;

		if (!adv[4].equals(derNull)) {
			vehicleClassCode = adv[4].data.getDerValue().getPrintableString();
		}
		else
			vehicleClassCode = null;

		if (!adv[5].equals(derNull)) {
			corporateID = adv[5].data.getDerValue().getPrintableString();
		}
		else
			corporateID = null;
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		autoApplicableRate.encode (dout);

		//kalau lateReturnHourlyRate ada, tulis, kalau tidak, tulis null
		if (lateReturnHourlyRate != null) {
			midout = new DerOutputStream();
			lateReturnHourlyRate.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau distanceRate ada, tulis, kalau tidak, tulis null
		if (distanceRate != null) {
			midout = new DerOutputStream();
			distanceRate.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau freeDistance ada, tulis, kalau tidak, tulis null
		if (freeDistance != null) {
			midout = new DerOutputStream();
			freeDistance.encode (midout);
			dout.writeImplicit((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau vehicleClassCode ada, tulis, kalau tidak, tulis null
		if (vehicleClassCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (vehicleClassCode);
			dout.write((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau corporateID ada, tulis, kalau tidak, tulis null
		if (corporateID != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (corporateID);
			dout.write((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the autoApplicableRate
	 */
	public AutoApplicableRate getAutoApplicableRate() {
		return autoApplicableRate;
	}

	/**
	 * Method for get the rentalLocation
	 */
	public CurrencyAmount getLateReturnHourlyRate() {
		return lateReturnHourlyRate;
	}

	/**
	 * Method for get the distanceRate
	 */
	public CurrencyAmount getDistanceRate() {
		return distanceRate;
	}

	/**
	 * Method for get the freeDistance
	 */
	public Distance getFreeDistance() {
		return freeDistance;
	}

	/**
	 * Method for get the vehicleClassCode
	 */
	public String getVehicleClassCode() {
		return vehicleClassCode;
	}

	/**
	 * Method for get the corporateID
	 */
	public String getCorporateID() {
		return corporateID;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		s = s + "AutoRateInfo ::= AutoApplicableRate : " + autoApplicableRate + "\n";

		if(lateReturnHourlyRate != null)
			s = s + "AutoRateInfo ::= LateReturnHourlyRate : " + lateReturnHourlyRate + "\n";

		if(distanceRate != null)
			s = s + "AutoRateInfo ::= DistanceRate : " + distanceRate + "\n";

		if(freeDistance != null)
			s = s + "AutoRateInfo ::= FreeDistance : " + freeDistance + "\n";

		if(vehicleClassCode != null)
			s = s + "AutoRateInfo ::= VehicleClassCode : " + vehicleClassCode + "\n";

		if(corporateID != null)
			s = s + "AutoRateInfo ::= CorporateID : " + corporateID + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";

			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);
			System.out.println(autoRateInfo);

			DerOutputStream dout = new DerOutputStream();
			autoRateInfo.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			autoRateInfo = new AutoRateInfo (new DerInputStream(dout.toByteArray()));
			System.out.println(autoRateInfo);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
