package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	1812 BatchID ::= INTEGER (0..MAX)
 *	</pre>
 */

public class BatchID implements DerEncoder {
	
	private BigInt batchID;

	/**
	 * Constructor for BatchID
	 */
	public BatchID(BigInt batchID) throws Exception {
		if(batchID.toInt()<0 || batchID.toInt()>999)
			throw new RuntimeException("Invalid input of BatchID");

		this.batchID = batchID;
	}

	/**
	 * Constructor for BatchID
	 * @param din DER representation of BatchID
	 */
	public BatchID(DerInputStream din) throws IOException {
		DerValue adv = din.getDerValue();

		// isi batchID
		batchID = adv.getInteger();
	}

	/**
	* Method for get DER Value 
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		//tulis batchID
		dout.putInteger(batchID);
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the batchID
	 *
	 * @return batchID
	 */
	public BigInt getBatchID() {
		return batchID;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of BatchID object
	 */
	public String toString() {
		String s = "BatchID ::= " + batchID + "\n";
		return s;
	}

	static void main(String[] args) {
		try 
		{
			BatchID batchID = new BatchID (new BigInt(10));
			System.out.println(batchID);

			DerOutputStream dout = new DerOutputStream();
			batchID.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			batchID = new BatchID (new DerInputStream(dout.toByteArray()));
			System.out.println(batchID);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

