package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	1814 BatchSequenceNum ::= INTEGER (1..MAX)
 *	</pre>
 */

public class BatchSequenceNum implements DerEncoder {
	
	private BigInt batchSequenceNum;

	/**
	 * Constructor for BatchSequenceNum
	 */
	public BatchSequenceNum(BigInt batchSequenceNum) throws Exception {
		if(batchSequenceNum.toInt()<1 || batchSequenceNum.toInt()>999)
			throw new RuntimeException("Invalid input of BatchSequenceNum");

		this.batchSequenceNum = batchSequenceNum;
	}

	/**
	 * Constructor for BatchSequenceNum
	 * @param din DER representation of BatchSequenceNum
	 */
	public BatchSequenceNum(DerInputStream din) throws IOException {
		DerValue adv = din.getDerValue();

		// isi batchSequenceNum
		batchSequenceNum = adv.getInteger();
	}

	/**
	* Method for get DER Value 
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		//tulis batchSequenceNum
		dout.putInteger(batchSequenceNum);
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the batchSequenceNum
	 *
	 * @return batchSequenceNum
	 */
	public BigInt getBatchID() {
		return batchSequenceNum;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of BatchSequenceNum object
	 */
	public String toString() {
		String s = "BatchSequenceNum ::= " + batchSequenceNum + "\n";
		return s;
	}

	static void main(String[] args) {
		try 
		{
			BatchSequenceNum batchSequenceNum = new BatchSequenceNum (new BigInt(8));
			System.out.println(batchSequenceNum);

			DerOutputStream dout = new DerOutputStream();
			batchSequenceNum.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			batchSequenceNum = new BatchSequenceNum (new DerInputStream(dout.toByteArray()));
			System.out.println(batchSequenceNum);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

