package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3175 ChargeInfo ::= SEQUENCE {
 *	3176    totalFreightShippingAmount  [ 0] CurrencyAmount  OPTIONAL,
 *	3177    totalDutyTariffAmount       [ 1] CurrencyAmount  OPTIONAL,
 *	3178    dutyTariffReference         [ 2] EXPLICIT SETString { ub-reference=28 }  OPTIONAL,
 *	3179    totalNationalTaxAmount      [ 3] CurrencyAmount  OPTIONAL,
 *	3180    totalLocalTaxAmount         [ 4] CurrencyAmount  OPTIONAL,
 *	3181    totalOtherTaxAmount         [ 5] CurrencyAmount  OPTIONAL,
 *	3182    totalTaxAmount              [ 6] CurrencyAmount  OPTIONAL,
 *	3183    merchantTaxID               [ 7] EXPLICIT SETString { ub-taxID=10 }  OPTIONAL,
 *	3184    merchantDutyTariffRef       [ 8] EXPLICIT SETString { ub-reference=28 }  OPTIONAL,
 *	3185    customerDutyTariffRef       [ 9] EXPLICIT SETString { ub-reference=28 }  OPTIONAL,
 *	3186    summaryCommodityCode        [10] EXPLICIT SETString { ub-commCode=15 }  OPTIONAL,
 *	3187    merchantType                [11] EXPLICIT SETString { ub-merType=4 }  OPTIONAL
 *	3188 }
 *	</pre>
 */

public class ChargeInfo implements DerEncoder {
	
	private CurrencyAmount totalFreightShippingAmount;
	private CurrencyAmount totalDutyTariffAmount;
	private String dutyTariffReference;
	private CurrencyAmount totalNationalTaxAmount;
	private CurrencyAmount totalLocalTaxAmount;
	private CurrencyAmount totalOtherTaxAmount;
	private CurrencyAmount totalTaxAmount;
	private String merchantTaxID;
	private String merchantDutyTariffRef;
	private String customerDutyTariffRef;
	private String summaryCommodityCode;
	private String merchantType;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for ChargeInfo
	 */
	public ChargeInfo(CurrencyAmount totalFreightShippingAmount, CurrencyAmount totalDutyTariffAmount,
		String dutyTariffReference, CurrencyAmount totalNationalTaxAmount,
		CurrencyAmount totalLocalTaxAmount,	CurrencyAmount totalOtherTaxAmount,
		CurrencyAmount totalTaxAmount, String merchantTaxID,
		String merchantDutyTariffRef, String customerDutyTariffRef,
		String summaryCommodityCode, String merchantType) throws Exception {

		if(dutyTariffReference.length()>2 || merchantTaxID.length()>10
               || merchantDutyTariffRef.length() > 28 || customerDutyTariffRef.length()>28
               || summaryCommodityCode.length() > 15 || merchantType.length()>4)
			throw new RuntimeException("Invalid input of ChargeInfo");

		this.totalFreightShippingAmount = totalFreightShippingAmount;
		this.totalDutyTariffAmount = totalDutyTariffAmount;
		this.dutyTariffReference = dutyTariffReference;
		this.totalNationalTaxAmount = totalNationalTaxAmount;
		this.totalLocalTaxAmount = totalLocalTaxAmount;
		this.totalOtherTaxAmount = totalOtherTaxAmount;
		this.totalTaxAmount = totalTaxAmount;
		this.merchantTaxID = merchantTaxID;
		this.merchantDutyTariffRef = merchantDutyTariffRef;
		this.customerDutyTariffRef = customerDutyTariffRef;
		this.summaryCommodityCode = summaryCommodityCode;
		this.merchantType = merchantType;
	}

	/**
	 * Constructor for ChargeInfo
	 * @param din DER representation of ChargeInfo
	 */
	public ChargeInfo(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(12);

		//isi totalFreightShippingAmount
		if (!adv[0].equals(derNull)) {
			adv[0].resetTag(DerValue.tag_Sequence);
			totalFreightShippingAmount = new CurrencyAmount(new DerInputStream(adv[0].toByteArray()));
		}
		else
			totalFreightShippingAmount = null;

		//isi totalDutyTariffAmount
		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_Sequence);
			totalDutyTariffAmount = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));
		}
		else
			totalDutyTariffAmount = null;

		// isi dutyTariffReference
		if (!adv[2].equals(derNull)) {
			adv[2].resetTag(DerValue.tag_PrintableString);
			dutyTariffReference = adv[2].getPrintableString();
		}
		else
			dutyTariffReference = null;

		//isi totalNationalTaxAmount
		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_Sequence);
			totalNationalTaxAmount = new CurrencyAmount(new DerInputStream(adv[3].toByteArray()));
		}
		else
			totalNationalTaxAmount = null;

		//isi totalLocalTaxAmount
		if (!adv[4].equals(derNull)) {
			adv[4].resetTag(DerValue.tag_Sequence);
			totalLocalTaxAmount = new CurrencyAmount(new DerInputStream(adv[4].toByteArray()));
		}
		else
			totalLocalTaxAmount = null;

		//isi totalOtherTaxAmount
		if (!adv[5].equals(derNull)) {
			adv[5].resetTag(DerValue.tag_Sequence);
			totalOtherTaxAmount = new CurrencyAmount(new DerInputStream(adv[5].toByteArray()));
		}
		else
			totalOtherTaxAmount = null;

		//isi totalTaxAmount
		if (!adv[6].equals(derNull)) {
			adv[6].resetTag(DerValue.tag_Sequence);
			totalTaxAmount = new CurrencyAmount(new DerInputStream(adv[6].toByteArray()));
		}
		else
			totalTaxAmount = null;

		// isi merchantTaxID
		if (!adv[7].equals(derNull)) {
			adv[7].resetTag(DerValue.tag_PrintableString);
			merchantTaxID = adv[7].getPrintableString();
		}
		else
			merchantTaxID = null;

		// isi merchantDutyTariffRef
		if (!adv[8].equals(derNull)) {
			adv[8].resetTag(DerValue.tag_PrintableString);
			merchantDutyTariffRef = adv[8].getPrintableString();
		}
		else
			merchantDutyTariffRef = null;

		// isi customerDutyTariffRef
		if (!adv[9].equals(derNull)) {
			adv[9].resetTag(DerValue.tag_PrintableString);
			customerDutyTariffRef = adv[9].getPrintableString();
		}
		else
			customerDutyTariffRef = null;

		// isi summaryCommodityCode
		if (!adv[10].equals(derNull)) {
			adv[10].resetTag(DerValue.tag_PrintableString);
			summaryCommodityCode = adv[10].getPrintableString();
		}
		else
			summaryCommodityCode = null;

		// isi merchantType
		if (!adv[11].equals(derNull)) {
			adv[11].resetTag(DerValue.tag_PrintableString);
			merchantType = adv[11].getPrintableString();
		}
		else
			merchantType = null;
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		//kalau totalFreightShippingAmount ada, tulis, kalau tidak, tulis null
		if (totalFreightShippingAmount != null) {
			midout = new DerOutputStream();
			totalFreightShippingAmount.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau totalDutyTariffAmount ada, tulis, kalau tidak, tulis null
		if (totalDutyTariffAmount != null) {
			midout = new DerOutputStream();
			totalDutyTariffAmount.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau dutyTariffReference ada, tulis, kalau tidak, tulis null
		if (dutyTariffReference != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (dutyTariffReference);
			dout.writeImplicit((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau totalNationalTaxAmount ada, tulis, kalau tidak, tulis null
		if (totalNationalTaxAmount != null) {
			midout = new DerOutputStream();
			totalNationalTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau totalLocalTaxAmount ada, tulis, kalau tidak, tulis null
		if (totalLocalTaxAmount != null) {
			midout = new DerOutputStream();
			totalLocalTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//kalau totalOtherTaxAmount ada, tulis, kalau tidak, tulis null
		if (totalOtherTaxAmount != null) {
			midout = new DerOutputStream();
			totalOtherTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xA5, midout);
		}
		else
			dout.putNull();

		//kalau totalTaxAmount ada, tulis, kalau tidak, tulis null
		if (totalTaxAmount != null) {
			midout = new DerOutputStream();
			totalTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xA6, midout);
		}
		else
			dout.putNull();

		//kalau merchantTaxID ada, tulis, kalau tidak, tulis null
		if (merchantTaxID != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (merchantTaxID);
			dout.writeImplicit((byte)0xA7, midout);
		}
		else
			dout.putNull();

		//kalau merchantDutyTariffRef ada, tulis, kalau tidak, tulis null
		if (merchantDutyTariffRef != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (merchantDutyTariffRef);
			dout.writeImplicit((byte)0xA8, midout);
		}
		else
			dout.putNull();

		//kalau customerDutyTariffRef ada, tulis, kalau tidak, tulis null
		if (customerDutyTariffRef != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (customerDutyTariffRef);
			dout.writeImplicit((byte)0xA9, midout);
		}
		else
			dout.putNull();

		//kalau summaryCommodityCode ada, tulis, kalau tidak, tulis null
		if (summaryCommodityCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (summaryCommodityCode);
			dout.writeImplicit((byte)0xAA, midout);
		}
		else
			dout.putNull();

		//kalau merchantType ada, tulis, kalau tidak, tulis null
		if (merchantType != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (merchantType);
			dout.writeImplicit((byte)0xAB, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the totalFreightShippingAmount
	 */
	public CurrencyAmount getTotalFreightShippingAmount() {
		return totalFreightShippingAmount;
	}

	/**
	 * Method for get the totalDutyTariffAmount
	 */
	public CurrencyAmount getTotalDutyTariffAmount() {
		return totalDutyTariffAmount;
	}

	/**
	 * Method for get the dutyTariffReference
	 */
	public String getDutyTariffReference() {
		return dutyTariffReference;
	}

	/**
	 * Method for get the totalNationalTaxAmount
	 */
	public CurrencyAmount getTotalNationalTaxAmount() {
		return totalNationalTaxAmount;
	}

	/**
	 * Method for get the totalLocalTaxAmount
	 */
	public CurrencyAmount getTotalLocalTaxAmount() {
		return totalLocalTaxAmount;
	}

	/**
	 * Method for get the totalOtherTaxAmount
	 */
	public CurrencyAmount getTotalOtherTaxAmount() {
		return totalOtherTaxAmount;
	}

	/**
	 * Method for get the totalTaxAmount
	 */
	public CurrencyAmount getTotalTaxAmount() {
		return totalTaxAmount;
	}

	/**
	 * Method for get the merchantTaxID
	 */
	public String getMerchantTaxID() {
		return merchantTaxID;
	}

	/**
	 * Method for get the merchantDutyTariffRef
	 */
	public String getMerchantDutyTariffRef() {
		return merchantDutyTariffRef;
	}

	/**
	 * Method for get the customerDutyTariffRef
	 */
	public String getCustomerDutyTariffRef() {
		return customerDutyTariffRef;
	}

	/**
	 * Method for get the summaryCommodityCode
	 */
	public String getSummaryCommodityCode() {
		return summaryCommodityCode;
	}

	/**
	 * Method for get the merchantType
	 */
	public String getMerchantType() {
		return merchantType;
	}


	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if(totalFreightShippingAmount != null)
			s = s + "ChargeInfo ::= TotalFreightShippingAmount : " + totalFreightShippingAmount + "\n";

		if(totalDutyTariffAmount != null)
			s = s + "ChargeInfo ::= TotalDutyTariffAmount : " + totalDutyTariffAmount + "\n";

		if(dutyTariffReference != null)
			s = s + "ChargeInfo ::= DutyTariffReference : " + dutyTariffReference + "\n";

		if(totalNationalTaxAmount != null)
			s = s + "ChargeInfo ::= TotalNationalTaxAmount : " + totalNationalTaxAmount + "\n";

		if(totalLocalTaxAmount != null)
			s = s + "ChargeInfo ::= TotalLocalTaxAmount : " + totalLocalTaxAmount + "\n";

		if(totalOtherTaxAmount != null)
			s = s + "ChargeInfo ::= TotalOtherTaxAmount : " + totalOtherTaxAmount + "\n";

		if(totalTaxAmount != null)
			s = s + "ChargeInfo ::= TotalTaxAmount : " + totalTaxAmount + "\n";

		if(merchantTaxID != null)
			s = s + "ChargeInfo ::= MerchantTaxID : " + merchantTaxID + "\n";

		if(merchantDutyTariffRef != null)
			s = s + "ChargeInfo ::= MerchantDutyTariffRef : " + merchantDutyTariffRef + "\n";

		if(customerDutyTariffRef != null)
			s = s + "ChargeInfo ::= CustomerDutyTariffRef : " + customerDutyTariffRef + "\n";

		if(summaryCommodityCode != null)
			s = s + "ChargeInfo ::= SummaryCommodityCode : " + summaryCommodityCode + "\n";

		if(merchantType != null)
			s = s + "ChargeInfo ::= MerchantType : " + merchantType + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CurrencyAmount totalFreightShippingAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalDutyTariffAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String dutyTariffReference = "A";
			CurrencyAmount totalNationalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalLocalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalOtherTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String merchantTaxID = "B";
			String merchantDutyTariffRef = "C";
			String customerDutyTariffRef = "D";
			String summaryCommodityCode = "E";
			String merchantType = "F";
			
			ChargeInfo chargeInfo = new ChargeInfo (totalFreightShippingAmount, totalDutyTariffAmount, 
				dutyTariffReference, totalNationalTaxAmount, totalLocalTaxAmount,
				totalOtherTaxAmount, totalTaxAmount, merchantTaxID, merchantDutyTariffRef,
				customerDutyTariffRef, summaryCommodityCode, merchantType);
			System.out.println(chargeInfo);

			DerOutputStream dout = new DerOutputStream();
			chargeInfo.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			chargeInfo = new ChargeInfo (new DerInputStream(dout.toByteArray()));
			System.out.println(chargeInfo);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
