package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
import digsec.set.payment.component.CountryCode;

/**
 *	<pre>
 *	3167 CommercialCardData ::= SEQUENCE {
 *	3168    chargeInfo        [0] ChargeInfo  OPTIONAL,
 *	3169    merchantLocation  [1] Location  OPTIONAL,
 *	3170    shipFrom          [2] Location  OPTIONAL,
 *	3171    shipTo            [3] Location  OPTIONAL,
 *	3172    itemSeq           [4] ItemSeq  OPTIONAL
 *	3173 }
 *
 *	3190 ItemSeq ::= SEQUENCE SIZE(1..ub-items=999) OF Item
 *
 *	</pre>
 */

public class CommercialCardData implements DerEncoder {

	private ChargeInfo chargeInfo;
	private Location merchantLocation;
	private Location shipFrom;
	private Location shipTo;
	private ItemSeq itemSeq;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for CommercialCardData
	 */
	public CommercialCardData(ChargeInfo chargeInfo, Location merchantLocation,
		Location shipFrom, Location shipTo, ItemSeq itemSeq) throws Exception {

		this.chargeInfo = chargeInfo;
		this.merchantLocation = merchantLocation;
		this.shipFrom = shipFrom;
		this.shipTo = shipTo;
		this.itemSeq = itemSeq;
	}

	/**
	 * Constructor for CommercialCardData
	 * @param din DER representation of CommercialCardData
	 */
	public CommercialCardData(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(5);

		if (!adv[0].equals(derNull)) {
			adv[0].resetTag(DerValue.tag_Sequence);
			chargeInfo = new ChargeInfo(new DerInputStream(adv[0].toByteArray()));
		}
		else
			chargeInfo = null;

		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_Sequence);
			merchantLocation = new Location(new DerInputStream(adv[1].toByteArray()));
		}
		else
			merchantLocation = null;

		if (!adv[2].equals(derNull)) {
			adv[2].resetTag(DerValue.tag_Sequence);
			shipFrom = new Location(new DerInputStream(adv[2].toByteArray()));
		}
		else
			shipFrom = null;

		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_Sequence);
			shipTo = new Location(new DerInputStream(adv[3].toByteArray()));
		}
		else
			shipTo = null;

		if (!adv[4].equals(derNull)) {
			adv[4].resetTag(DerValue.tag_Sequence);
			itemSeq = new ItemSeq(new DerInputStream(adv[4].toByteArray()));
		}
		else
			itemSeq = null;

	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		//kalau chargeInfo ada, tulis, kalau tidak, tulis null
		if (chargeInfo != null) {
			midout = new DerOutputStream();
			chargeInfo.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau merchantLocation ada, tulis, kalau tidak, tulis null
		if (merchantLocation != null) {
			midout = new DerOutputStream();
			merchantLocation.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau shipFrom ada, tulis, kalau tidak, tulis null
		if (shipFrom != null) {
			midout = new DerOutputStream();
			shipFrom.encode (midout);
			dout.writeImplicit((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau shipTo ada, tulis, kalau tidak, tulis null
		if (shipTo != null) {
			midout = new DerOutputStream();
			shipTo.encode (midout);
			dout.writeImplicit((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau itemSeq ada, tulis, kalau tidak, tulis null
		if (itemSeq != null) {
			midout = new DerOutputStream();
			itemSeq.encode (midout);
			dout.writeImplicit((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the chargeInfo
	 */
	public ChargeInfo getChargeInfo() {
		return chargeInfo;
	}

	/**
	 * Method for get the merchantLocation
	 */
	public Location getmerchantLocation() {
		return merchantLocation;
	}

	/**
	 * Method for get the shipFrom
	 */
	public Location getShipFrom() {
		return shipFrom;
	}

	/**
	 * Method for get the shipTo
	 */
	public Location getShipTo() {
		return shipTo;
	}

	/**
	 * Method for get the itemSeq
	 */
	public ItemSeq getItemSeq() {
		return itemSeq;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if(chargeInfo != null)
			s = s + "CommercialCardData ::= chargeInfo : " + chargeInfo + "\n";

		if(merchantLocation != null)
			s = s + "CommercialCardData ::= merchantLocation : " + merchantLocation + "\n";

		if(shipFrom != null)
			s = s + "CommercialCardData ::= shipFrom : " + shipFrom + "\n";

		if(shipTo != null)
			s = s + "CommercialCardData ::= shipTo : " + shipTo + "\n";

		if(itemSeq != null)
			s = s + "CommercialCardData ::= itemSeq : " + itemSeq + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CurrencyAmount totalFreightShippingAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalDutyTariffAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String dutyTariffReference = "A";
			CurrencyAmount totalNationalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalLocalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalOtherTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String merchantTaxID = "B";
			String merchantDutyTariffRef = "C";
			String customerDutyTariffRef = "D";
			String summaryCommodityCode = "E";
			String merchantType = "F";
			
			ChargeInfo chargeInfo = new ChargeInfo (totalFreightShippingAmount, totalDutyTariffAmount, 
				dutyTariffReference, totalNationalTaxAmount, totalLocalTaxAmount,
				totalOtherTaxAmount, totalTaxAmount, merchantTaxID, merchantDutyTariffRef,
				customerDutyTariffRef, summaryCommodityCode, merchantType);

			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			
			Location merchantLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipFrom = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipTo = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			BigInt quantity = new BigInt(1);
			String unitOfMeasureCode = "AA";
			String descriptor = "DD";
			String commodityCode = "CC";
			String productCode = "AB";
			CurrencyAmount unitCost = new CurrencyAmount(new BigInt(110),new BigInt(150),new BigInt(2));
			CurrencyAmount netCost  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			boolean discountInd = false;
			CurrencyAmount discountAmount = new CurrencyAmount(new BigInt(130),new BigInt(150),new BigInt(2));
			CurrencyAmount nationalTaxAmount = new CurrencyAmount(new BigInt(140),new BigInt(150),new BigInt(2));
			BigInt nationalTaxRate = new BigInt(2);
			String nationalTaxType = "B";
			CurrencyAmount localTaxAmount = new CurrencyAmount(new BigInt(150),new BigInt(150),new BigInt(2));
			CurrencyAmount otherTaxAmount = new CurrencyAmount(new BigInt(160),new BigInt(150),new BigInt(2));
			CurrencyAmount itemTotalCost = new CurrencyAmount(new BigInt(170),new BigInt(150),new BigInt(2));

			Item item = new Item (quantity, unitOfMeasureCode, descriptor, commodityCode,
				productCode, unitCost, netCost, discountInd, discountAmount,
				nationalTaxAmount, nationalTaxRate, nationalTaxType,
				localTaxAmount, otherTaxAmount, itemTotalCost);

			Item[] itemArray = new Item[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = item;

			ItemSeq itemSeq = new ItemSeq(itemArray);

			CommercialCardData commercialCardData = new CommercialCardData (chargeInfo,
				merchantLocation, shipFrom, shipTo, itemSeq);
			System.out.println(commercialCardData);

			DerOutputStream dout = new DerOutputStream();
			commercialCardData.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			commercialCardData = new CommercialCardData (new DerInputStream(dout.toByteArray()));
			System.out.println(commercialCardData);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
