package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	267 DateTime :== SEQUENCE {
 *	268 	date		Date,
 *	269	timeInd	BOOLEAN DEFAULT FALSE
 *	270 }
 *	</pre>
 */

public class DateTime implements DerEncoder {

	private Date date;
	private boolean timeInd;

	/**
	 * Constructor for DateTime
	 */
	public DateTime(Date date, boolean timeInd) throws Exception {
		this.date = date;
		this.timeInd = timeInd;
	}

	/**
	 * Constructor for DateTime
	 * @param din DER representation of DateTime
	 */
	public DateTime(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(2);

		date = (new DerInputStream(adv[0].toByteArray())).getGeneralizedTime();
		timeInd = adv[1].getBoolean();
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putGeneralizedTime(date);
		dout.putBoolean (timeInd);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the date
	 */
	public Date getDate() {
		return date;
	}

	/**
	 * Method for get the timeInd
	 */
	public boolean getTimeInd() {
		return timeInd;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		s = s + "DateTime ::= Date : " + date + "\n";
		s = s + "DateTime ::= TimeInd : " + timeInd + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			Date date = new Date();
			boolean timeInd = true;

			DateTime dateTime = new DateTime (date, timeInd);
			System.out.println(dateTime);

			DerOutputStream dout = new DerOutputStream();
			dateTime.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			dateTime = new DateTime (new DerInputStream(dout.toByteArray()));
			System.out.println(dateTime);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
