package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	1890 Distance ::= SEQUENCE {
 *	1891    scale   DistanceScale,
 *	1892    dist  INTEGER (0..MAX)
 *	1893 }
 *	</pre>
 */

public class Distance implements DerEncoder {
	
	private DistanceScale scale;
	private BigInt dist;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for Distance
	 */
	public Distance(DistanceScale scale, BigInt dist) throws Exception {
		this.scale = scale;
		this.dist = dist;
	}

	/**
	 * Constructor for Distance
	 * @param din DER representation of Distance
	 */
	public Distance(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(2);

		// isi scale
		scale = new DistanceScale(new DerInputStream(adv[0].toByteArray()));

		// isi dist
		dist = adv[1].getInteger();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		//tulis scale
		scale.encode (dout);

		//tulis dist
		dout.putInteger(dist);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the scale
	 *
	 * @return scale
	 */
	public DistanceScale getScale() {
		return scale;
	}


	/**
	 * Method for get the dist
	 *
	 * @return dist
	 */
	public BigInt getDist() {
		return dist;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of Distance object
	 */
	public String toString() {
		String s = "Distance ::= Scale : " + scale + "\n";

		s = s + "Distance ::= Dist : " + dist + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			
			Distance distance = new Distance (scale, dist);
			System.out.println(distance);

			DerOutputStream dout = new DerOutputStream();
			distance.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			distance = new Distance (new DerInputStream(dout.toByteArray()));
			System.out.println(distance);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

