package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3224 DistanceScale ::= ENUMERATED {
 *	3225    miles   (0),
 *	3226    kilometers  (1)
 *	3227 }
 *	</pre>
 */

public class DistanceScale implements DerEncoder {
	
	public static BigInt miles = new BigInt(0);
	public static BigInt kilometers = new BigInt(1);
	private BigInt value;

	/**
	 * Constructor for DistanceScale
	 */
	public DistanceScale(BigInt value) throws Exception {
		if(value.toInt()<0 || value.toInt()>1)
			throw new RuntimeException("Invalid input of DistanceScale");

		this.value = value;
	}

	/**
	 * Constructor for DistanceScale
	 * @param din DER representation of DistanceScale
	 */
	public DistanceScale(DerInputStream din) throws IOException {
		this.value = din.getEnumerated();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putEnumerated (value);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public BigInt getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of DistanceScale object
	 */
	public String toString() {
		String s = "";

		if (value.equals(miles))
			s = s + "DistanceScale ::= miles \n";
		else
			s = s + "DistanceScale ::= kilometers \n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt value = new BigInt(1);
			DistanceScale distanceScale = new DistanceScale (value);
			System.out.println(distanceScale);

			DerOutputStream dout = new DerOutputStream();
			distanceScale.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			distanceScale = new DistanceScale (new DerInputStream(dout.toByteArray()));
			System.out.println(distanceScale);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

