package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3278 HotelRateInfo ::= SEQUENCE {
 *	3279    dailyRoomRate  CurrencyAmount,
 *	3280    dailyTaxRate   CurrencyAmount  OPTIONAL
 *	3281 }
 *	</pre>
 */

public class HotelRateInfo implements DerEncoder {
	
	private CurrencyAmount dailyRoomRate;
	private CurrencyAmount dailyTaxRate;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for HotelRateInfo
	 */
	public HotelRateInfo(CurrencyAmount dailyRoomRate, CurrencyAmount dailyTaxRate) throws Exception {
		this.dailyRoomRate = dailyRoomRate;
		this.dailyTaxRate = dailyTaxRate;
	}

	/**
	 * Constructor for HotelRateInfo
	 * @param din DER representation of HotelRateInfo
	 */
	public HotelRateInfo(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(2);

		// isi dailyRoomRate
		dailyRoomRate = new CurrencyAmount(new DerInputStream(adv[0].toByteArray()));


		// isi dailyTaxRate
		if (!adv[1].equals(derNull))
			dailyTaxRate = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));
		else
			dailyTaxRate = null;
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dailyRoomRate.encode (dout);

		//kalau dailyTaxRate ada, tulis, kalau tidak, tulis null
		if (dailyTaxRate != null)
			dailyTaxRate.encode (dout);
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the dailyRoomRate
	 *
	 * @return dailyRoomRate
	 */
	public CurrencyAmount getDailyRoomRate() {
		return dailyRoomRate;
	}


	/**
	 * Method for get the dailyTaxRate
	 *
	 * @return dailyTaxRate
	 */
	public CurrencyAmount getDailyTaxRate() {
		return dailyTaxRate;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "HotelRateInfo ::= DailyRoomRate : " + dailyRoomRate + "\n";

		if(dailyTaxRate != null)
			s = s + "HotalRateInfo ::= DailyTaxRate : " + dailyTaxRate + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			CurrencyAmount dailyRoomRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount dailyTaxRate = new CurrencyAmount(new BigInt(120),new BigInt(100),new BigInt(1));
			
			HotelRateInfo hotelRateInfo = new HotelRateInfo (dailyRoomRate, dailyTaxRate);
			System.out.println(hotelRateInfo);

			DerOutputStream dout = new DerOutputStream();
			hotelRateInfo.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			hotelRateInfo = new HotelRateInfo (new DerInputStream(dout.toByteArray()));
			System.out.println(hotelRateInfo);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
