package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3190 ItemSeq ::= SEQUENCE SIZE(1..ub-items) OF Item
 *
 *	3192 Item ::= SEQUENCE {
 *	3193    quantity           INTEGER (1..MAX) DEFAULT 1,
 *	3194    unitOfMeasureCode  [ 0] EXPLICIT SETString { ub-unitMeasure=12 }  OPTIONAL,
 *	3195    descriptor         SETString { ub-description  },
 *	3196    commodityCode      [ 1] EXPLICIT SETString { ub-commCode=15 }  OPTIONAL,
 *	3197    productCode        [ 2] EXPLICIT SETString { ub-productCode=12 }  OPTIONAL,
 *	3198    unitCost           [ 3] CurrencyAmount  OPTIONAL,
 *	3199    netCost            [ 4] CurrencyAmount  OPTIONAL,
 *	3200    discountInd        BOOLEAN DEFAULT FALSE,
 *	3201    discountAmount     [ 5] CurrencyAmount  OPTIONAL,
 *	3202    nationalTaxAmount  [ 6] CurrencyAmount  OPTIONAL,
 *	3203    nationalTaxRate    [ 7] FloatingPoint  OPTIONAL,
 *	3204    nationalTaxType    [ 8] EXPLICIT SETString { ub-taxType=4 }  OPTIONAL,
 *	3205    localTaxAmount     [ 9] CurrencyAmount  OPTIONAL,
 *	3206    otherTaxAmount     [10] CurrencyAmount  OPTIONAL,
 *	3207    itemTotalCost      CurrencyAmount
 *	3208 }
 *	</pre>
 */

public class Item implements DerEncoder {
	
	private BigInt quantity = new BigInt(1);
	private String unitOfMeasureCode;
	private String descriptor;
	private String commodityCode;
	private String productCode;
	private CurrencyAmount unitCost;
	private CurrencyAmount netCost;
	private boolean discountInd = false;
	private CurrencyAmount discountAmount;
	private CurrencyAmount nationalTaxAmount;
	private BigInt nationalTaxRate;
	private String nationalTaxType;
	private CurrencyAmount localTaxAmount;
	private CurrencyAmount otherTaxAmount;
	private CurrencyAmount itemTotalCost;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for Item
	 */
	public Item(BigInt quantity, String unitOfMeasureCode, String descriptor, String commodityCode,
		String productCode, CurrencyAmount unitCost, CurrencyAmount netCost, boolean discountInd,
		CurrencyAmount discountAmount, CurrencyAmount nationalTaxAmount, BigInt nationalTaxRate,
		String nationalTaxType, CurrencyAmount localTaxAmount, CurrencyAmount otherTaxAmount,
		CurrencyAmount itemTotalCost) throws Exception {

		if(unitOfMeasureCode.length()>12 || commodityCode.length()>15
               || productCode.length() > 12 || nationalTaxType.length()>4)
			throw new RuntimeException("Invalid input of Item");

		this.quantity = quantity;
		this.unitOfMeasureCode = unitOfMeasureCode;
		this.descriptor = descriptor;
		this.commodityCode = commodityCode;
		this.productCode = productCode;
		this.unitCost = unitCost;
		this.netCost = netCost;
		this.discountInd = discountInd;
		this.discountAmount = discountAmount;
		this.nationalTaxAmount = nationalTaxAmount;
		this.nationalTaxRate = nationalTaxRate;
		this.nationalTaxType = nationalTaxType;
		this.localTaxAmount = localTaxAmount;
		this.otherTaxAmount = otherTaxAmount;
		this.itemTotalCost = itemTotalCost;
	}

	/**
	 * Constructor for Item
	 * @param din DER representation of Item
	 */
	public Item(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(15);

		//isi quantity
		quantity = adv[0].getInteger();

		//isi unitOfMeasureCode
		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_PrintableString);
			unitOfMeasureCode = adv[1].getPrintableString();
		}
		else
			unitOfMeasureCode = null;

		// isi descriptor
		descriptor = adv[2].getPrintableString();

		//isi commodityCode
		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_PrintableString);
			commodityCode = adv[3].getPrintableString();
		}
		else
			commodityCode = null;

		//isi productCode
		if (!adv[4].equals(derNull)) {
			adv[4].resetTag(DerValue.tag_PrintableString);
			productCode = adv[4].getPrintableString();
		}
		else
			productCode = null;

		//isi unitCost
		if (!adv[5].equals(derNull)) {
			adv[5].resetTag(DerValue.tag_Sequence);
			unitCost = new CurrencyAmount(new DerInputStream(adv[5].toByteArray()));
		}
		else
			unitCost = null;

		//isi netCost
		if (!adv[6].equals(derNull)) {
			adv[6].resetTag(DerValue.tag_Sequence);
			netCost = new CurrencyAmount(new DerInputStream(adv[6].toByteArray()));
		}
		else
			netCost = null;

		// isi discountInd
		discountInd = adv[7].getBoolean();

		// isi discountAmount
		if (!adv[8].equals(derNull)) {
			adv[8].resetTag(DerValue.tag_Sequence);
			discountAmount = new CurrencyAmount(new DerInputStream(adv[8].toByteArray()));
		}
		else
			discountAmount = null;

		// isi nationalTaxAmount
		if (!adv[9].equals(derNull)) {
			adv[9].resetTag(DerValue.tag_Sequence);
			nationalTaxAmount = new CurrencyAmount(new DerInputStream(adv[9].toByteArray()));
		}
		else
			nationalTaxAmount = null;

		// isi nationalTaxRate
		if (!adv[10].equals(derNull)) {
			adv[10].resetTag(DerValue.tag_Integer);
			nationalTaxRate = adv[10].getInteger();
		}
		else
			nationalTaxRate = null;

		// isi nationalTaxType
		if (!adv[11].equals(derNull)) {
			adv[11].resetTag(DerValue.tag_PrintableString);
			nationalTaxType = adv[11].getPrintableString();
		}
		else
			nationalTaxType = null;

		// isi localTaxAmount
		if (!adv[12].equals(derNull)) {
			adv[12].resetTag(DerValue.tag_Sequence);
			localTaxAmount = new CurrencyAmount(new DerInputStream(adv[12].toByteArray()));
		}
		else
			localTaxAmount = null;

		// isi otherTaxAmount
		if (!adv[13].equals(derNull)) {
			adv[13].resetTag(DerValue.tag_Sequence);
			otherTaxAmount = new CurrencyAmount(new DerInputStream(adv[13].toByteArray()));
		}
		else
			otherTaxAmount = null;

		// isi itemTotalCost
		if (!adv[14].equals(derNull)) {
			adv[14].resetTag(DerValue.tag_Sequence);
			itemTotalCost = new CurrencyAmount(new DerInputStream(adv[14].toByteArray()));
		}
		else
			itemTotalCost = null;
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		//tulis quantity
		dout.putInteger (quantity);

		//kalau unitOfMeasureCode ada, tulis, kalau tidak, tulis null
		if (unitOfMeasureCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (unitOfMeasureCode);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau descriptor ada, tulis, kalau tidak, tulis null
		dout.putPrintableString (descriptor);

		//kalau commodityCode ada, tulis, kalau tidak, tulis null
		if (commodityCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (commodityCode);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau productCode ada, tulis, kalau tidak, tulis null
		if (productCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (productCode);
			dout.writeImplicit((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau unitCost ada, tulis, kalau tidak, tulis null
		if (unitCost != null) {
			midout = new DerOutputStream();
			unitCost.encode (midout);
			dout.writeImplicit((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau netCost ada, tulis, kalau tidak, tulis null
		if (netCost != null) {
			midout = new DerOutputStream();
			netCost.encode (midout);
			dout.writeImplicit((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//kalau discountInd ada, tulis, kalau tidak, tulis null
		dout.putBoolean (discountInd);

		//kalau discountAmount ada, tulis, kalau tidak, tulis null
		if (discountAmount != null) {
			midout = new DerOutputStream();
			discountAmount.encode (midout);
			dout.writeImplicit((byte)0xA5, midout);
		}
		else
			dout.putNull();

		//kalau nationalTaxAmount ada, tulis, kalau tidak, tulis null
		if (nationalTaxAmount != null) {
			midout = new DerOutputStream();
			nationalTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xA6, midout);
		}
		else
			dout.putNull();

		//kalau nationalTaxRate ada, tulis, kalau tidak, tulis null
		if (nationalTaxRate != null) {
			midout = new DerOutputStream();
			midout.putInteger (nationalTaxRate);
			dout.writeImplicit((byte)0xA7, midout);
		}
		else
			dout.putNull();

		//kalau nationalTaxType ada, tulis, kalau tidak, tulis null
		if (nationalTaxType != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (nationalTaxType);
			dout.writeImplicit((byte)0xA8, midout);
		}
		else
			dout.putNull();

		//kalau localTaxAmount ada, tulis, kalau tidak, tulis null
		if (localTaxAmount != null) {
			midout = new DerOutputStream();
			localTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xA9, midout);
		}
		else
			dout.putNull();

		//kalau otherTaxAmount ada, tulis, kalau tidak, tulis null
		if (otherTaxAmount != null) {
			midout = new DerOutputStream();
			otherTaxAmount.encode (midout);
			dout.writeImplicit((byte)0xAA, midout);
		}
		else
			dout.putNull();

		//kalau itemTotalCost ada, tulis, kalau tidak, tulis null
		itemTotalCost.encode (dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the quantity
	 */
	public BigInt getQuantity() {
		return quantity;
	}

	/**
	 * Method for get the unitOfMeasureCode
	 */
	public String getUnitOfMeasureCode() {
		return unitOfMeasureCode;
	}

	/**
	 * Method for get the descriptor
	 */
	public String getDescriptor() {
		return descriptor;
	}

	/**
	 * Method for get the commodityCode
	 */
	public String getCommodityCode() {
		return commodityCode;
	}

	/**
	 * Method for get the productCode
	 */
	public String getProductCode() {
		return productCode;
	}

	/**
	 * Method for get the unitCost
	 */
	public CurrencyAmount getUnitCost() {
		return unitCost;
	}

	/**
	 * Method for get the netCost
	 */
	public CurrencyAmount getNetCost() {
		return netCost;
	}

	/**
	 * Method for get the merchantTaxID
	 */
	public boolean getDiscountInd() {
		return discountInd;
	}

	/**
	 * Method for get the discountAmount
	 */
	public CurrencyAmount getDiscountAmount() {
		return discountAmount;
	}

	/**
	 * Method for get the nationalTaxAmount
	 */
	public CurrencyAmount getNationalTaxAmount() {
		return nationalTaxAmount;
	}

	/**
	 * Method for get the nationalTaxRate
	 */
	public BigInt getNationalTaxRate() {
		return nationalTaxRate;
	}

	/**
	 * Method for get the nationalTaxType
	 */
	public String getNationalTaxType() {
		return nationalTaxType;
	}

	/**
	 * Method for get the localTaxAmount
	 */
	public CurrencyAmount getLocalTaxAmount() {
		return localTaxAmount;
	}

	/**
	 * Method for get the otherTaxAmount
	 */
	public CurrencyAmount getOtherTaxAmount() {
		return otherTaxAmount;
	}

	/**
	 * Method for get the itemTotalCost
	 */
	public CurrencyAmount getItemTotalCost() {
		return itemTotalCost;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		s = s + "Item ::= Quantity : " + quantity + "\n";

		if(unitOfMeasureCode != null)
			s = s + "Item ::= UnitOfMeasureCode : " + unitOfMeasureCode + "\n";

		s = s + "Item ::= Descriptor : " + descriptor + "\n";

		if(commodityCode != null)
			s = s + "Item ::= CommodityCode : " + commodityCode + "\n";

		if(productCode != null)
			s = s + "Item ::= ProductCode : " + productCode + "\n";

		if(unitCost != null)
			s = s + "Item ::= UnitCost : " + unitCost + "\n";

		if(netCost != null)
			s = s + "Item ::= NetCost : " + netCost + "\n";

		s = s + "Item ::= DiscountInd : " + discountInd + "\n";

		if(discountAmount != null)
			s = s + "Item ::= DiscountAmount : " + discountAmount + "\n";

		if(nationalTaxAmount != null)
			s = s + "Item ::= NationalTaxAmount : " + nationalTaxAmount + "\n";

		if(nationalTaxRate != null)
			s = s + "Item ::= NationalTaxRate : " + nationalTaxRate + "\n";

		if(nationalTaxType != null)
			s = s + "Item ::= NationalTaxType : " + nationalTaxType + "\n";

		if(localTaxAmount != null)
			s = s + "Item ::= LocalTaxAmount : " + localTaxAmount + "\n";

		if(otherTaxAmount != null)
			s = s + "Item ::= OtherTaxAmount : " + otherTaxAmount + "\n";

		s = s + "Item ::= ItemTotalCost : " + itemTotalCost + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt quantity = new BigInt(1);
			String unitOfMeasureCode = "AA";
			String descriptor = "DD";
			String commodityCode = "CC";
			String productCode = "AB";
			CurrencyAmount unitCost = new CurrencyAmount(new BigInt(110),new BigInt(150),new BigInt(2));
			CurrencyAmount netCost  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			boolean discountInd = false;
			CurrencyAmount discountAmount = new CurrencyAmount(new BigInt(130),new BigInt(150),new BigInt(2));
			CurrencyAmount nationalTaxAmount = new CurrencyAmount(new BigInt(140),new BigInt(150),new BigInt(2));
			BigInt nationalTaxRate = new BigInt(2);
			String nationalTaxType = "B";
			CurrencyAmount localTaxAmount = new CurrencyAmount(new BigInt(150),new BigInt(150),new BigInt(2));
			CurrencyAmount otherTaxAmount = new CurrencyAmount(new BigInt(160),new BigInt(150),new BigInt(2));
			CurrencyAmount itemTotalCost = new CurrencyAmount(new BigInt(170),new BigInt(150),new BigInt(2));

			Item item = new Item (quantity, unitOfMeasureCode, descriptor, commodityCode,
				productCode, unitCost, netCost, discountInd, discountAmount,
				nationalTaxAmount, nationalTaxRate, nationalTaxType,
				localTaxAmount, otherTaxAmount, itemTotalCost);
			System.out.println(item);

			DerOutputStream dout = new DerOutputStream();
			item.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			item = new Item (new DerInputStream(dout.toByteArray()));
			System.out.println(item);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
