package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	3314 ItemSeq ::= SEQUENCE SIZE(1..16) OF Item
 *	</pre>
 */

public class ItemSeq implements DerEncoder {

	private Item[] item;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for ItemSeq
	 */
	public ItemSeq(Item[] item) throws Exception {

		if(item.length>16)
			throw new RuntimeException("Invalid length of ItemSeq");
		this.item = item;
	}

	/**
	 * Constructor for ItemSeq
	 * @param din DER representation of ItemSeq
	 */
	public ItemSeq(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(16);

		item = new Item[adv.length];
		for (int i=0; i<adv.length; i++)
			item[i] = new Item(new DerInputStream(adv[i].toByteArray()));
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		for (int i=0; i<this.item.length; i++)
			item[i].encode(dout);

		//autoApplicableRate.encode (dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the Item
	 */
	public Item[] getItem() {
		return item;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		s = s + "ItemSeq ::= \n";

		if(item != null)
			for (int i=0; i<item.length; i++)
				s = s + "Item[" + i +"]: " + item[i] + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt quantity = new BigInt(1);
			String unitOfMeasureCode = "AA";
			String descriptor = "DD";
			String commodityCode = "CC";
			String productCode = "AB";
			CurrencyAmount unitCost = new CurrencyAmount(new BigInt(110),new BigInt(150),new BigInt(2));
			CurrencyAmount netCost  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			boolean discountInd = false;
			CurrencyAmount discountAmount = new CurrencyAmount(new BigInt(130),new BigInt(150),new BigInt(2));
			CurrencyAmount nationalTaxAmount = new CurrencyAmount(new BigInt(140),new BigInt(150),new BigInt(2));
			BigInt nationalTaxRate = new BigInt(2);
			String nationalTaxType = "B";
			CurrencyAmount localTaxAmount = new CurrencyAmount(new BigInt(150),new BigInt(150),new BigInt(2));
			CurrencyAmount otherTaxAmount = new CurrencyAmount(new BigInt(160),new BigInt(150),new BigInt(2));
			CurrencyAmount itemTotalCost = new CurrencyAmount(new BigInt(170),new BigInt(150),new BigInt(2));

			Item item = new Item (quantity, unitOfMeasureCode, descriptor, commodityCode,
				productCode, unitCost, netCost, discountInd, discountAmount,
				nationalTaxAmount, nationalTaxRate, nationalTaxType,
				localTaxAmount, otherTaxAmount, itemTotalCost);

			Item[] itemArray = new Item[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = item;

			ItemSeq itemSeq = new ItemSeq(itemArray);
			System.out.println(itemSeq);

			DerOutputStream dout = new DerOutputStream();
			itemSeq.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			itemSeq = new ItemSeq (new DerInputStream(dout.toByteArray()));
			System.out.println(itemSeq);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

