package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
import digsec.set.payment.component.CountryCode;


/**
 *	<pre>
 *	3210 MarketAutoCap ::= SEQUENCE {
 *	3211    renterName             [0] EXPLICIT SETString { ub-renterName=40 }  OPTIONAL,
 *	3212    rentalLocation         [1] Location  OPTIONAL,
 *	3213    rentalDateTime         DateTime,
 *	3214    autoNoShow             [2] AutoNoShow  OPTIONAL,
 *	3215    rentalAgreementNumber  [3] EXPLICIT SETString { ub-rentalNum=25 }  OPTIONAL,
 *	3216    referenceNumber        [4] EXPLICIT SETString { ub-rentalRefNum=8 }  OPTIONAL,
 *	3217    insuranceType          [5] EXPLICIT SETString { ub-insuranceType=1 }  OPTIONAL,
 *	3218    autoRateInfo           [6] AutoRateInfo  OPTIONAL,
 *	3219    returnLocation         [7] Location  OPTIONAL,
 *	3220    returnDateTime         DateTime,
 *	3221    autoCharges            AutoCharges
 *	3222 }
 *	</pre>
 */

public class MarketAutoCap implements DerEncoder {

	private String renterName;
	private Location rentalLocation;
	private DateTime rentalDateTime;
	private AutoNoShow autoNoShow;
	private String rentalAgreementNumber;
	private String referenceNumber;
	private String insuranceType;
	private AutoRateInfo autoRateInfo;
	private Location returnLocation;
	private DateTime returnDateTime;
	private AutoCharges autoCharges;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for MarketAutoCap
	 */
	public MarketAutoCap(String renterName, Location rentalLocation, DateTime rentalDateTime,
		AutoNoShow autoNoShow, String rentalAgreementNumber, String referenceNumber,
		String insuranceType, AutoRateInfo autoRateInfo, Location returnLocation,
		DateTime returnDateTime, AutoCharges autoCharges) throws Exception {

		if(renterName.length()>40 || rentalAgreementNumber.length()>25
               || referenceNumber.length() > 8 || insuranceType.length()>1)
			throw new RuntimeException("Invalid input of MarketAutoCap");

		this.renterName = renterName;
		this.rentalLocation = rentalLocation;
		this.rentalDateTime = rentalDateTime;
		this.autoNoShow = autoNoShow;
		this.rentalAgreementNumber = rentalAgreementNumber;
		this.referenceNumber = referenceNumber;
		this.insuranceType = insuranceType;
		this.autoRateInfo = autoRateInfo;
		this.returnLocation = returnLocation;
		this.returnDateTime = returnDateTime;
		this.autoCharges = autoCharges;
	}

	/**
	 * Constructor for MarketAutoCap
	 * @param din DER representation of MarketAutoCap
	 */
	public MarketAutoCap(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(11);

		if (!adv[0].equals(derNull)) {
			renterName = adv[0].data.getDerValue().getPrintableString();
		}
		else
			renterName = null;

		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_Sequence);
			rentalLocation = new Location(new DerInputStream(adv[1].toByteArray()));
		}
		else
			rentalLocation = null;

		rentalDateTime = new DateTime(new DerInputStream(adv[2].toByteArray()));

		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_Enumerated);
			autoNoShow = new AutoNoShow(new DerInputStream(adv[3].toByteArray()));
		}
		else
			autoNoShow = null;

		if (!adv[4].equals(derNull)) {
			rentalAgreementNumber = adv[4].data.getDerValue().getPrintableString();
		}
		else
			rentalAgreementNumber = null;

		if (!adv[5].equals(derNull)) {
			referenceNumber = adv[5].data.getDerValue().getPrintableString();
		}
		else
			referenceNumber = null;

		if (!adv[6].equals(derNull)) {
			insuranceType = adv[6].data.getDerValue().getPrintableString();
		}
		else
			insuranceType = null;

		if (!adv[7].equals(derNull)) {
			adv[7].resetTag(DerValue.tag_Sequence);
			autoRateInfo = new AutoRateInfo(new DerInputStream(adv[7].toByteArray()));
		}
		else
			autoRateInfo = null;

		if (!adv[8].equals(derNull)) {
			adv[8].resetTag(DerValue.tag_Sequence);
			returnLocation = new Location(new DerInputStream(adv[8].toByteArray()));
		}
		else
			returnLocation = null;

		returnDateTime = new DateTime(new DerInputStream(adv[9].toByteArray()));
		autoCharges = new AutoCharges(new DerInputStream(adv[10].toByteArray()));
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		//kalau renterName ada, tulis, kalau tidak, tulis null
		if (renterName != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (renterName);
			dout.write((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau rentalLocation ada, tulis, kalau tidak, tulis null
		if (rentalLocation != null) {
			midout = new DerOutputStream();
			rentalLocation.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		rentalDateTime.encode (dout);

		//kalau autoNoShow ada, tulis, kalau tidak, tulis null
		if (autoNoShow != null) {
			midout = new DerOutputStream();
			autoNoShow.encode (midout);
			dout.writeImplicit((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau rentalAgreementNumber ada, tulis, kalau tidak, tulis null
		if (rentalAgreementNumber != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (rentalAgreementNumber);
			dout.write((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau referenceNumber ada, tulis, kalau tidak, tulis null
		if (referenceNumber != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (referenceNumber);
			dout.write((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//kalau insuranceType ada, tulis, kalau tidak, tulis null
		if (insuranceType != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (insuranceType);
			dout.write((byte)0xA5, midout);
		}
		else
			dout.putNull();

		//kalau autoRateInfo ada, tulis, kalau tidak, tulis null
		if (autoRateInfo != null) {
			midout = new DerOutputStream();
			autoRateInfo.encode (midout);
			dout.writeImplicit((byte)0xA6, midout);
		}
		else
			dout.putNull();

		//kalau returnLocation ada, tulis, kalau tidak, tulis null
		if (returnLocation != null) {
			midout = new DerOutputStream();
			returnLocation.encode (midout);
			dout.writeImplicit((byte)0xA7, midout);
		}
		else
			dout.putNull();

		returnDateTime.encode (dout);
		autoCharges.encode (dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the renterName
	 */
	public String getRenterName() {
		return renterName;
	}

	/**
	 * Method for get the rentalLocation
	 */
	public Location getRentalLocation() {
		return rentalLocation;
	}

	/**
	 * Method for get the rentalDateTime
	 */
	public DateTime getRentalDateTime() {
		return rentalDateTime;
	}

	/**
	 * Method for get the autoNoShow
	 */
	public AutoNoShow getAutoNoShow() {
		return autoNoShow;
	}

	/**
	 * Method for get the rentalAgreementNumber
	 */
	public String getRentalAgreementNumber() {
		return rentalAgreementNumber;
	}

	/**
	 * Method for get the referenceNumber
	 */
	public String getReferenceNumber() {
		return referenceNumber;
	}

	/**
	 * Method for get the insuranceType
	 */
	public String getInsuranceType() {
		return insuranceType;
	}

	/**
	 * Method for get the autoRateInfo
	 */
	public AutoRateInfo getAutoRateInfo() {
		return autoRateInfo;
	}

	/**
	 * Method for get the returnLocation
	 */
	public Location getReturnLocation() {
		return returnLocation;
	}

	/**
	 * Method for get the returnDateTime
	 */
	public DateTime getReturnDateTime() {
		return returnDateTime;
	}

	/**
	 * Method for get the autoCharges
	 */
	public AutoCharges getAutoCharges() {
		return autoCharges;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if(renterName != null)
			s = s + "MarketAutoCap ::= RenterName : " + renterName + "\n";

		if(rentalLocation != null)
			s = s + "MarketAutoCap ::= RentalLocation : " + rentalLocation + "\n";

		s = s + "MarketAutoCap ::= RentalDateTime : " + rentalDateTime + "\n";

		if(autoNoShow != null)
			s = s + "MarketAutoCap ::= AutoNoShow : " + autoNoShow + "\n";

		if(rentalAgreementNumber != null)
			s = s + "MarketAutoCap ::= RentalAgreementNumber : " + rentalAgreementNumber + "\n";

		if(referenceNumber != null)
			s = s + "MarketAutoCap ::= ReferenceNumber : " + referenceNumber + "\n";

		if(insuranceType != null)
			s = s + "MarketAutoCap ::= InsuranceType : " + insuranceType + "\n";

		if(autoRateInfo != null)
			s = s + "MarketAutoCap ::= AutoRateInfo : " + autoRateInfo + "\n";

		if(returnLocation != null)
			s = s + "MarketAutoCap ::= ReturnLocation : " + returnLocation + "\n";

		s = s + "MarketAutoCap ::= ReturnDateTime : " + returnDateTime + "\n";

		s = s + "MarketAutoCap ::= AutoCharges : " + autoCharges + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			String renterName = "Iman Budi Setiawan";

			// definisikan rentalLocation
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			Location rentalLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime rentalDateTime = new DateTime(new Date(), true);
			AutoNoShow autoNoShow = new AutoNoShow (new BigInt(1));
			String rentalAgreementNumber = "7357715";
			String referenceNumber = "261";
			String insuranceType = "A";

			// definisikan autoRateInfo
			CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";
			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);

			// definisikan returnLocation
			city = "Depok";
			stateProvince = "Jawa Barat";
			postalCode = "12235";
			locationID = "UI";
			Location returnLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime returnDateTime = new DateTime(new Date(), true);

			// definisikan autoCharges
			CurrencyAmount regularDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount lateReturnCharges  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			scale = new DistanceScale(new BigInt(0));
			dist = new BigInt(10);
			Distance totalDistance = new Distance (scale, dist);

			CurrencyAmount extraDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount insuranceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount fuelCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount autoTowingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount oneWayDropOffCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount violationsCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
		 	CurrencyAmount deliveryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			AutoCharges autoCharges = new AutoCharges (regularDistanceCharges,
				lateReturnCharges, totalDistance, extraDistanceCharges,
				insuranceCharges, fuelCharges, autoTowingCharges, oneWayDropOffCharges,
				telephoneCharges, violationsCharges, deliveryCharges, parkingCharges,
				otherCharges, totalTaxAmount, auditAdjustment);

			// buat objek marketAutoCap
			MarketAutoCap marketAutoCap = new MarketAutoCap (renterName, rentalLocation,
				rentalDateTime, autoNoShow, rentalAgreementNumber, referenceNumber,
				insuranceType, autoRateInfo, returnLocation, returnDateTime,
				autoCharges);
			System.out.println(marketAutoCap);

			DerOutputStream dout = new DerOutputStream();
			marketAutoCap.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			marketAutoCap = new MarketAutoCap (new DerInputStream(dout.toByteArray()));
			System.out.println(marketAutoCap);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
