package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3261 MarketHotelCap ::= SEQUENCE {
 *	3262    arrivalDate           Date,
 *	3263    hotelNoShow           [0] HotelNoShow  OPTIONAL,
 *	3264    departureDate         Date,
 *	3265    durationOfStay        [1] INTEGER (0..99)  OPTIONAL,
 *	3266    folioNumber           [2] EXPLICIT SETString { ub-hotelFolio=25 }  OPTIONAL,
 *	3267    propertyPhone         [3] Phone  OPTIONAL,
 *	3268    customerServicePhone  [4] Phone  OPTIONAL,
 *	3269    programCode           [5] EXPLICIT SETString { ub-programCode=2 }  OPTIONAL,
 *	3270    hotelRateInfo         [6] HotelRateInfo  OPTIONAL,
 *	3271    hotelCharges          HotelCharges
 *	3272 }
 *
 *	</pre>
 */

public class MarketHotelCap implements DerEncoder {

	private Date arrivalDate;
	private HotelNoShow hotelNoShow;
	private Date departureDate;
	private BigInt durationOfStay;
	private String folioNumber;
	private Phone propertyPhone;
	private Phone customerServicePhone;
	private String programCode;
	private HotelRateInfo hotelRateInfo;
	private HotelCharges hotelCharges;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for MarketHotelCap
	 */
	public MarketHotelCap(Date arrivalDate, HotelNoShow hotelNoShow, Date departureDate,
		BigInt durationOfStay, String folioNumber, Phone propertyPhone,
		Phone customerServicePhone, String programCode, HotelRateInfo hotelRateInfo,
		HotelCharges hotelCharges) throws Exception {

		if(durationOfStay.toInt()<0  || durationOfStay.toInt()>99 ||
			folioNumber.length()>25 || programCode.length() > 2)
			throw new RuntimeException("Invalid input of MarketHotelCap");

		this.arrivalDate = arrivalDate;
		this.hotelNoShow = hotelNoShow;
		this.departureDate = departureDate;
		this.durationOfStay = durationOfStay;
		this.folioNumber = folioNumber;
		this.propertyPhone = propertyPhone;
		this.customerServicePhone = customerServicePhone;
		this.programCode = programCode;
		this.hotelRateInfo = hotelRateInfo;
		this.hotelCharges = hotelCharges;
	}

	/**
	 * Constructor for MarketHotelCap
	 * @param din DER representation of MarketHotelCap
	 */
	public MarketHotelCap(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(10);

		arrivalDate = (new DerInputStream(adv[0].toByteArray())).getGeneralizedTime();

		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_Enumerated);
			hotelNoShow = new HotelNoShow(new DerInputStream(adv[1].toByteArray()));
		}
		else
			hotelNoShow = null;

		departureDate = (new DerInputStream(adv[2].toByteArray())).getGeneralizedTime();

		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_Integer);
			durationOfStay = adv[3].getInteger();
		}
		else
			durationOfStay = null;

		if (!adv[4].equals(derNull)) {
			folioNumber = adv[4].data.getDerValue().getPrintableString();
		}
		else
			folioNumber = null;

		if (!adv[5].equals(derNull)) {
			adv[5].resetTag(DerValue.tag_PrintableString);
			propertyPhone = new Phone(new DerInputStream(adv[5].toByteArray()));
		}
		else
			propertyPhone = null;

		if (!adv[6].equals(derNull)) {
			adv[6].resetTag(DerValue.tag_PrintableString);
			customerServicePhone = new Phone(new DerInputStream(adv[6].toByteArray()));
		}
		else
			customerServicePhone = null;

		if (!adv[7].equals(derNull)) {
			programCode = adv[7].data.getDerValue().getPrintableString();
		}
		else
			programCode = null;

		if (!adv[8].equals(derNull)) {
			adv[8].resetTag(DerValue.tag_Sequence);
			hotelRateInfo = new HotelRateInfo(new DerInputStream(adv[8].toByteArray()));
		}
		else
			hotelRateInfo = null;

		hotelCharges = new HotelCharges(new DerInputStream(adv[9].toByteArray()));
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		dout.putGeneralizedTime(arrivalDate);

		//kalau hotelNoShow ada, tulis, kalau tidak, tulis null
		if (hotelNoShow != null) {
			midout = new DerOutputStream();
			hotelNoShow.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		dout.putGeneralizedTime(departureDate);

		//kalau durationOfStay ada, tulis, kalau tidak, tulis null
		if (durationOfStay != null) {
			midout = new DerOutputStream();
			midout.putInteger(durationOfStay);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau folioNumber ada, tulis, kalau tidak, tulis null
		if (folioNumber != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (folioNumber);
			dout.write((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau propertyPhone ada, tulis, kalau tidak, tulis null
		if (propertyPhone != null) {
			midout = new DerOutputStream();
			propertyPhone.encode (midout);
			dout.writeImplicit((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau customerServicePhone ada, tulis, kalau tidak, tulis null
		if (customerServicePhone != null) {
			midout = new DerOutputStream();
			customerServicePhone.encode (midout);
			dout.writeImplicit((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//kalau programCode ada, tulis, kalau tidak, tulis null
		if (programCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (programCode);
			dout.write((byte)0xA5, midout);
		}
		else
			dout.putNull();

		//kalau hotelRateInfo ada, tulis, kalau tidak, tulis null
		if (hotelRateInfo != null) {
			midout = new DerOutputStream();
			hotelRateInfo.encode (midout);
			dout.writeImplicit((byte)0xA6, midout);
		}
		else
			dout.putNull();

		hotelCharges.encode (dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the arrivalDate
	 */
	public Date getArrivalDate() {
		return arrivalDate;
	}

	/**
	 * Method for get the hotelNoShow
	 */
	public HotelNoShow getHotelNoShow() {
		return hotelNoShow;
	}

	/**
	 * Method for get the departureDate
	 */
	public Date getDepartureDate() {
		return departureDate;
	}

	/**
	 * Method for get the durationOfStay
	 */
	public BigInt getDurationOfStay() {
		return durationOfStay;
	}

	/**
	 * Method for get the folioNumber
	 */
	public String getFolioNumber() {
		return folioNumber;
	}

	/**
	 * Method for get the propertyPhone
	 */
	public Phone getPropertyPhone() {
		return propertyPhone;
	}

	/**
	 * Method for get the customerServicePhone
	 */
	public Phone getCustomerServicePhone() {
		return customerServicePhone;
	}

	/**
	 * Method for get the programCode
	 */
	public String getProgramCode() {
		return programCode;
	}

	/**
	 * Method for get the hotelRateInfo
	 */
	public HotelRateInfo getHotelRateInfo() {
		return hotelRateInfo;
	}

	/**
	 * Method for get the hotelCharges
	 */
	public HotelCharges getHotelCharges() {
		return hotelCharges;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		s = s + "MarketHotelCap ::= arrivalDate : " + arrivalDate + "\n";

		if(hotelNoShow != null)
			s = s + "MarketHotelCap ::= hotelNoShow : " + hotelNoShow + "\n";

		s = s + "MarketHotelCap ::= departureDate : " + departureDate + "\n";

		if(durationOfStay != null)
			s = s + "MarketHotelCap ::= durationOfStay : " + durationOfStay + "\n";

		if(folioNumber != null)
			s = s + "MarketHotelCap ::= folioNumber : " + folioNumber + "\n";

		if(propertyPhone != null)
			s = s + "MarketHotelCap ::= propertyPhone : " + propertyPhone + "\n";

		if(customerServicePhone != null)
			s = s + "MarketHotelCap ::= customerServicePhone : " + customerServicePhone + "\n";

		if(programCode != null)
			s = s + "MarketHotelCap ::= programCode : " + programCode + "\n";

		if(hotelRateInfo != null)
			s = s + "MarketHotelCap ::= hotelRateInfo : " + hotelRateInfo + "\n";

		s = s + "MarketHotelCap ::= hotelCharges : " + hotelCharges + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			Date arrivalDate= new Date();
			HotelNoShow hotelNoShow = new HotelNoShow(new BigInt(0));
			Date departureDate = new Date();
			BigInt durationOfStay = new BigInt(0);
			String folioNumber ="1";
			Phone propertyPhone = new Phone("021-7357715");
			Phone customerServicePhone = new Phone ("021-7357715");
			String programCode = "A";

			CurrencyAmount dailyRoomRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount dailyTaxRate = new CurrencyAmount(new BigInt(120),new BigInt(100),new BigInt(1));
			HotelRateInfo hotelRateInfo = new HotelRateInfo (dailyRoomRate, dailyTaxRate);

			CurrencyAmount roomCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount roomTax = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount prepaidExpenses = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount foodBeverageCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount roomServiceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount miniBarCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount laundryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount businessCenterCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
 			CurrencyAmount movieCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount healthClubCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount giftShopPurchases = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount folioCashAdvances = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			HotelCharges hotelCharges = new HotelCharges (roomCharges, roomTax,
				prepaidExpenses, foodBeverageCharges, roomServiceCharges, miniBarCharges,
				laundryCharges, telephoneCharges, businessCenterCharges, parkingCharges, movieCharges,
				healthClubCharges, giftShopPurchases, folioCashAdvances, otherCharges,
				totalTaxAmount, auditAdjustment);

			// buat objek marketHotelCap
			MarketHotelCap marketHotelCap = new MarketHotelCap (arrivalDate,
				hotelNoShow, departureDate, durationOfStay, folioNumber, propertyPhone,
				customerServicePhone, programCode, hotelRateInfo, hotelCharges);
			System.out.println(marketHotelCap);

			DerOutputStream dout = new DerOutputStream();
			marketHotelCap.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			marketHotelCap = new MarketHotelCap(new DerInputStream(dout.toByteArray()));
			System.out.println(marketHotelCap);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
