package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
import digsec.set.payment.component.CountryCode;


/**
 *	<pre>
 *	1884 MarketSpecCapData ::= CHOICE {
 *	1885    auto-rental  [0] MarketAutoCap,
 *	1886    hotel        [1] MarketHotelCap,
 *	1887    transport    [2] MarketTransportCap
 *	1888 }
 *	</pre>
 */

public class MarketSpecCapData implements DerEncoder {
	private MarketAutoCap auto_rental;
	private MarketHotelCap hotel;
	private MarketTransportCap transport;
	private int flag;

	/**
	 * Constructor for MarketSpecCapData
	 */
	public MarketSpecCapData(int flag, MarketAutoCap auto_rental) throws Exception {		
		if(flag!=0)
			throw new RuntimeException("Invalid input of MarketSpecCapData");
		this.auto_rental = auto_rental;
	}

	/**
	 * Constructor for MarketSpecCapData
	 */
	public MarketSpecCapData(int flag, MarketHotelCap hotel) throws Exception {		
		if(flag!=0)
			throw new RuntimeException("Invalid input of MarketSpecCapData");
		this.hotel = hotel;
	}

	/**
	 * Constructor for MarketSpecCapData
	 */
	public MarketSpecCapData(int flag, MarketTransportCap transport) throws Exception {		
		if(flag!=0)
			throw new RuntimeException("Invalid input of MarketSpecCapData");
		this.transport = transport;
	}

	/**
	 * Constructor for MarketSpecCapData
	 * @param din DER representation of MarketSpecCapData
	 */
	public MarketSpecCapData(DerInputStream din) throws IOException {
		DerValue adv = din.getDerValue();

		if (adv.tag==(byte)0xA0) {
			flag=0;
			adv.resetTag(DerValue.tag_Sequence);
			auto_rental = new MarketAutoCap(new DerInputStream(adv.toByteArray()));
		}
		else if (adv.tag==(byte)0xA1) {
			flag=1;
			adv.resetTag(DerValue.tag_Sequence);
			hotel = new MarketHotelCap(new DerInputStream(adv.toByteArray()));
		}
		else {
			flag=2;
			adv.resetTag(DerValue.tag_Sequence);
			transport = new MarketTransportCap(new DerInputStream(adv.toByteArray()));
		}
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout = new DerOutputStream();

		// tulis sequence untuk objek ini
		if (flag==0) {
			auto_rental.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else if (flag==1) {
			hotel.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else {
			transport.encode (midout);
			dout.writeImplicit((byte)0xA2, midout);
		}
		out.write(dout.toByteArray());
	}

	/**
	 * Method for get DER Value
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 * @return value the value
	 */
	public MarketAutoCap getAutoRental() {
		if (flag==0) return auto_rental;
		else return null;
	}

	/**
	 * Method for get the value
	 * @return value the value
	 */
	public MarketHotelCap getHotel() {
		if (flag==1) return hotel;
		else return null;
	}

	/**
	 * Method for get the value
	 * @return value the value
	 */
	public MarketTransportCap getTransport() {
		if (flag==2) return transport;
		else return null;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MarketSpecCapData object
	 */
	public String toString() {
		String s = "";

		if (flag==0)
			s = s + "MarketSpecCapData ::= auto-rental = " + auto_rental + "\n";
		else if (flag==1)
			s = s + "MarketSpecCapData ::= hotel = " + hotel + "\n";
		else
			s = s + "MarketSpecCapData ::= transport = " + transport + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			String renterName = "Iman Budi Setiawan";

			// definisikan rentalLocation
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			Location rentalLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime rentalDateTime = new DateTime(new Date(), true);
			AutoNoShow autoNoShow = new AutoNoShow (new BigInt(1));
			String rentalAgreementNumber = "7357715";
			String referenceNumber = "261";
			String insuranceType = "A";

			// definisikan autoRateInfo
			CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";
			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);

			// definisikan returnLocation
			city = "Depok";
			stateProvince = "Jawa Barat";
			postalCode = "12235";
			locationID = "UI";
			Location returnLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime returnDateTime = new DateTime(new Date(), true);

			// definisikan autoCharges
			CurrencyAmount regularDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount lateReturnCharges  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			scale = new DistanceScale(new BigInt(0));
			dist = new BigInt(10);
			Distance totalDistance = new Distance (scale, dist);

			CurrencyAmount extraDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount insuranceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount fuelCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount autoTowingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount oneWayDropOffCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount violationsCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
		 	CurrencyAmount deliveryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			AutoCharges autoCharges = new AutoCharges (regularDistanceCharges,
				lateReturnCharges, totalDistance, extraDistanceCharges,
				insuranceCharges, fuelCharges, autoTowingCharges, oneWayDropOffCharges,
				telephoneCharges, violationsCharges, deliveryCharges, parkingCharges,
				otherCharges, totalTaxAmount, auditAdjustment);

			// buat objek marketAutoCap
			MarketAutoCap marketAutoCap = new MarketAutoCap (renterName, rentalLocation,
				rentalDateTime, autoNoShow, rentalAgreementNumber, referenceNumber,
				insuranceType, autoRateInfo, returnLocation, returnDateTime,
				autoCharges);

			MarketSpecCapData marketSpecCapData = new MarketSpecCapData (0, marketAutoCap);
			System.out.println(marketSpecCapData);

			DerOutputStream dout = new DerOutputStream();
			marketSpecCapData.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			marketSpecCapData = new MarketSpecCapData (new DerInputStream(dout.toByteArray()));
			System.out.println(marketSpecCapData);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
