package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	1897 MarketSpecDataID ::= ENUMERATED {
 *	1898   failedEdit  (0),
 *	1899   auto        (1),
 *	1900   hotel       (2),
 *	1901   transport   (3)
 *	1902 }
 *	</pre>
 */

public class MarketSpecDataID implements DerEncoder {
	
	public static BigInt failedEdit = new BigInt(0);
	public static BigInt auto = new BigInt(1);
	public static BigInt hotel = new BigInt(2);
	public static BigInt transport = new BigInt(3);
	private BigInt value;

	/**
	 * Constructor for MarketSpecDataID
	 */
	public MarketSpecDataID(BigInt value) throws Exception {
		if(value.toInt()<0 || value.toInt()>3)
			throw new RuntimeException("Invalid input of MarketSpecDataID");

		this.value = value;
	}

	/**
	 * Constructor for MarketSpecDataID
	 * @param din DER representation of MarketSpecDataID
	 */
	public MarketSpecDataID(DerInputStream din) throws IOException {
		this.value = din.getEnumerated();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putEnumerated (value);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public BigInt getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if (value.equals(failedEdit))
			s = s + "MarketSpecDataID ::= failedEdit \n";
		else if (value.equals(auto))
			s = s + "MarketSpecDataID ::= auto \n";
		else if (value.equals(hotel))
			s = s + "MarketSpecDataID ::= hotel \n";
		else
			s = s + "MarketSpecDataID ::= transport \n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt value = new BigInt(2);
			MarketSpecDataID marketSpecDataID = new MarketSpecDataID (value);
			System.out.println(marketSpecDataID);

			DerOutputStream dout = new DerOutputStream();
			marketSpecDataID.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			marketSpecDataID = new MarketSpecDataID (new DerInputStream(dout.toByteArray()));
			System.out.println(marketSpecDataID);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

