package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
import digsec.set.payment.component.CountryCode;

/**
 *	<pre>
 *	1890 MarketSpecSaleData ::= SEQUENCE {
 *	1891    marketSpecDataID   MarketSpecDataID OPTIONAL,
 *	1892    marketSpecCapData  MarketSpecCapData OPTIONAL
 *	1893 }
 *	</pre>
 */

public class MarketSpecSaleData implements DerEncoder {
	
	private MarketSpecDataID marketSpecDataID;
	private MarketSpecCapData marketSpecCapData;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for MarketSpecSaleData
	 */
	public MarketSpecSaleData(MarketSpecDataID marketSpecDataID, MarketSpecCapData marketSpecCapData) throws Exception {
		this.marketSpecDataID = marketSpecDataID;
		this.marketSpecCapData = marketSpecCapData;
	}

	/**
	 * Constructor for MarketSpecSaleData
	 * @param din DER representation of MarketSpecSaleData
	 */
	public MarketSpecSaleData(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(2);

		// isi marketSpecDataID
		if (!adv[0].equals(derNull))
			marketSpecDataID = new MarketSpecDataID(new DerInputStream(adv[0].toByteArray()));
		else
			marketSpecDataID = null;

		// isi marketSpecCapData
		if (!adv[1].equals(derNull))
			marketSpecCapData = new MarketSpecCapData(new DerInputStream(adv[1].toByteArray()));
		else
			marketSpecCapData = null;
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		//kalau marketSpecDataID ada, tulis, kalau tidak, tulis null
		if (marketSpecDataID != null)
			marketSpecDataID.encode (dout);
		else
			dout.putNull();

		//kalau marketSpecCapData ada, tulis, kalau tidak, tulis null
		if (marketSpecCapData != null)
			marketSpecCapData.encode (dout);
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the marketSpecDataID
	 *
	 * @return marketSpecDataID
	 */
	public MarketSpecDataID getMarketSpecDataID() {
		return marketSpecDataID;
	}


	/**
	 * Method for get the marketSpecCapData
	 *
	 * @return marketSpecCapData
	 */
	public MarketSpecCapData getMarketSpecCapData() {
		return marketSpecCapData;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "MarketSpecSaleData ::= MarketSpecDataID : " + marketSpecDataID + "\n";

		if(marketSpecCapData != null)
			s = s + "MarketSpecSaleData ::= MarketSpecCapData : " + marketSpecCapData + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			MarketSpecDataID marketSpecDataID = new MarketSpecDataID(new BigInt(1));

			String renterName = "Iman Budi Setiawan";

			// definisikan rentalLocation
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			Location rentalLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime rentalDateTime = new DateTime(new Date(), true);
			AutoNoShow autoNoShow = new AutoNoShow (new BigInt(1));
			String rentalAgreementNumber = "7357715";
			String referenceNumber = "261";
			String insuranceType = "A";

			// definisikan autoRateInfo
			CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";
			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);

			// definisikan returnLocation
			city = "Depok";
			stateProvince = "Jawa Barat";
			postalCode = "12235";
			locationID = "UI";
			Location returnLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime returnDateTime = new DateTime(new Date(), true);

			// definisikan autoCharges
			CurrencyAmount regularDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount lateReturnCharges  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			scale = new DistanceScale(new BigInt(0));
			dist = new BigInt(10);
			Distance totalDistance = new Distance (scale, dist);

			CurrencyAmount extraDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount insuranceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount fuelCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount autoTowingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount oneWayDropOffCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount violationsCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
		 	CurrencyAmount deliveryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			AutoCharges autoCharges = new AutoCharges (regularDistanceCharges,
				lateReturnCharges, totalDistance, extraDistanceCharges,
				insuranceCharges, fuelCharges, autoTowingCharges, oneWayDropOffCharges,
				telephoneCharges, violationsCharges, deliveryCharges, parkingCharges,
				otherCharges, totalTaxAmount, auditAdjustment);

			// buat objek marketAutoCap
			MarketAutoCap marketAutoCap = new MarketAutoCap (renterName, rentalLocation,
				rentalDateTime, autoNoShow, rentalAgreementNumber, referenceNumber,
				insuranceType, autoRateInfo, returnLocation, returnDateTime,
				autoCharges);

			MarketSpecCapData marketSpecCapData = new MarketSpecCapData (0, marketAutoCap);
			
			MarketSpecSaleData marketSpecSaleData = new MarketSpecSaleData (marketSpecDataID, marketSpecCapData);
			System.out.println(marketSpecSaleData);

			DerOutputStream dout = new DerOutputStream();
			marketSpecSaleData.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			marketSpecSaleData = new MarketSpecSaleData (new DerInputStream(dout.toByteArray()));
			System.out.println(marketSpecSaleData);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

