package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3303 MarketTransportCap ::= SEQUENCE {
 *	3304    passengerName     SETString { ub-passName=20 },
 *	3305    departureDate     Date,
 *	3306    origCityAirport   SETString { ub-airportCode=3 },
 *	3307    tripLegSeq        [0] TripLegSeq  OPTIONAL,
 *	3308    ticketNumber      [1] EXPLICIT SETString { ub-ticketNum=13 }  OPTIONAL,
 *	3309    travelAgencyCode  [2] EXPLICIT SETString { ub-taCode=8 }  OPTIONAL,
 *	3310    travelAgencyName  [3] EXPLICIT SETString { ub-taName=25 }  OPTIONAL,
 *	3311    restrictions      [4] Restrictions  OPTIONAL
 *	3312 }
 *
 *	3314 TripLegSeq ::= SEQUENCE SIZE(1..16) OF TripLeg
 *	</pre>
 */

public class MarketTransportCap implements DerEncoder {

	private String passengerName;
	private Date departureDate;
	private String origCityAirport;
	private TripLegSeq tripLegSeq;
	private String ticketNumber;
	private String travelAgencyCode;
	private String travelAgencyName;
	private Restrictions restrictions;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for MarketTransportCap
	 */
	public MarketTransportCap(String passengerName, Date departureDate, String origCityAirport,
		TripLegSeq tripLegSeq, String ticketNumber, String travelAgencyCode,
		String travelAgencyName, Restrictions restrictions) throws Exception {

		if(passengerName.length()>20 || origCityAirport.length()>3 ||
			ticketNumber.length()>13 || travelAgencyCode.length()>8 ||
			travelAgencyName.length()>25)
			throw new RuntimeException("Invalid input of MarketTransportCap");

		this.passengerName = passengerName;
		this.departureDate = departureDate;
		this.origCityAirport = origCityAirport;
		this.tripLegSeq = tripLegSeq;
		this.ticketNumber = ticketNumber;
		this.travelAgencyCode = travelAgencyCode;
		this.travelAgencyName = travelAgencyName;
		this.restrictions = restrictions;
	}

	/**
	 * Constructor for MarketTransportCap
	 * @param din DER representation of MarketTransportCap
	 */
	public MarketTransportCap(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(8);

		if (!adv[0].equals(derNull)) {
			passengerName = adv[0].getPrintableString();
		}
		else
			passengerName = null;

		if (!adv[1].equals(derNull)) {
			departureDate = (new DerInputStream(adv[1].toByteArray())).getGeneralizedTime();
		}
		else
			departureDate = null;

		if (!adv[2].equals(derNull)) {
			origCityAirport = adv[2].getPrintableString();
		}
		else
			origCityAirport = null;

		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_Sequence);
			tripLegSeq = new TripLegSeq(new DerInputStream(adv[3].toByteArray()));
		}
		else
			tripLegSeq = null;

		if (!adv[4].equals(derNull)) {
			adv[4].resetTag(DerValue.tag_PrintableString);
			ticketNumber = adv[4].data.getDerValue().getPrintableString();
		}
		else
			ticketNumber = null;

		if (!adv[5].equals(derNull)) {
			adv[5].resetTag(DerValue.tag_PrintableString);
			travelAgencyCode = adv[5].data.getDerValue().getPrintableString();
		}
		else
			travelAgencyCode = null;

		if (!adv[6].equals(derNull)) {
			adv[6].resetTag(DerValue.tag_PrintableString);
			travelAgencyName = adv[6].data.getDerValue().getPrintableString();
		}
		else
			travelAgencyName = null;

		if (!adv[7].equals(derNull)) {
			adv[7].resetTag(DerValue.tag_Enumerated);
			restrictions = new Restrictions(new DerInputStream(adv[7].toByteArray()));
		}
		else
			restrictions = null;
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		//kalau passengerName ada, tulis, kalau tidak, tulis null
		if (passengerName != null) {
			dout.putPrintableString(passengerName);
		}
		else
			dout.putNull();

		//kalau departureDate ada, tulis, kalau tidak, tulis null
		if (departureDate != null) {
			dout.putGeneralizedTime(departureDate);
		}
		else
			dout.putNull();

		//kalau origCityAirport ada, tulis, kalau tidak, tulis null
		if (origCityAirport != null) {
			dout.putPrintableString(origCityAirport);
		}
		else
			dout.putNull();

		//kalau tripLegSeq ada, tulis, kalau tidak, tulis null
		if (tripLegSeq != null) {
			midout = new DerOutputStream();
			tripLegSeq.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau ticketNumber ada, tulis, kalau tidak, tulis null
		if (ticketNumber != null) {
			midout = new DerOutputStream();
			midout.putPrintableString(ticketNumber);
			dout.write((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau travelAgencyCode ada, tulis, kalau tidak, tulis null
		if (travelAgencyCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString(travelAgencyCode);
			dout.write((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau travelAgencyName ada, tulis, kalau tidak, tulis null
		if (travelAgencyName != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (travelAgencyName);
			dout.write((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau restrictions ada, tulis, kalau tidak, tulis null
		if (restrictions != null) {
			midout = new DerOutputStream();
			restrictions.encode (midout);
			dout.writeImplicit((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the passengerName
	 */
	public String getPassengerName() {
		return passengerName;
	}

	/**
	 * Method for get the departureDate
	 */
	public Date getDepartureDate() {
		return departureDate;
	}

	/**
	 * Method for get the origCityAirport
	 */
	public String getOrigCityAirport() {
		return origCityAirport;
	}

	/**
	 * Method for get the tripLegSeq
	 */
	public TripLegSeq getTripLegSeq() {
		return tripLegSeq;
	}

	/**
	 * Method for get the ticketNumber
	 */
	public String getTicketNumber() {
		return ticketNumber;
	}

	/**
	 * Method for get the travelAgencyCode
	 */
	public String getTravelAgencyCode() {
		return travelAgencyCode;
	}

	/**
	 * Method for get the travelAgencyNameer
	 */
	public String getTravelAgencyName() {
		return travelAgencyName;
	}

	/**
	 * Method for get the restrictions
	 */
	public Restrictions getRestrictions() {
		return restrictions;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if(passengerName != null)
			s = s + "MarketTransportCap ::= passengerName : " + passengerName + "\n";

		if(departureDate != null)
			s = s + "MarketTransportCap ::= departureDate : " + departureDate + "\n";

		if(origCityAirport != null)
			s = s + "MarketTransportCap ::= origCityAirport : " + origCityAirport + "\n";

		if(tripLegSeq != null)
			s = s + "MarketTransportCap ::= tripLegSeq : " + tripLegSeq + "\n";

		if(ticketNumber != null)
			s = s + "MarketTransportCap ::= ticketNumber : " + ticketNumber + "\n";

		if(travelAgencyCode != null)
			s = s + "MarketTransportCap ::= travelAgencyCode : " + travelAgencyCode + "\n";

		if(travelAgencyName != null)
			s = s + "MarketTransportCap ::= travelAgencyName : " + travelAgencyName + "\n";

		if(restrictions != null)
			s = s + "MarketTransportCap ::= restrictions : " + restrictions + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			String passengerName = "Iman Budi Setiawan";
			Date departureDate = new Date();
			String origCityAirport = "SBY";

			Date dateOfTravel = new Date();
			String carrierCode = "BD";
			String serviceClass = "A";
			StopOverCode stopOverCode = new StopOverCode(new BigInt(0));
			String destCityAirport = "JKT";
			String fareBasisCode = "NEWGIA";
			CurrencyAmount departureTax = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			TripLeg tripLeg = new TripLeg (dateOfTravel, carrierCode, serviceClass, stopOverCode, destCityAirport, fareBasisCode, departureTax);

			TripLeg[] itemArray = new TripLeg[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = tripLeg;
			TripLegSeq tripLegSeq = new TripLegSeq(itemArray);

			String ticketNumber = "12345678";
			String travelAgencyCode = "GIA";
			String travelAgencyName = "Garuda Indonesia";
			Restrictions restrictions = new Restrictions(new BigInt(0));

			MarketTransportCap marketTransportCap = new MarketTransportCap (passengerName,
				departureDate, origCityAirport, tripLegSeq, ticketNumber, travelAgencyCode,
				travelAgencyName, restrictions);
			System.out.println(marketTransportCap);

			DerOutputStream dout = new DerOutputStream();
			marketTransportCap.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			marketTransportCap = new MarketTransportCap (new DerInputStream(dout.toByteArray()));
			System.out.println(marketTransportCap);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

