package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	1904 MerOrderNum ::= VisibleString (SIZE(1..ub-merOrderNum=25))
 *	</pre>
 */

public class MerOrderNum implements DerEncoder {
	
	private String merOrderNum;

	/**
	 * Constructor for MerOrderNum
	 */
	public MerOrderNum(String merOrderNum) throws Exception {
		if(merOrderNum.length()>25)
			throw new RuntimeException("Invalid input of MerOrderNum");

		this.merOrderNum = merOrderNum;
	}

	/**
	 * Constructor for MerOrderNum
	 * @param din DER representation of MerOrderNum
	 */
	public MerOrderNum(DerInputStream din) throws IOException {
		DerValue adv = din.getDerValue();

		// isi merOrderNum
		merOrderNum = adv.getPrintableString();
	}

	/**
	* Method for get DER Value 
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		//tulis dist
		dout.putPrintableString(merOrderNum);
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the merOrderNum
	 *
	 * @return merOrderNum
	 */
	public String getMerOrderNum() {
		return merOrderNum;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of MerOrderNum object
	 */
	public String toString() {
		String s = "MerOrderNum ::= " + merOrderNum + "\n";
		return s;
	}

	static void main(String[] args) {
		try 
		{
			String data = "123456789";
			MerOrderNum merOrderNum = new MerOrderNum (data);
			System.out.println(merOrderNum);

			DerOutputStream dout = new DerOutputStream();
			merOrderNum.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			merOrderNum = new MerOrderNum (new DerInputStream(dout.toByteArray()));
			System.out.println(merOrderNum);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

