package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	1937 PayRecurInd ::= ENUMERATED {
 *	1938    unknown               (0),
 *	1939    singleTransaction     (1),
 *	1940    recurringTransaction  (2),
 *	1941    installmentPayment    (3),
 *	1942    otherMailOrder        (4)
 *	1943 }
 *
 *	</pre>
 */

public class PayRecurInd implements DerEncoder {
	
	public static BigInt unknown = new BigInt(0);
	public static BigInt singleTransaction = new BigInt(1);
	public static BigInt recurringTransaction = new BigInt(2);
	public static BigInt installmentPayment = new BigInt(3);
	public static BigInt otherMailOrder = new BigInt(4);
	private BigInt value;

	/**
	 * Constructor for PayRecurInd
	 */
	public PayRecurInd(BigInt value) throws Exception {
		if(value.toInt()<0 || value.toInt()>4)
			throw new RuntimeException("Invalid input of PayRecurInd");

		this.value = value;
	}

	/**
	 * Constructor for PayRecurInd
	 * @param din DER representation of PayRecurInd
	 */
	public PayRecurInd(DerInputStream din) throws IOException {
		this.value = din.getEnumerated();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putEnumerated (value);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public BigInt getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if (value.equals(unknown))
			s = s + "PayRecurInd ::= unknown \n";
		else if (value.equals(singleTransaction))
			s = s + "PayRecurInd ::= singleTransaction \n";
		else if (value.equals(recurringTransaction))
			s = s + "PayRecurInd ::= recurringTransaction \n";
		else if (value.equals(installmentPayment))
			s = s + "PayRecurInd ::= installmentPayment \n";
		else
			s = s + "PayRecurInd ::= otherMailOrder \n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			PayRecurInd payRecurInd = new PayRecurInd (new BigInt(2));
			System.out.println(payRecurInd);

			DerOutputStream dout = new DerOutputStream();
			payRecurInd.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			payRecurInd = new PayRecurInd (new DerInputStream(dout.toByteArray()));
			System.out.println(payRecurInd);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

