package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
import digsec.set.payment.component.CountryCode;

/**
 *	<pre>
 *	1920 SaleDetail ::= SEQUENCE {
 *	1921    batchID                  [ 0] BatchID  OPTIONAL,
 *	1922    batchSequenceNum         [ 1] BatchSequenceNum  OPTIONAL,
 *	1923    payRecurInd              [ 2] PayRecurInd  OPTIONAL,
 *	1924    merOrderNum              [ 3] MerOrderNum  OPTIONAL,
 *	1925    authCharInd              [ 4] AuthCharInd  OPTIONAL,
 *	1926    marketSpecSaleData       [ 5] MarketSpecSaleData  OPTIONAL,
 *	1927    commercialCardData       [ 6] CommercialCardData  OPTIONAL,
 *	1928    orderSummary             [ 7] EXPLICIT SETString { ub-summary=35 }  OPTIONAL,
 *	1929    customerReferenceNumber  [ 8] EXPLICIT SETString { ub-reference=28 }  OPTIONAL,
 *	1930    customerServicePhone     [ 9] EXPLICIT Phone  OPTIONAL,
 *	1931    okToPrintPhoneInd        [10] BOOLEAN DEFAULT TRUE,
 *	1932    saleExtensions           [11] MsgExtensions {{SaleExtensionsIOS}}  OPTIONAL
 *	1933 }
 *	
 *	
 *	</pre>
 */

public class SaleDetail implements DerEncoder {

	private BatchID batchID;
	private BatchSequenceNum batchSequenceNum;
	private PayRecurInd payRecurInd;
	private MerOrderNum merOrderNum;
	private AuthCharInd authCharInd;
	private MarketSpecSaleData marketSpecSaleData;
	private CommercialCardData commercialCardData;
	private String orderSummary;
	private String customerReferenceNumber;
	private Phone customerServicePhone;
	private boolean okToPrintPhoneInd;
	private MsgExtensions saleExtensions;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for SaleDetail
	 */
	public SaleDetail(BatchID batchID, BatchSequenceNum batchSequenceNum, PayRecurInd payRecurInd,
		MerOrderNum merOrderNum, AuthCharInd authCharInd, MarketSpecSaleData marketSpecSaleData,
		CommercialCardData commercialCardData, String orderSummary, String customerReferenceNumber,
		Phone customerServicePhone, boolean okToPrintPhoneInd,
		MsgExtensions saleExtensions) throws Exception {

		if(orderSummary.length()>35 || customerReferenceNumber.length() > 28)
			throw new RuntimeException("Invalid input of SaleDetail");

		this.batchID = batchID;
		this.batchSequenceNum = batchSequenceNum;
		this.payRecurInd = payRecurInd;
		this.merOrderNum = merOrderNum;
		this.authCharInd = authCharInd;
		this.marketSpecSaleData = marketSpecSaleData;
		this.commercialCardData = commercialCardData;
		this.orderSummary = orderSummary;
		this.customerReferenceNumber = customerReferenceNumber;
		this.customerServicePhone = customerServicePhone;
		this.okToPrintPhoneInd = okToPrintPhoneInd;
		this.saleExtensions = saleExtensions;
	}

	/**
	 * Constructor for SaleDetail
	 * @param din DER representation of SaleDetail
	 */
	public SaleDetail(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(10);

		if (!adv[0].equals(derNull)) {
			adv[0].resetTag(DerValue.tag_Integer);
			batchID = new BatchID(new DerInputStream(adv[0].toByteArray()));
		}
		else
			batchID = null;

		if (!adv[1].equals(derNull)) {
			adv[1].resetTag(DerValue.tag_Integer);
			batchSequenceNum = new BatchSequenceNum(new DerInputStream(adv[1].toByteArray()));
		}
		else
			batchSequenceNum = null;

		if (!adv[2].equals(derNull)) {
			adv[2].resetTag(DerValue.tag_Enumerated);
			payRecurInd = new PayRecurInd(new DerInputStream(adv[2].toByteArray()));
		}
		else
			payRecurInd = null;


		if (!adv[3].equals(derNull)) {
			adv[3].resetTag(DerValue.tag_PrintableString);
			merOrderNum = new MerOrderNum(new DerInputStream(adv[3].toByteArray()));
		}
		else
			merOrderNum = null;

		if (!adv[4].equals(derNull)) {
			adv[4].resetTag(DerValue.tag_Enumerated);
			authCharInd = new AuthCharInd(new DerInputStream(adv[4].toByteArray()));
		}
		else
			authCharInd = null;

		if (!adv[5].equals(derNull)) {
			adv[5].resetTag(DerValue.tag_Sequence);
			marketSpecSaleData = new MarketSpecSaleData(new DerInputStream(adv[5].toByteArray()));
		}
		else
			marketSpecSaleData = null;

		if (!adv[6].equals(derNull)) {
			adv[6].resetTag(DerValue.tag_Sequence);
			commercialCardData = new CommercialCardData(new DerInputStream(adv[6].toByteArray()));
		}
		else
			commercialCardData = null;

		if (!adv[7].equals(derNull)) {
			orderSummary = adv[7].data.getDerValue().getPrintableString();
		}
		else
			orderSummary = null;

		if (!adv[8].equals(derNull)) {
			customerReferenceNumber = adv[8].data.getDerValue().getPrintableString();
		}
		else
			customerReferenceNumber= null;

		
		if (!adv[9].equals(derNull)) {
			customerServicePhone = new Phone(adv[9].data);

		}
		else
			customerServicePhone = null;

		adv[10].resetTag(DerValue.tag_Boolean);
		okToPrintPhoneInd = adv[10].getBoolean();

		if (!adv[11].equals(derNull)) {
			adv[11].resetTag(DerValue.tag_PrintableString);
			saleExtensions = new MsgExtensions(new DerInputStream(adv[11].toByteArray()));
		}
		else
			saleExtensions = null;

	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		//kalau batchID ada, tulis, kalau tidak, tulis null
		if (batchID != null) {
			midout = new DerOutputStream();
			batchID.encode (midout);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau batchSequenceNum ada, tulis, kalau tidak, tulis null
		if (batchSequenceNum != null) {
			midout = new DerOutputStream();
			batchSequenceNum.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//kalau payRecurInd ada, tulis, kalau tidak, tulis null
		if (payRecurInd != null) {
			midout = new DerOutputStream();
			payRecurInd.encode (midout);
			dout.writeImplicit((byte)0xA2, midout);
		}
		else
			dout.putNull();

		//kalau merOrderNum ada, tulis, kalau tidak, tulis null
		if (merOrderNum != null) {
			midout = new DerOutputStream();
			merOrderNum.encode (midout);
			dout.writeImplicit((byte)0xA3, midout);
		}
		else
			dout.putNull();

		//kalau authCharInd ada, tulis, kalau tidak, tulis null
		if (authCharInd != null) {
			midout = new DerOutputStream();
			authCharInd.encode (midout);
			dout.writeImplicit((byte)0xA4, midout);
		}
		else
			dout.putNull();

		//kalau marketSpecSaleData ada, tulis, kalau tidak, tulis null
		if (marketSpecSaleData != null) {
			midout = new DerOutputStream();
			marketSpecSaleData.encode (midout);
			dout.writeImplicit((byte)0xA5, midout);
		}
		else
			dout.putNull();
	
		//kalau commercialCardData ada, tulis, kalau tidak, tulis null
		if (commercialCardData != null) {
			midout = new DerOutputStream();
			commercialCardData.encode (midout);
			dout.writeImplicit((byte)0xA6, midout);
		}
		else
			dout.putNull();

		//kalau orderSummary ada, tulis, kalau tidak, tulis null
		if (orderSummary != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (orderSummary);
			dout.write((byte)0xA7, midout);
		}
		else
			dout.putNull();

		//kalau customerReferenceNumber ada, tulis, kalau tidak, tulis null
		if (customerReferenceNumber != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (customerReferenceNumber);
			dout.write((byte)0xA8, midout);
		}
		else
			dout.putNull();

		//kalau customerServicePhone ada, tulis, kalau tidak, tulis null
		if (customerServicePhone != null) {
			midout = new DerOutputStream();
			customerServicePhone.encode (midout);
			dout.write((byte)0xA9, midout);
		}
		else
			dout.putNull();

		midout = new DerOutputStream();
		midout.putBoolean(okToPrintPhoneInd);
		dout.writeImplicit((byte)0xAA, midout);

		//kalau saleExtensions ada, tulis, kalau tidak, tulis null
		if (saleExtensions != null) {
			midout = new DerOutputStream();
			saleExtensions.encode (midout);
			dout.writeImplicit((byte)0xAB, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the batchID
	 */
	public BatchID getBatchID() {
		return batchID;
	}

	/**
	 * Method for get the batchSequenceNum
	 */
	public BatchSequenceNum getBatchSequenceNum() {
		return batchSequenceNum;
	}

	/**
	 * Method for get the payRecurInd
	 */
	public PayRecurInd getPayRecurInd() {
		return payRecurInd;
	}

	/**
	 * Method for get the merOrderNum
	 */
	public MerOrderNum getMerOrderNum() {
		return merOrderNum;
	}

	/**
	 * Method for get the authCharInd
	 */
	public AuthCharInd getAuthCharInd() {
		return authCharInd;
	}

	/**
	 * Method for get the marketSpecSaleData
	 */
	public MarketSpecSaleData getMarketSpecSaleData() {
		return marketSpecSaleData;
	}

	/**
	 * Method for get the commercialCardData
	 */
	public CommercialCardData getCommercialCardData() {
		return commercialCardData;
	}

	/**
	 * Method for get the orderSummary
	 */
	public String getOrderSummary() {
		return orderSummary;
	}

	/**
	 * Method for get the customerReferenceNumber
	 */
	public String getCustomerReferenceNumber() {
		return customerReferenceNumber;
	}

	/**
	 * Method for get the customerServicePhone
	 */
	public Phone getCustomerServicePhone() {
		return customerServicePhone;
	}

	/**
	 * Method for get the okToPrintPhoneInd
	 */
	public boolean getOkToPrintPhoneInd() {
		return okToPrintPhoneInd;
	}

	/**
	 * Method for get the saleExtensions
	 */
	public MsgExtensions getSaleExtensions() {
		return saleExtensions;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if(batchID != null)
			s = s + "SaleDetail ::= batchID : " + batchID + "\n";

		if(batchSequenceNum != null)
			s = s + "SaleDetail ::= batchSequenceNum : " + batchSequenceNum + "\n";

		if(payRecurInd != null)
			s = s + "SaleDetail ::= payRecurInd : " + payRecurInd + "\n";

		if(merOrderNum != null)
			s = s + "SaleDetail ::= merOrderNum : " + merOrderNum + "\n";

		if(authCharInd != null)
			s = s + "SaleDetail ::= authCharInd : " + authCharInd + "\n";

		if(marketSpecSaleData != null)
			s = s + "SaleDetail ::= marketSpecSaleData : " + marketSpecSaleData + "\n";

		if(commercialCardData != null)
			s = s + "SaleDetail ::= commercialCardData : " + commercialCardData + "\n";

		if(orderSummary != null)
			s = s + "SaleDetail ::= orderSummary : " + orderSummary + "\n";

		if(customerReferenceNumber != null)
			s = s + "SaleDetail ::= customerReferenceNumber : " + customerReferenceNumber + "\n";

		if(customerServicePhone != null)
			s = s + "SaleDetail ::= customerServicePhone : " + customerServicePhone + "\n";

		s = s + "SaleDetail ::= okToPrintPhoneInd : " + okToPrintPhoneInd + "\n";

		if(saleExtensions != null)
			s = s + "SaleDetail ::= saleExtensions : " + saleExtensions + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BatchID batchID = new BatchID (new BigInt(10));
			BatchSequenceNum batchSequenceNum = new BatchSequenceNum (new BigInt(8));
			PayRecurInd payRecurInd = new PayRecurInd (new BigInt(2));
			MerOrderNum merOrderNum = new MerOrderNum ("123456789");
			AuthCharInd authCharInd = new AuthCharInd (new BigInt(1));

			MarketSpecDataID marketSpecDataID = new MarketSpecDataID(new BigInt(1));

			String renterName = "Iman Budi Setiawan";

			// definisikan rentalLocation
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			Location rentalLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime rentalDateTime = new DateTime(new Date(), true);
			AutoNoShow autoNoShow = new AutoNoShow (new BigInt(1));
			String rentalAgreementNumber = "7357715";
			String referenceNumber = "261";
			String insuranceType = "A";

			// definisikan autoRateInfo
			CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";
			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);

			// definisikan returnLocation
			city = "Depok";
			stateProvince = "Jawa Barat";
			postalCode = "12235";
			locationID = "UI";
			Location returnLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime returnDateTime = new DateTime(new Date(), true);

			// definisikan autoCharges
			CurrencyAmount regularDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount lateReturnCharges  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			scale = new DistanceScale(new BigInt(0));
			dist = new BigInt(10);
			Distance totalDistance = new Distance (scale, dist);

			CurrencyAmount extraDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount insuranceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount fuelCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount autoTowingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount oneWayDropOffCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount violationsCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
		 	CurrencyAmount deliveryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			AutoCharges autoCharges = new AutoCharges (regularDistanceCharges,
				lateReturnCharges, totalDistance, extraDistanceCharges,
				insuranceCharges, fuelCharges, autoTowingCharges, oneWayDropOffCharges,
				telephoneCharges, violationsCharges, deliveryCharges, parkingCharges,
				otherCharges, totalTaxAmount, auditAdjustment);

			// buat objek marketAutoCap
			MarketAutoCap marketAutoCap = new MarketAutoCap (renterName, rentalLocation,
				rentalDateTime, autoNoShow, rentalAgreementNumber, referenceNumber,
				insuranceType, autoRateInfo, returnLocation, returnDateTime,
				autoCharges);

			MarketSpecCapData marketSpecCapData = new MarketSpecCapData (0, marketAutoCap);
			MarketSpecSaleData marketSpecSaleData = new MarketSpecSaleData (marketSpecDataID, marketSpecCapData);

			CurrencyAmount totalFreightShippingAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalDutyTariffAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String dutyTariffReference = "A";
			CurrencyAmount totalNationalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalLocalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalOtherTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String merchantTaxID = "B";
			String merchantDutyTariffRef = "C";
			String customerDutyTariffRef = "D";
			String summaryCommodityCode = "E";
			String merchantType = "F";
			
			ChargeInfo chargeInfo = new ChargeInfo (totalFreightShippingAmount, totalDutyTariffAmount, 
				dutyTariffReference, totalNationalTaxAmount, totalLocalTaxAmount,
				totalOtherTaxAmount, totalTaxAmount, merchantTaxID, merchantDutyTariffRef,
				customerDutyTariffRef, summaryCommodityCode, merchantType);

			countryCode = new CountryCode (new BigInt(20));
			city = "Surabaya";
			stateProvince = "East Java";
			postalCode = "723411";
			locationID = "Tower";
			
			Location merchantLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipFrom = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipTo = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			BigInt quantity = new BigInt(1);
			String unitOfMeasureCode = "AA";
			String descriptor = "DD";
			String commodityCode = "CC";
			String productCode = "AB";
			CurrencyAmount unitCost = new CurrencyAmount(new BigInt(110),new BigInt(150),new BigInt(2));
			CurrencyAmount netCost  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			boolean discountInd = false;
			CurrencyAmount discountAmount = new CurrencyAmount(new BigInt(130),new BigInt(150),new BigInt(2));
			CurrencyAmount nationalTaxAmount = new CurrencyAmount(new BigInt(140),new BigInt(150),new BigInt(2));
			BigInt nationalTaxRate = new BigInt(2);
			String nationalTaxType = "B";
			CurrencyAmount localTaxAmount = new CurrencyAmount(new BigInt(150),new BigInt(150),new BigInt(2));
			CurrencyAmount otherTaxAmount = new CurrencyAmount(new BigInt(160),new BigInt(150),new BigInt(2));
			CurrencyAmount itemTotalCost = new CurrencyAmount(new BigInt(170),new BigInt(150),new BigInt(2));

			Item item = new Item (quantity, unitOfMeasureCode, descriptor, commodityCode,
				productCode, unitCost, netCost, discountInd, discountAmount,
				nationalTaxAmount, nationalTaxRate, nationalTaxType,
				localTaxAmount, otherTaxAmount, itemTotalCost);

			Item[] itemArray = new Item[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = item;

			ItemSeq itemSeq = new ItemSeq(itemArray);

			CommercialCardData commercialCardData = new CommercialCardData (chargeInfo,
				merchantLocation, shipFrom, shipTo, itemSeq);

			String orderSummary = "DETAIL OF PAYMENT";
			String customerReferenceNumber = "333777888";
			Phone customerServicePhone = new Phone("7357715");
			boolean okToPrintPhoneInd = true;
			MsgExtensions saleExtensions = new MsgExtensions("END MESSAGE");


			// buat objek saleDetail
			SaleDetail saleDetail = new SaleDetail (batchID, batchSequenceNum, payRecurInd,
				merOrderNum, authCharInd, marketSpecSaleData, commercialCardData,
				orderSummary, customerReferenceNumber, customerServicePhone,
				okToPrintPhoneInd, saleExtensions);
			System.out.println(saleDetail);

			DerOutputStream dout = new DerOutputStream();
			saleDetail.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			saleDetail = new SaleDetail(new DerInputStream(dout.toByteArray()));
			System.out.println(saleDetail);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}