package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3326 StopOverCode ::= ENUMERATED {
 *	3327    noStopOverPermitted  (0),
 *	3328    stopOverPermitted    (1)
 *	3329 }
 *	</pre>
 */

public class StopOverCode implements DerEncoder {
	
	public static BigInt noStopOverPermitted = new BigInt(0);
	public static BigInt stopOverPermitted = new BigInt(1);
	private BigInt value;

	/**
	 * Constructor for StopOverCode
	 */
	public StopOverCode(BigInt value) throws Exception {
		if(value.toInt()<0 || value.toInt()>1)
			throw new RuntimeException("Invalid input of StopOverCode");

		this.value = value;
	}

	/**
	 * Constructor for StopOverCode
	 * @param din DER representation of StopOverCode
	 */
	public StopOverCode(DerInputStream din) throws IOException {
		this.value = din.getEnumerated();
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		dout.putEnumerated (value);

		//tulis tag sequence
		out.write(dout.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the value
	 *
	 * @return value the value
	 */
	public BigInt getValue() {
		return value;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		if (value.equals(noStopOverPermitted))
			s = s + "StopOverCode ::= noStopOverPermitted \n";
		else
			s = s + "StopOverCode ::= stopOverPermitted \n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			BigInt value = new BigInt(2);
			StopOverCode stopOverCode = new StopOverCode (value);
			System.out.println(stopOverCode);

			DerOutputStream dout = new DerOutputStream();
			stopOverCode.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			stopOverCode = new StopOverCode (new DerInputStream(dout.toByteArray()));
			System.out.println(stopOverCode);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}

