package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import java.util.Date;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;

/**
 *	<pre>
 *	3316 TripLeg ::= SEQUENCE {
 *	3317    dateOfTravel     Date,
 *	3318    carrierCode      SETString { ub-carrierCode=2 },
 *	3319    serviceClass     SETString { ub-serviceClass=1 },
 *	3320    stopOverCode     StopOverCode,
 *	3321    destCityAirport  SETString { ub-airportCode=3 },
 *	3322    fareBasisCode    [0] SETString { ub-fareBasis=6 }  OPTIONAL,
 *	3323    departureTax     [1] CurrencyAmount  OPTIONAL
 *	3324 }
 *	</pre>
 */

public class TripLeg implements DerEncoder {
	
	private Date dateOfTravel;
	private String carrierCode;
	private String serviceClass;
	private StopOverCode stopOverCode;
	private String destCityAirport;
	private String fareBasisCode;
	private CurrencyAmount departureTax;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for TripLeg
	 */
	public TripLeg(Date dateOfTravel, String carrierCode, String serviceClass, StopOverCode stopOverCode, String destCityAirport, String fareBasisCode, CurrencyAmount departureTax) throws Exception {
		if(carrierCode.length()>2 || serviceClass.length()>1
               || destCityAirport.length() > 3 || fareBasisCode.length()>6)  
			throw new RuntimeException("Invalid input of TripLeg");

		this.dateOfTravel = dateOfTravel;
		this.carrierCode = carrierCode;
		this.serviceClass = serviceClass;
		this.stopOverCode = stopOverCode;
		this.destCityAirport = destCityAirport;
		this.fareBasisCode = fareBasisCode;
		this.departureTax = departureTax;
	}

	/**
	 * Constructor for TripLeg
	 * @param din DER representation of TripLeg
	 */
	public TripLeg(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(7);

		//isi dateOfTravel
		dateOfTravel = (new DerInputStream(adv[0].toByteArray())).getGeneralizedTime();

		// isi carrierCode
		carrierCode = adv[1].getPrintableString();

		// isi serviceClass
		serviceClass = adv[2].getPrintableString();

		// isi stopOverCode
		stopOverCode = new StopOverCode(new DerInputStream(adv[3].toByteArray()));

		// isi destCityAirport
		destCityAirport = adv[4].getPrintableString();

		// isi fareBasisCode
		if (!adv[5].equals(derNull)) {
			adv[5].resetTag(DerValue.tag_PrintableString);
			fareBasisCode = adv[5].getPrintableString();
		}
		else
			fareBasisCode = null;

		// isi departureTax
		if (!adv[6].equals(derNull)) {
			adv[6].resetTag(DerValue.tag_Sequence);
			departureTax = new CurrencyAmount(new DerInputStream(adv[6].toByteArray()));
		}
		else
			departureTax = null;
	}

	/**
	* Method for get DER Value 
	*
	* @param out OutputStream for storing the result
	*/	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream midout;

		dout.putGeneralizedTime (dateOfTravel);
		dout.putPrintableString (carrierCode);
		dout.putPrintableString (serviceClass);
		stopOverCode.encode (dout);
		dout.putPrintableString (destCityAirport);

		//kalau fareBasisCode ada, tulis, kalau tidak, tulis null
		if (fareBasisCode != null) {
			midout = new DerOutputStream();
			midout.putPrintableString (fareBasisCode);
			dout.writeImplicit((byte)0xA0, midout);
		}
		else
			dout.putNull();

		//kalau departureTax ada, tulis, kalau tidak, tulis null
		if (departureTax != null) {
			midout = new DerOutputStream();
			departureTax.encode (midout);
			dout.writeImplicit((byte)0xA1, midout);
		}
		else
			dout.putNull();

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke out
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the dateOfTravel
	 *
	 * @return date and time of message generation
	 */
	public Date getDateOfTravel() {
		return dateOfTravel;
	}

	/**
	 * Method for get the carrierCode
	 *
	 * @return carrierCode
	 */
	public String getCarrierCode() {
		return carrierCode;
	}

	/**
	 * Method for get the serviceClass
	 *
	 * @return serviceClass
	 */
	public String getServiceClass() {
		return serviceClass;
	}

	/**
	 * Method for get the stopOverCode
	 *
	 * @return stopOverCode
	 */
	public StopOverCode getStopOverCode() {
		return stopOverCode;
	}

	/**
	 * Method for get the destCityAirport
	 *
	 * @return destCityAirport
	 */
	public String getDestCityAirport() {
		return destCityAirport;
	}

	/**
	 * Method for get the fareBasisCode
	 *
	 * @return fareBasisCode
	 */
	public String getFareBasisCode() {
		return fareBasisCode;
	}

	/**
	 * Method for get the departureTax
	 *
	 * @return departureTax
	 */
	public CurrencyAmount getDepartureTax() {
		return departureTax;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";
		s = s + "TripLeg ::= DateOfTravel: " + dateOfTravel + "\n";
		s = s + "TripLeg ::= CarrierCode : " + carrierCode + "\n";
		s = s + "TripLeg ::= ServiceClass: " + serviceClass + "\n";
		s = s + "TripLeg ::= StopOverCode: " + stopOverCode + "\n";
		s = s + "TripLeg ::= DestCityAirport: " + destCityAirport + "\n";

		if(fareBasisCode != null)
			s = s + "TripLeg ::= FareBasisCode: " + fareBasisCode + "\n";

		if(departureTax != null)
			s = s + "TripLeg ::= DepartureTax : " + departureTax + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			Date dateOfTravel = new Date();
			String carrierCode = "BD";
			String serviceClass = "A";
			StopOverCode stopOverCode = new StopOverCode(new BigInt(0));
			String destCityAirport = "JKT";
			String fareBasisCode = "NEWGIA";
			CurrencyAmount departureTax = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			
			TripLeg tripleg = new TripLeg (dateOfTravel, carrierCode, serviceClass, stopOverCode, destCityAirport, fareBasisCode, departureTax);
			System.out.println(tripleg);

			DerOutputStream dout = new DerOutputStream();
			tripleg.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			tripleg = new TripLeg (new DerInputStream(dout.toByteArray()));
			System.out.println(tripleg);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
