package digsec.set.payment.component.SaleDetail;

import java.io.OutputStream;
import java.io.IOException;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;
import digsec.set.payment.component.Location;
/**
 *	<pre>
 *	3314 TripLegSeq ::= SEQUENCE SIZE(1..16) OF TripLeg
 *	</pre>
 */

public class TripLegSeq implements DerEncoder {

	private TripLeg[] tripLeg;
	private DerValue derNull = new DerValue(DerValue.tag_Null,new byte[0]);

	/**
	 * Constructor for TripLegSeq
	 */
	public TripLegSeq(TripLeg[] tripLeg) throws Exception {

		if(tripLeg.length>16)
			throw new RuntimeException("Invalid length of TripLegSeq");
		this.tripLeg = tripLeg;
	}

	/**
	 * Constructor for TripLegSeq
	 * @param din DER representation of TripLegSeq
	 */
	public TripLegSeq(DerInputStream din) throws IOException {
		DerValue[] adv = din.getSequence(16);

		tripLeg = new TripLeg[adv.length];
		for (int i=0; i<adv.length; i++)
			tripLeg[i] = new TripLeg(new DerInputStream(adv[i].toByteArray()));
	}

	/**
	* Method for get DER Value
	*
	* @param out OutputStream for storing the result
	*/
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();

		for (int i=0; i<this.tripLeg.length; i++)
			tripLeg[i].encode(dout);

		//autoApplicableRate.encode (dout);

		//tulis tag sequence
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(DerValue.tag_Sequence,dout);
		out.write(dout2.toByteArray());		//tulis semuanya ke dout
	}		

	/**
	 * Method for get DER Value 
	 *
	 * @param dout DerOutputStream for storing the result
	 */
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	/**
	 * Method for get the TripLeg
	 */
	public TripLeg[] getTripLegSeq() {
		return tripLeg;
	}

	/**
	 * Method for get the string representation of this object
	 *
	 * @return string representation of TriLeg object
	 */
	public String toString() {
		String s = "";

		s = s + "TripLegSeq ::= \n";

		if(tripLeg != null)
			for (int i=0; i<tripLeg.length; i++)
				s = s + "TripLeg[" + i +"]: " + tripLeg[i] + "\n";

		return s;
	}

	static void main(String[] args) {
		try 
		{
			Date dateOfTravel = new Date();
			String carrierCode = "BD";
			String serviceClass = "A";
			StopOverCode stopOverCode = new StopOverCode(new BigInt(0));
			String destCityAirport = "JKT";
			String fareBasisCode = "NEWGIA";
			CurrencyAmount departureTax = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			
			TripLeg tripLeg = new TripLeg (dateOfTravel, carrierCode, serviceClass, stopOverCode, destCityAirport, fareBasisCode, departureTax);

			TripLeg[] itemArray = new TripLeg[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = tripLeg;

			TripLegSeq tripLegSeq = new TripLegSeq(itemArray);
			System.out.println(tripLegSeq);

			DerOutputStream dout = new DerOutputStream();
			tripLegSeq.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			System.out.println("*******reconstruct********");
			tripLegSeq = new TripLegSeq (new DerInputStream(dout.toByteArray()));
			System.out.println(tripLegSeq);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}
