package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import digsec.set.payment.component. *;
import sun.security.util.*;
import sun.misc.HexDumpEncoder;

/**
 *	<pre>
 *	1030 AVSData ::= SEQUENCE {
 *	1031    streetAddress  SETString { ub-AVSData = 128 } OPTIONAL,
 *	1032    location       Location
 *	1033 }
 *	</pre>
 */

public class AVSData implements DerEncoder {

	//fields
	public String 	streetAddress;
	public Location	location;

	//Constructor
	public AVSData(String street, Location loc) throws Exception {
		streetAddress = street;
		location = loc;
	}

	public AVSData(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);
		if(adv[0].tag!=(byte)0x05)
			streetAddress = adv[0].getPrintableString();
		else
			streetAddress = null;
		location = new Location(new DerInputStream(adv[1].toByteArray()));
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		if(streetAddress!=null)
			dout.putPrintableString(streetAddress);
		else
			dout.putNull();
		location.encode(dout);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AVSData>\n";
		s += streetAddress + "\n";
		s += location + "\n";
		return s;
	}

	static void main(String[] args) {
		try 
		{
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "13470";
			String locationID = "DurenSawit";
			
			Location location = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			AVSData avs = new AVSData("Jl. Protokol",location);
			System.out.println(avs);

			DerOutputStream dout = new DerOutputStream();
			avs.encode(dout);
			HexDumpEncoder hde = new HexDumpEncoder();
			System.out.println(hde.encodeBuffer(dout.toByteArray()));

			FileOutputStream fo = new FileOutputStream("avsData.dat");
			fo.write(dout.toByteArray());
			fo.flush();
			fo.close();

			FileInputStream fi = new FileInputStream("avsData.dat");
			byte[] input = new byte[fi.available()];			
			fi.read(input);

			System.out.println("*******reconstruct********");
			avs = new AVSData (new DerInputStream(input));
			System.out.println(avs);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}
}