package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	1134 AuthHeader ::= SEQUENCE {
 *	1135    authAmt       CurrencyAmount,
 *	1136    authCode      AuthCode,
 *	1137    responseData  ResponseData,
 *	1138    batchStatus   [0] BatchStatus  OPTIONAL,
 *	1139    currConv      CurrConv  OPTIONAL          -- Merchant to cardholder
 *	1140 }
 *	</pre>
 */

public class AuthHeader implements DerEncoder {

	//fields
	public CurrencyAmount	authAmt;
	public AuthCode		authCode;
	public ResponseData	responseData;
	public BatchStatus	batchStatus;
	//public CurrConv		currConv;

	//Constructor
	public AuthHeader(CurrencyAmount 	authAmt,
				AuthCode	   	authCode,
				ResponseData	responseData,
				BatchStatus		batchStatus) throws Exception {
		this.authAmt = authAmt;
		this.authCode = authCode;
		this.responseData = responseData;
		this.batchStatus = batchStatus;
	}

	public AuthHeader(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		authAmt = new CurrencyAmount(din);
		authCode = new AuthCode(din);
		responseData = new ResponseData(din);
		if((byte)din.peekByte()==(byte)0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			batchStatus = new BatchStatus(new DerInputStream(dv.toByteArray()));
		}
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		authAmt.encode(dout);
		authCode.encode(dout);
		responseData.encode(dout);
		if(batchStatus!=null) {
			DerOutputStream temp = new DerOutputStream();
			batchStatus.encode(temp);
			dout.writeImplicit((byte)0xA0,temp);
		}
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthHeader>\n";
		s += "authAmt:\n" + authAmt;
		s += "authCode:\n" + authCode;
		s += "responseData:\n" + responseData;
		if(batchStatus!=null)
			s += "batchStatus:\n" + batchStatus;
		return s;
	}

	public static void main(String[] args) {
		try {
			AuthCode ac = new AuthCode(AuthCode.approved);
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			AuthValCodes avc = new AuthValCodes("dhany", AuthValCodes.AuthCharInd_directMarketing,
									"arif", AuthValCodes.MarketSpecDataID_hotel);
			
			ResponseData rd = new ResponseData(avc,new BigInt(8), new BigInt(1),
								     new BigInt(2), "accepted");

			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
		
			BatchDetails bd = new BatchDetails(bt,seq);

			ClosedWhen cw = new ClosedWhen(ClosedWhen.closedbyMerchant,
							     new Date());

			BatchStatus bs = new BatchStatus(new Date(),cw, bd);

			AuthHeader ah = new AuthHeader(cur,ac,rd,bs);
			System.out.println(ah);

			DerOutputStream dout = new DerOutputStream();
			ah.encode(dout);

			ah = new AuthHeader(new DerInputStream(dout.toByteArray()));
			System.out.println(ah);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}	