package digsec.set.payment.message.AuthReqRes;

import digsec.set.payment.component.*;
import java.io.*;
import sun.security.util.*;
import java.security.*;
import java.security.cert.*;
import java.io.*;
import java.util.*;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import sun.security.x509.X500Name;
import sun.misc.HexDumpEncoder;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.pkcs.AlgorithmId;
import digsec.set.core.crypto.DetachedDigest;
import digsec.set.core.crypto.EX;
import digsec.set.core.crypto.HMAC;
import digsec.set.core.crypto.FalseVerifiedException;
import digsec.set.core.encapsulation.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import au.net.aba.crypto.provider.*;
import cryptix.provider.Cryptix;
import digsec.set.payment.component.SaleDetail.*;
import digsec.set.payment.wrapper.*;

/**
 *	<pre> 
 *	983 AuthReq ::= EncB { M, P, AuthReqData, PI }
 *	</pre>
 */

public class AuthReq extends Message implements DerEncoder {

	//fields
	public AuthReqData	authReqData;
	public byte[]		pi;

	private EncB		encB;

	//constructor
	public AuthReq(X509Certificate m_cert, PrivateKey m_priv,
			   X509Certificate p_cert, AuthReqData ard,
			   byte[] pi) throws Exception {
		super();
		this.authReqData = ard;
		this.pi = pi;
		DerOutputStream dout = new DerOutputStream();		

		//make ard ContentInfo
		ard.encode(dout);		
		ContentInfo ardInfo = new ContentInfo(new DerInputStream(dout.toByteArray()));
		//make pi ContentInfo
		ContentInfo piInfo = new ContentInfo(new DerInputStream(pi));

		encB = new EncB(m_cert,m_priv,p_cert,ardInfo,piInfo);
	}

	public AuthReq(PrivateKey p_priv, DerInputStream din) throws Exception {
		
		DerValue dv = din.getDerValue();
		dv.resetTag((byte)0x30);
		din = dv.toDerInputStream();
		encB = new EncB(p_priv,din);
		ContentInfo ardInfo = encB.getContentInfo();
		ContentInfo piInfo = encB.getBaggage();

		DerOutputStream dout = new DerOutputStream();
		ardInfo.encode(dout);
		authReqData = new AuthReqData(new DerInputStream(dout.toByteArray()));
		dout.reset();
		piInfo.encode(dout);
		pi = dout.toByteArray();				
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		encB.derEncode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write(Message.tag_AuthReq,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthReq>\n";
		s += authReqData;
		HexDumpEncoder hde = new HexDumpEncoder();
		s += "PI:\n" + hde.encodeBuffer(pi) + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			//->->->->->->->->->make authReqItem<-<-<-<-<-<-<-<-<-<-<

			//**************make AuthTags******************
			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);

			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			byte[] lid_M = {2,1,0,3,1,7};
			Calendar calendar = Calendar.getInstance();
			calendar.set(1999,1,14);
			Date pReqDate = calendar.getTime();
			SecureRandom sr = new SecureRandom();	
			byte[] XID = new byte[20];
			sr.nextBytes(XID);

			TransIDs transIDs = new TransIDs(lid_C,lid_M, XID, pReqDate, null, "English");
			

			AuthTags au = new AuthTags(rr,transIDs,new BigInt(812));

			//**************make AuthReqPayload*************
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150), new BigInt(2));  
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			
			Location location = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			AVSData avs = new AVSData("Jl. Bacang",location);

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			MarketSpecAuthData market = new MarketSpecAuthData(new BigInt(812),
											   new BigInt(317));

			MerchData merch = new MerchData("dodol",
								   new BigInt(2));


			AuthReqPayload authRPL = new AuthReqPayload(false,cur,null, //avs,
										  new BigInt(0),
										  new BigInt(0),
										  true, null, //install,
										  market, merch);	
		
			//************make CheckDigests**************
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

		
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod2 = new HOD(hodi);

			//*******bikin transIDs*********
			//byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			//TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			//udah ada di atas: hod2

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod2,odSalt,challM,brandID,bin);

			//******baru deh bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);

			//******baru juga bikin CheckDigests*****
			CheckDigests cd = new CheckDigests(hoiData,hod2);


			AuthReqItem aRI = new AuthReqItem(au,cd,authRPL);

			//->->->->->->Make SaleDetail<-<-<-<-<-<-<-<-<-<-<-

			BatchID batchID = new BatchID (new BigInt(10));
			BatchSequenceNum batchSequenceNum = new BatchSequenceNum (new BigInt(8));
			PayRecurInd payRecurInd = new PayRecurInd (new BigInt(2));
			MerOrderNum merOrderNum = new MerOrderNum ("123456789");
			AuthCharInd authCharInd = new AuthCharInd (new BigInt(1));

			MarketSpecDataID marketSpecDataID = new MarketSpecDataID(new BigInt(1));

			String renterName = "Iman Budi Setiawan";

			// definisikan rentalLocation
			//CountryCode countryCode = new CountryCode (new BigInt(20));
			//String city = "Jakarta";
			//String stateProvince = "DKI Jaya";
			//String postalCode = "15225";
			//String locationID = "Bintaro";
			Location rentalLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime rentalDateTime = new DateTime(new Date(), true);
			AutoNoShow autoNoShow = new AutoNoShow (new BigInt(1));
			String rentalAgreementNumber = "7357715";
			String referenceNumber = "261";
			String insuranceType = "A";

			// definisikan autoRateInfo
			//CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";
			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);

			// definisikan returnLocation
			city = "Depok";
			stateProvince = "Jawa Barat";
			postalCode = "12235";
			locationID = "UI";
			Location returnLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime returnDateTime = new DateTime(new Date(), true);

			// definisikan autoCharges
			CurrencyAmount regularDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount lateReturnCharges  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			scale = new DistanceScale(new BigInt(0));
			dist = new BigInt(10);
			Distance totalDistance = new Distance (scale, dist);

			CurrencyAmount extraDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount insuranceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount fuelCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount autoTowingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount oneWayDropOffCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount violationsCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
		 	CurrencyAmount deliveryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			AutoCharges autoCharges = new AutoCharges (regularDistanceCharges,
				lateReturnCharges, totalDistance, extraDistanceCharges,
				insuranceCharges, fuelCharges, autoTowingCharges, oneWayDropOffCharges,
				telephoneCharges, violationsCharges, deliveryCharges, parkingCharges,
				otherCharges, totalTaxAmount, auditAdjustment);

			// buat objek marketAutoCap
			MarketAutoCap marketAutoCap = new MarketAutoCap (renterName, rentalLocation,
				rentalDateTime, autoNoShow, rentalAgreementNumber, referenceNumber,
				insuranceType, autoRateInfo, returnLocation, returnDateTime,
				autoCharges);

			MarketSpecCapData marketSpecCapData = new MarketSpecCapData (0, marketAutoCap);
			MarketSpecSaleData marketSpecSaleData = new MarketSpecSaleData (marketSpecDataID, marketSpecCapData);

			CurrencyAmount totalFreightShippingAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalDutyTariffAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String dutyTariffReference = "A";
			CurrencyAmount totalNationalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalLocalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalOtherTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String merchantTaxID = "B";
			String merchantDutyTariffRef = "C";
			String customerDutyTariffRef = "D";
			String summaryCommodityCode = "E";
			String merchantType = "F";
			
			ChargeInfo chargeInfo = new ChargeInfo (totalFreightShippingAmount, totalDutyTariffAmount, 
				dutyTariffReference, totalNationalTaxAmount, totalLocalTaxAmount,
				totalOtherTaxAmount, totalTaxAmount, merchantTaxID, merchantDutyTariffRef,
				customerDutyTariffRef, summaryCommodityCode, merchantType);

			countryCode = new CountryCode (new BigInt(20));
			city = "Surabaya";
			stateProvince = "East Java";
			postalCode = "723411";
			locationID = "Tower";
			
			Location merchantLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipFrom = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipTo = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			BigInt quantity = new BigInt(1);
			String unitOfMeasureCode = "AA";
			String descriptor = "DD";
			String commodityCode = "CC";
			String productCode = "AB";
			CurrencyAmount unitCost = new CurrencyAmount(new BigInt(110),new BigInt(150),new BigInt(2));
			CurrencyAmount netCost  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			boolean discountInd = false;
			CurrencyAmount discountAmount = new CurrencyAmount(new BigInt(130),new BigInt(150),new BigInt(2));
			CurrencyAmount nationalTaxAmount = new CurrencyAmount(new BigInt(140),new BigInt(150),new BigInt(2));
			BigInt nationalTaxRate = new BigInt(2);
			String nationalTaxType = "B";
			CurrencyAmount localTaxAmount = new CurrencyAmount(new BigInt(150),new BigInt(150),new BigInt(2));
			CurrencyAmount otherTaxAmount = new CurrencyAmount(new BigInt(160),new BigInt(150),new BigInt(2));
			CurrencyAmount itemTotalCost = new CurrencyAmount(new BigInt(170),new BigInt(150),new BigInt(2));

			Item item = new Item (quantity, unitOfMeasureCode, descriptor, commodityCode,
				productCode, unitCost, netCost, discountInd, discountAmount,
				nationalTaxAmount, nationalTaxRate, nationalTaxType,
				localTaxAmount, otherTaxAmount, itemTotalCost);

			Item[] itemArray = new Item[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = item;

			ItemSeq itemSeq = new ItemSeq(itemArray);

			CommercialCardData commercialCardData = new CommercialCardData (chargeInfo,
				merchantLocation, shipFrom, shipTo, itemSeq);

			String orderSummary = "DETAIL OF PAYMENT";
			String customerReferenceNumber = "333777888";
			Phone customerServicePhone = new Phone("7357715");
			boolean okToPrintPhoneInd = true;
			MsgExtensions saleExtensions = new MsgExtensions("END MESSAGE");


			// buat objek saleDetail
			SaleDetail saleDetail = new SaleDetail (batchID, batchSequenceNum, payRecurInd,
				merOrderNum, authCharInd, marketSpecSaleData, commercialCardData,
				orderSummary, customerReferenceNumber, customerServicePhone,
				okToPrintPhoneInd, saleExtensions);

			AuthReqData ard = new AuthReqData(aRI,false,saleDetail);
		
	//=+=+=+=+=+=+=+ B A T A S =+=+=+=+=+=+=+=+=+=+=+=

			Security.addProvider(new ABAProvider());
			Security.addProvider(new Cryptix());

			//get payment-gateway's certificate
			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\source\\digsec\\set\\application\\keystore\\pg.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate p_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get payment-gateway's private key
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\source\\digsec\\set\\application\\keystore\\pg.jks");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"PaymentGateway".toCharArray());
			PrivateKey p_priv = (PrivateKey) ks.getKey("PG", "PaymentGateway".toCharArray());

			//******bikin PANData*******
			byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANData panData = new PANData("1294", "199908", secret, ab);

			//*******bikin transIDs*********
			//byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			//TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			//byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			//byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			//String strOD = "Ini barang-barang yang kubeli";
			//byte[] od = strOD.getBytes();

			//CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			//*******ini odSalt*******
			//byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			//Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			//Recurring rec = new Recurring(new BigInt(28),
							//	cal.getTime());
			//InstallRecurData install = new InstallRecurData(rec);

			//HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod = new HOD(hodi);

			//*******bikin chall-M*********
			//byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			//String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			//String bin = "123456";

			//*******bikin Inputs*********
			Inputs inputs = new Inputs(hodi,cur);

			//*******bikin merchantID*********
			String merchantID = "Toko Bunga Selalu Sayang";

			//*******bikin transStain*********
			XID = transIDs.xid;
			ContentInfo cInfo = new ContentInfo(ContentInfo.DATA_OID,
						            new DerValue(DerValue.tag_OctetString,XID));		
			//byte[] secret = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			HMAC transStain = new HMAC(cInfo,secret);

			//*******bikin swIdent*********
			String swIdent = "Digsec SET ver 0.001";

			//*******bikin acqBackKeyData*********
			String backkey = "Ini ceritanya kunci yang";
			byte[] backKey = backkey.getBytes();

			ObjectIdentifier oid_desCBC = new ObjectIdentifier("1.3.14.3.2.7");
			AlgorithmId id_desCBC = new AlgorithmId(oid_desCBC,null);
			
			BackKeyData acqBackKeyData = new BackKeyData(backKey, id_desCBC);

			//******bikin OIData*******
			OIData oiData = new OIData(transIDs,rrpid,challC,hod,odSalt,challM,brandID,bin);

			//******bikin PIHead*******
			PIHead piHead = new PIHead(transIDs,inputs,merchantID,install,transStain,swIdent,acqBackKeyData);

			//******bikin PIOILink*******
			PIOILink pioiLink = new PIOILink(piHead, oiData);

			//******bikin PIDualSigned*******
			PI pi = new PI(p_cert, pioiLink, panData);

			DerOutputStream piDer = new DerOutputStream();
			pi.encode(piDer);

			AuthReq ar = new AuthReq(p_cert,p_priv,p_cert,ard,piDer.toByteArray());

			System.out.println(ar);

			DerOutputStream dout = new DerOutputStream();
			ar.encode(dout);

			ar = new AuthReq(p_priv, new DerInputStream(dout.toByteArray()));
			System.out.println(ar);
			System.out.println(ar.tag);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}	
}