package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.core.crypto.FalseVerifiedException;
import digsec.set.payment.component.*;
import digsec.set.payment.component.SaleDetail.*;
import java.util.*;

/** 
 *	<pre>
 * 	990    AuthReqData ::= SEQUENCE {
 *	991    authReqItem  AuthReqItem,
 *	992    mThumbs      [0] EXPLICIT Thumbs  OPTIONAL,
 *	993    captureNow   BOOLEAN DEFAULT FALSE,
 *	994    saleDetail   [1] SaleDetail  OPTIONAL
 *	995 } ( WITH COMPONENTS {..., captureNow (TRUE) } |
 *	996     WITH COMPONENTS {..., captureNow (FALSE), saleDetail ABSENT } )
 *	<pre>
 */

public class AuthReqData implements DerEncoder {

	//fields
	public AuthReqItem	authReqItem;
	public boolean		captureNow;
	public SaleDetail		saleDetail;

	public static ObjectIdentifier id_set_content_AuthReqTBE = new ObjectIdentifier("2.23.42.0.52");

	//constructor
	public AuthReqData (AuthReqItem authReqItem, boolean captureNow,
				  SaleDetail saleDetail) throws Exception {
		this.authReqItem = authReqItem;
		this.captureNow = captureNow;

		if(captureNow)
			this.saleDetail = null;
		else
			this.saleDetail = saleDetail;
	}

	public AuthReqData (DerInputStream din) throws Exception {

		ContentInfo cInfo = new ContentInfo(din);
		
		if(!cInfo.getContentType().equals(id_set_content_AuthReqTBE))	
			throw new FalseVerifiedException("AuthReqData");

		din = cInfo.getContent().toDerInputStream();

		authReqItem = new AuthReqItem(din);
		if((byte)din.peekByte()==(byte)0xA0) {
			din.getDerValue();
		}
		captureNow = din.getDerValue().getBoolean();
		if((byte)din.peekByte()==(byte)0xA1) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			saleDetail = new SaleDetail(new DerInputStream(dv.toByteArray()));
		}
	}
			
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		authReqItem.encode(dout);
		dout.putBoolean(captureNow);
		if(captureNow) {
		}
		else {
			DerOutputStream dout2 = new DerOutputStream();
			saleDetail.encode(dout2);
			dout.writeImplicit((byte)0xA1,dout2);
		}
		DerOutputStream temp = new DerOutputStream();
		temp.write((byte)48,dout);

		ContentInfo cInfo = new ContentInfo(id_set_content_AuthReqTBE,
								new DerValue(temp.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());		
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthReqData>\n";
		s += authReqItem;
		s += "captureNow: " + captureNow + "\n";
		if(saleDetail!=null)
			s += saleDetail;
		return s;
	}

	public static void main(String[] args) {

		try {
			//->->->->->->->->->make authReqItem<-<-<-<-<-<-<-<-<-<-<

			//**************make AuthTags******************
			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			AuthTags au = new AuthTags(rr,id,new BigInt(812));

			//**************make AuthReqPayload*************
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150), new BigInt(2));  
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			
			Location location = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			AVSData avs = new AVSData("Jl. Bacang",location);

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			MarketSpecAuthData market = new MarketSpecAuthData(new BigInt(812),
											   new BigInt(317));

			MerchData merch = new MerchData("dodol",
								   new BigInt(2));


			AuthReqPayload authRPL = new AuthReqPayload(false,cur,null, //avs,
										  new BigInt(0),
										  new BigInt(0),
										  true, null, //install,
										  market, merch);	
		
			//************make CheckDigests**************
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

		
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod2 = new HOD(hodi);

			//*******bikin transIDs*********
			//byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			//udah ada di atas: hod2

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod2,odSalt,challM,brandID,bin);

			//******baru deh bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);

			//******baru juga bikin CheckDigests*****
			CheckDigests cd = new CheckDigests(hoiData,hod2);


			AuthReqItem aRI = new AuthReqItem(au,cd,authRPL);

			//->->->->->->Make SaleDetail<-<-<-<-<-<-<-<-<-<-<-

			BatchID batchID = new BatchID (new BigInt(10));
			BatchSequenceNum batchSequenceNum = new BatchSequenceNum (new BigInt(8));
			PayRecurInd payRecurInd = new PayRecurInd (new BigInt(2));
			MerOrderNum merOrderNum = new MerOrderNum ("123456789");
			AuthCharInd authCharInd = new AuthCharInd (new BigInt(1));

			MarketSpecDataID marketSpecDataID = new MarketSpecDataID(new BigInt(1));

			String renterName = "Iman Budi Setiawan";

			// definisikan rentalLocation
			//CountryCode countryCode = new CountryCode (new BigInt(20));
			//String city = "Jakarta";
			//String stateProvince = "DKI Jaya";
			//String postalCode = "15225";
			//String locationID = "Bintaro";
			Location rentalLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime rentalDateTime = new DateTime(new Date(), true);
			AutoNoShow autoNoShow = new AutoNoShow (new BigInt(1));
			String rentalAgreementNumber = "7357715";
			String referenceNumber = "261";
			String insuranceType = "A";

			// definisikan autoRateInfo
			//CurrencyAmount cur = new CurrencyAmount(new BigInt(110),new BigInt(120),new BigInt(1));
			AutoApplicableRate autoApplicableRate = new AutoApplicableRate (0, cur);

			CurrencyAmount lateReturnHourlyRate = new CurrencyAmount(new BigInt(100),new BigInt(100),new BigInt(3));
			CurrencyAmount distanceRate = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			DistanceScale scale = new DistanceScale(new BigInt(1));
			BigInt dist = new BigInt(8);
			Distance freeDistance = new Distance (scale, dist);

			String vehicleClassCode="AB";
			String corporateID="DIGISEC";
			AutoRateInfo autoRateInfo = new AutoRateInfo (autoApplicableRate,
				lateReturnHourlyRate, distanceRate, freeDistance,
				vehicleClassCode, corporateID);

			// definisikan returnLocation
			city = "Depok";
			stateProvince = "Jawa Barat";
			postalCode = "12235";
			locationID = "UI";
			Location returnLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			DateTime returnDateTime = new DateTime(new Date(), true);

			// definisikan autoCharges
			CurrencyAmount regularDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount lateReturnCharges  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			scale = new DistanceScale(new BigInt(0));
			dist = new BigInt(10);
			Distance totalDistance = new Distance (scale, dist);

			CurrencyAmount extraDistanceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount insuranceCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount fuelCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount autoTowingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount oneWayDropOffCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount telephoneCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount violationsCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
		 	CurrencyAmount deliveryCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount parkingCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount otherCharges = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount auditAdjustment = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));

			AutoCharges autoCharges = new AutoCharges (regularDistanceCharges,
				lateReturnCharges, totalDistance, extraDistanceCharges,
				insuranceCharges, fuelCharges, autoTowingCharges, oneWayDropOffCharges,
				telephoneCharges, violationsCharges, deliveryCharges, parkingCharges,
				otherCharges, totalTaxAmount, auditAdjustment);

			// buat objek marketAutoCap
			MarketAutoCap marketAutoCap = new MarketAutoCap (renterName, rentalLocation,
				rentalDateTime, autoNoShow, rentalAgreementNumber, referenceNumber,
				insuranceType, autoRateInfo, returnLocation, returnDateTime,
				autoCharges);

			MarketSpecCapData marketSpecCapData = new MarketSpecCapData (0, marketAutoCap);
			MarketSpecSaleData marketSpecSaleData = new MarketSpecSaleData (marketSpecDataID, marketSpecCapData);

			CurrencyAmount totalFreightShippingAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalDutyTariffAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String dutyTariffReference = "A";
			CurrencyAmount totalNationalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalLocalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			CurrencyAmount totalOtherTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			totalTaxAmount = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			String merchantTaxID = "B";
			String merchantDutyTariffRef = "C";
			String customerDutyTariffRef = "D";
			String summaryCommodityCode = "E";
			String merchantType = "F";
			
			ChargeInfo chargeInfo = new ChargeInfo (totalFreightShippingAmount, totalDutyTariffAmount, 
				dutyTariffReference, totalNationalTaxAmount, totalLocalTaxAmount,
				totalOtherTaxAmount, totalTaxAmount, merchantTaxID, merchantDutyTariffRef,
				customerDutyTariffRef, summaryCommodityCode, merchantType);

			countryCode = new CountryCode (new BigInt(20));
			city = "Surabaya";
			stateProvince = "East Java";
			postalCode = "723411";
			locationID = "Tower";
			
			Location merchantLocation = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipFrom = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			Location shipTo = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			BigInt quantity = new BigInt(1);
			String unitOfMeasureCode = "AA";
			String descriptor = "DD";
			String commodityCode = "CC";
			String productCode = "AB";
			CurrencyAmount unitCost = new CurrencyAmount(new BigInt(110),new BigInt(150),new BigInt(2));
			CurrencyAmount netCost  = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));
			boolean discountInd = false;
			CurrencyAmount discountAmount = new CurrencyAmount(new BigInt(130),new BigInt(150),new BigInt(2));
			CurrencyAmount nationalTaxAmount = new CurrencyAmount(new BigInt(140),new BigInt(150),new BigInt(2));
			BigInt nationalTaxRate = new BigInt(2);
			String nationalTaxType = "B";
			CurrencyAmount localTaxAmount = new CurrencyAmount(new BigInt(150),new BigInt(150),new BigInt(2));
			CurrencyAmount otherTaxAmount = new CurrencyAmount(new BigInt(160),new BigInt(150),new BigInt(2));
			CurrencyAmount itemTotalCost = new CurrencyAmount(new BigInt(170),new BigInt(150),new BigInt(2));

			Item item = new Item (quantity, unitOfMeasureCode, descriptor, commodityCode,
				productCode, unitCost, netCost, discountInd, discountAmount,
				nationalTaxAmount, nationalTaxRate, nationalTaxType,
				localTaxAmount, otherTaxAmount, itemTotalCost);

			Item[] itemArray = new Item[5];
			for(int i=0; i<itemArray.length;i++)
				itemArray[i] = item;

			ItemSeq itemSeq = new ItemSeq(itemArray);

			CommercialCardData commercialCardData = new CommercialCardData (chargeInfo,
				merchantLocation, shipFrom, shipTo, itemSeq);

			String orderSummary = "DETAIL OF PAYMENT";
			String customerReferenceNumber = "333777888";
			Phone customerServicePhone = new Phone("7357715");
			boolean okToPrintPhoneInd = true;
			MsgExtensions saleExtensions = new MsgExtensions("END MESSAGE");


			// buat objek saleDetail
			SaleDetail saleDetail = new SaleDetail (batchID, batchSequenceNum, payRecurInd,
				merOrderNum, authCharInd, marketSpecSaleData, commercialCardData,
				orderSummary, customerReferenceNumber, customerServicePhone,
				okToPrintPhoneInd, saleExtensions);

			AuthReqData ard = new AuthReqData(aRI,false,saleDetail);
			System.out.println(ard);

			DerOutputStream dout = new DerOutputStream();
			ard.encode(dout);

			ard = new AuthReqData(new DerInputStream(dout.toByteArray()));
			System.out.println(ard);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}