package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.payment.component.*;
import java.util.*;

/**
 *	<pre>
 *	998 AuthReqItem ::= SEQUENCE {
 *	999    authTags        AuthTags,
 *	1000    checkDigests    [0] CheckDigests  OPTIONAL,
 *	1001    authReqPayload  AuthReqPayload
 *	1002 }
 *	</pre>
 */

public class AuthReqItem implements DerEncoder {

	//fields
	public AuthTags		authTags;
	public CheckDigests 	checkDigests;	
	public AuthReqPayload	authReqPayload;
	
	//constructor
	public AuthReqItem(AuthTags authTags, CheckDigests checkDigests,
				 AuthReqPayload authReqPayload) throws Exception {

		this.authTags = authTags;
		this.checkDigests = checkDigests;
		this.authReqPayload = authReqPayload;
	}

	public AuthReqItem(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		authTags = new AuthTags(din);
		if((byte)din.peekByte() == (byte)0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			checkDigests = new CheckDigests(new DerInputStream(dv.toByteArray()));
		}
		authReqPayload = new AuthReqPayload(din);
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		authTags.encode(dout);
		if(checkDigests!=null) {
			DerOutputStream dout1 = new DerOutputStream();
			checkDigests.encode(dout1);
			dout.writeImplicit((byte)0xA0,dout1);
		}
		authReqPayload.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthReqItem>\n";
		s += authTags + "\n";
		if(checkDigests!=null)
			s += checkDigests + "\n";
		s += authReqPayload + "\n";
		return s;
	}	

	public static void main(String[] args) {

		try {
			//**************make AuthTags******************
			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			AuthTags au = new AuthTags(rr,id,new BigInt(812));

			//**************make AuthReqPayload*************
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150), new BigInt(2));  
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			
			Location location = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			AVSData avs = new AVSData("Jl. Bacang",location);

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			MarketSpecAuthData market = new MarketSpecAuthData(new BigInt(812),
											   new BigInt(317));

			MerchData merch = new MerchData("dodol",
								   new BigInt(2));


			AuthReqPayload authRPL = new AuthReqPayload(false,cur,null, //avs,
										  new BigInt(0),
										  new BigInt(0),
										  true, null, //install,
										  market, merch);	
		
			//************make CheckDigests**************
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

		
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod2 = new HOD(hodi);

			//*******bikin transIDs*********
			//byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			//udah ada di atas: hod2

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod2,odSalt,challM,brandID,bin);

			//******baru deh bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);

			//******baru juga bikin CheckDigests*****
			CheckDigests cd = new CheckDigests(hoiData,hod2);


			AuthReqItem aRI = new AuthReqItem(au,cd,authRPL);
			System.out.println(aRI);

			DerOutputStream dout = new DerOutputStream();
			aRI.encode(dout);

			aRI = new AuthReqItem(new DerInputStream(dout.toByteArray()));
			System.out.println(aRI);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}