package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.payment.component.*;
import java.util.*;

/**
 *	<pre>
 *	1015 AuthReqPayload ::= SEQUENCE {
 *	1016    subsequentAuthInd   BOOLEAN DEFAULT FALSE,
 *	1017    authReqAmt          CurrencyAmount,        -- May differ from PurchAmt
 *	1018    avsData             [0] AVSData  OPTIONAL,
 *	1019    specialProcessing   [1] SpecialProcessing  OPTIONAL,
 *	1020    cardSuspect         [2] CardSuspect  OPTIONAL,
 *	1021    requestCardTypeInd  BOOLEAN DEFAULT FALSE,
 *	1022    installRecurData    [3] InstallRecurData  OPTIONAL,
 *	1023    marketSpecAuthData  [4] EXPLICIT MarketSpecAuthData  OPTIONAL,
 *	1024    merchData           MerchData,
 *	1025    aRqExtensions       [5] MsgExtensions {{ARqExtensionsIOS}} OPTIONAL
 *	1026 }
 *
 *	1035 SpecialProcessing ::= ENUMERATED {
 *	1036    directMarketing    (0),
 *	1037    preferredCustomer  (1)
 *	1038 }
 *
 *	1040 CardSuspect ::= ENUMERATED {   -- Indicates merchant suspects cardholder
 *	1041    --
 *	1042    -- Specific values indicate why the merchant is suspicious
 *	1043    --
 *	1044    unspecifiedReason  (0)  -- Either the merchant does not differentiate
 *	1045                            -- reasons for suspicion, or the specific
 *	1046                            -- reason does not appear in the list
 *	1047 }
 *	</pre>
 */

public class AuthReqPayload implements DerEncoder {

	//fields
	public boolean			subsequentAuthInd = false;
	public CurrencyAmount		authReqAmt;
	public AVSData			avsData;
	public BigInt			specialProcessing;
	public BigInt			cardSuspect;
	public boolean			requestCardTypeInd = false;
	public InstallRecurData 	installRecurData;
	public MarketSpecAuthData 	marketSpecAuthData;
	public MerchData			merchData;
	//public ArqExtensions		argExtensions;

	//constructor
	public AuthReqPayload(	boolean 			subsequentAuthInd,
				    	CurrencyAmount 		authReqAmt,
					AVSData 			avsData,
					BigInt	 		specialProcessing,
					BigInt			cardSuspect,
					boolean			requestCardTypeInd,
					InstallRecurData  	installRecurData,
					MarketSpecAuthData 	marketSpecAuthData,
					MerchData			merchData ) throws Exception {
		
		this.subsequentAuthInd = subsequentAuthInd;
		this.authReqAmt = authReqAmt;
		this.avsData = avsData;
		this.specialProcessing = specialProcessing;
		this.cardSuspect = cardSuspect;
		this.requestCardTypeInd = requestCardTypeInd;
		this.installRecurData = installRecurData;
		this.marketSpecAuthData = marketSpecAuthData;
		this.merchData = merchData;
	}

	public AuthReqPayload(DerInputStream din) throws Exception {
		DerValue dv = din.getDerValue();
		din = dv.toDerInputStream();	  
		subsequentAuthInd = din.getDerValue().getBoolean();
		authReqAmt = new CurrencyAmount(din);
		if((byte) din.peekByte() == (byte)0xA0) {
			dv = din.getDerValue();
			dv.resetTag((byte)48);
			avsData = new AVSData(new DerInputStream(dv.toByteArray()));
		}
		if((byte) din.peekByte() == (byte)0x81) { 
			dv = din.getDerValue();
			dv.resetTag(DerValue.tag_Enumerated);
			specialProcessing = new BigInt(dv.getEnumerated().toInt());
		}
		if((byte) din.peekByte() == (byte)0x82) { 
			dv = din.getDerValue();
			dv.resetTag(DerValue.tag_Enumerated);
			cardSuspect = new BigInt(dv.getEnumerated().toInt());
		}
		requestCardTypeInd = din.getDerValue().getBoolean();
		if((byte) din.peekByte() == (byte)0xA3) {
			dv = din.getDerValue();
			dv.resetTag((byte)48);
			installRecurData = new InstallRecurData(new DerInputStream(dv.toByteArray()));
		}
		if((byte) din.peekByte() == (byte)0xA4) {
			DerValue dv1 = din.getDerValue();
			dv1.resetTag((byte)48);
			marketSpecAuthData = new MarketSpecAuthData(dv1.toDerInputStream());
		}
		merchData = new MerchData(din);
	} 
	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putBoolean(subsequentAuthInd);
		authReqAmt.encode(dout);		
		if(avsData!=null) {
			DerOutputStream dout1 = new DerOutputStream();
			avsData.encode(dout1);
			dout.writeImplicit((byte)0xA0,dout1);
		}
		
		if(specialProcessing!=null) {
			DerOutputStream dout2 = new DerOutputStream();
			dout2.putEnumerated(specialProcessing);
			dout.writeImplicit((byte)0x81,dout2);
		}
		
		if(cardSuspect!=null) {
			DerOutputStream dout3 = new DerOutputStream();
			dout3.putEnumerated(cardSuspect);
			dout.writeImplicit((byte)0x82,dout3);
		}
		
		dout.putBoolean(requestCardTypeInd);

		if(installRecurData!=null) {
			DerOutputStream dout4 = new DerOutputStream();
			installRecurData.derEncode(dout4);
			dout.writeImplicit((byte)0xA3,dout4);
		}
		
		if(marketSpecAuthData!=null) {
			DerOutputStream dout5 = new DerOutputStream();
			marketSpecAuthData.encode(dout5);
			dout.write((byte)0xA4,dout5);
		}
		
		merchData.encode(dout);

		DerOutputStream doutFinal = new DerOutputStream();
		doutFinal.write((byte)48,dout);
		out.write(doutFinal.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthReqPayload>\n";
		s += "subsequentAuthInd: " + subsequentAuthInd + "\n";
		s += "authReqAmt: \n" + authReqAmt + "\n";
		if(avsData != null)
			s += avsData + "\n";
		if(specialProcessing != null)
			s += "specialProcessing: " + specialProcessing + "\n";
		if(cardSuspect != null)
			s += "cardSuspect: " + cardSuspect + "\n";
		s += "requestCardTypeInd: " + requestCardTypeInd + "\n";
		if(installRecurData != null)
			s += installRecurData + "\n";
		if(marketSpecAuthData != null)
			s += marketSpecAuthData + "\n";
		s += merchData + "\n";
		return s;
	}	

	public static void main(String[] args) {
		try {
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150), new BigInt(2));  
			CountryCode countryCode = new CountryCode (new BigInt(20));
			String city = "Jakarta";
			String stateProvince = "DKI Jaya";
			String postalCode = "15225";
			String locationID = "Bintaro";
			
			Location location = new Location (countryCode, city, stateProvince,
				postalCode, locationID);

			AVSData avs = new AVSData("Jl. Bacang",location);

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			MarketSpecAuthData market = new MarketSpecAuthData(new BigInt(812),
											   new BigInt(317));

			MerchData merch = new MerchData("dodol",
								   new BigInt(2));


			AuthReqPayload authRPL = new AuthReqPayload(false,cur,null, //avs,
										  new BigInt(0),
										  new BigInt(0),
										  true, null, //install,
										  market, merch);	
			System.out.println(authRPL);
			DerOutputStream dout = new DerOutputStream();
			authRPL.encode(dout);
			authRPL = new AuthReqPayload(new DerInputStream(dout.toByteArray()));
			System.out.println(authRPL);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}