package digsec.set.payment.message.AuthReqRes;

import sun.security.util.*;
import java.io.*;
import java.security.*;
import java.security.cert.*;
import digsec.set.core.pkcs.*;
import digsec.set.core.encapsulation.*;
import digsec.set.core.crypto.*;
import digsec.set.payment.component.*;
import java.util.*;

/**
 *	<pre>
 *	1096 AuthResBaggage ::= SEQUENCE {
 *	1097    capToken    [0] EXPLICIT CapToken  OPTIONAL,
 *	1098    acqCardMsg  [1] EXPLICIT AcqCardMsg  OPTIONAL,
 *	1099    authToken   [2] EXPLICIT AuthToken  OPTIONAL
 *	1100 }
 *	</pre>
 */

public class AuthResBaggage implements DerEncoder {

	//fields
	public CapToken	capToken;
	public AuthToken 	authToken;

	public static ObjectIdentifier id_set_content_AuthResBaggage = 
				new ObjectIdentifier("2.23.42.0.8");

	//constructor
	public AuthResBaggage(CapToken capToken, AuthToken authToken) throws Exception {
		this.capToken = capToken;
		this.authToken = authToken;
	}

	public AuthResBaggage(PrivateKey priv, DerInputStream din) throws Exception {
		ContentInfo cInfo = new ContentInfo(din);
		ObjectIdentifier oid = cInfo.getContentType();
		if(!oid.equals(AuthResBaggage.id_set_content_AuthResBaggage))
			throw new FalseVerifiedException("AuthResBaggage");
		din = cInfo.getContent().toDerInputStream();
		
		if((byte)din.peekByte()==(byte)0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			capToken = new CapToken(priv, dv.toDerInputStream());
		}
		if((byte)din.peekByte()==(byte)0xA2) {
			DerValue dv2 = din.getDerValue();
			dv2.resetTag((byte)48);
			authToken = new AuthToken(priv, dv2.toDerInputStream());
		}
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream dout2 = new DerOutputStream();		

		if(capToken!=null) {
 			capToken.encode(dout2);
			dout.write((byte)0xA0,dout2);
		}

		if(authToken!=null) {
			dout2.reset();
			authToken.encode(dout2);
			dout.write((byte)0xA2,dout2);
		}

		dout2.reset();
		dout2.write((byte)48,dout);

		ContentInfo cInfo = new ContentInfo(id_set_content_AuthResBaggage,
								new DerValue(dout2.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthResBaggage>\n";
		if(capToken!=null)
			s += capToken;
		if(authToken!=null)
			s += authToken;
		return s;
	}

	public static void main(String[] args) {
		try {
			//get payment-gateway's certificate
			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate p_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get payment-gateway's private key
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.jks");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"PaymentGateway".toCharArray());
			PrivateKey p_priv = (PrivateKey) ks.getKey("PG", "PaymentGateway".toCharArray());

			byte[] authAmt = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  

			CapTokenData ctd = new CapTokenData(authAmt,cur);

			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);

			CapToken ct = new CapToken(p_cert,p_priv,p_cert,ctd,pToken);

			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");
			
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);
			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			AuthTokenData auth = new AuthTokenData(id,cur,"dodol812",install,
									   new BigInt(1), new Date(), 
									   null);
					
			AuthToken authToken = new AuthToken(p_priv,p_cert,p_cert,auth,pToken);

			AuthResBaggage arb = new AuthResBaggage(ct,authToken);
			System.out.println(arb);

			DerOutputStream dout = new DerOutputStream();
			arb.encode(dout);
			
			arb = new AuthResBaggage(p_priv, new DerInputStream(dout.toByteArray()));
			System.out.println(arb);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}	