package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import java.util.Date;
import digsec.set.payment.component.*;
import digsec.set.core.pkcs.ContentInfo;

/**
 *	<pre>
 *	1089 AuthResData ::= SEQUENCE {
 *	1090    authTags            AuthTags,
 *	1091    brandCRLIdentifier  [0] EXPLICIT BrandCRLIdentifier  OPTIONAL,
 *	1092    peThumb             [1] EXPLICIT CertThumb  OPTIONAL,
 *	1093    authResPayload      AuthResPayload
 *	1094 }
 *	</pre>
 */

public class AuthResData implements DerEncoder {

	//fields
	public AuthTags		authTags;
	public AuthResPayload 	authResPayload;
	public static ObjectIdentifier id_set_content_AuthResTBE = new ObjectIdentifier("2.23.42.0.53");	

	//constructor
	public AuthResData(AuthTags authTags, AuthResPayload authResPayload) throws Exception {
		this.authTags = authTags;
		this.authResPayload = authResPayload;
	}

	public AuthResData(DerInputStream din) throws Exception {
		ContentInfo cInfo = new ContentInfo(din);
		din = cInfo.getContent().toDerInputStream();
		authTags = new AuthTags(din);
		authResPayload = new AuthResPayload(din);
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		authTags.encode(dout);
		authResPayload.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);

		ContentInfo cInfo = new ContentInfo(id_set_content_AuthResTBE,
								new DerValue(dout2.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthResData>\n";
		s += authTags;
		s += authResPayload;
		return s;
	}

	public static void main(String[] args) {
		try {
			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");

			AuthTags au = new AuthTags(rr,id,new BigInt(812));

			AuthCode ac = new AuthCode(AuthCode.approved);
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			AuthValCodes avc = new AuthValCodes("dhany", AuthValCodes.AuthCharInd_directMarketing,
									"arif", AuthValCodes.MarketSpecDataID_hotel);
			
			ResponseData rd = new ResponseData(avc,new BigInt(8), new BigInt(1),
								     new BigInt(2), "accepted");

			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
		
			BatchDetails bd = new BatchDetails(bt,seq);

			ClosedWhen cw = new ClosedWhen(ClosedWhen.closedbyMerchant,
							     new Date());

			BatchStatus bs = new BatchStatus(new Date(),cw, bd);

			AuthHeader ah = new AuthHeader(cur,ac,rd,bs);

			CapResPayload crp = new CapResPayload(new BigInt(0),cur,new BigInt(812), new BigInt(317));
		
			AuthResPayload arp = new AuthResPayload(ah,crp);

			AuthResData ard = new AuthResData(au,arp);

			System.out.println(ard);
			DerOutputStream dout = new DerOutputStream();
			ard.encode(dout);

			ard = new AuthResData(new DerInputStream(dout.toByteArray()));
			System.out.println(ard);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}