package digsec.set.payment.message.AuthReqRes;

import sun.security.util.*;
import java.io.*;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	1126 AuthResPayload ::= SEQUENCE {
 *	1127    authHeader     AuthHeader,
 *	1128    capResPayload  CapResPayload  OPTIONAL,
 *	1129    aRsExtensions  [0] MsgExtensions {{ARsExtensionsIOS}} OPTIONAL
 *	1130 }
 *	</pre>
 */

public class AuthResPayload implements DerEncoder {

	//fields
	public AuthHeader 	authHeader;
	public CapResPayload	capResPayload;

	//constructor
	public AuthResPayload(AuthHeader authHeader, CapResPayload capResPayload) throws Exception {
		this.authHeader = authHeader;
		this.capResPayload = capResPayload;
	}

	public AuthResPayload(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		authHeader = new AuthHeader(din);

		if((byte) din.peekByte() == (byte)0x30) {
			capResPayload = new CapResPayload(din);
		}
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		authHeader.encode(dout);
		if(capResPayload!=null)
			capResPayload.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthResPayload>\n";
		s += authHeader;
		if(capResPayload!=null)
			s += capResPayload;
		return s;
	}

	public static void main(String[] args) {
		try{

			AuthCode ac = new AuthCode(AuthCode.approved);
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			AuthValCodes avc = new AuthValCodes("dhany", AuthValCodes.AuthCharInd_directMarketing,
									"arif", AuthValCodes.MarketSpecDataID_hotel);
			
			ResponseData rd = new ResponseData(avc,new BigInt(8), new BigInt(1),
								     new BigInt(2), "accepted");

			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
		
			BatchDetails bd = new BatchDetails(bt,seq);

			ClosedWhen cw = new ClosedWhen(ClosedWhen.closedbyMerchant,
							     new Date());

			BatchStatus bs = new BatchStatus(new Date(),cw, bd);

			AuthHeader ah = new AuthHeader(cur,ac,rd,bs);

			CapResPayload crp = new CapResPayload(new BigInt(0),cur,new BigInt(812), new BigInt(317));
		
			AuthResPayload arp = new AuthResPayload(ah,crp);
	
			System.out.println(arp);

			DerOutputStream dout = new DerOutputStream();
			arp.encode(dout);

			arp = new AuthResPayload(new DerInputStream(dout.toByteArray()));
			System.out.println(arp);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}