package digsec.set.payment.message.AuthReqRes;

import digsec.set.payment.component.*;
import sun.security.util.*;
import java.io.*;


/**
 *	<pre>
 *	1004 AuthTags ::= SEQUENCE {
 *	1005    authRRTags  RRTags,
 *	1006    transIDs    TransIDs,
 *	1007    authRetNum  AuthRetNum  OPTIONAL
 *	1008 }
 *
 *	1259 AuthRetNum ::= INTEGER (0..MAX)
 *	</pre>
 */

public class AuthTags implements DerEncoder {

	//fields
	public RRTags 	authRRTags;
	public TransIDs 	transIDs;
	public BigInt	authRetNum;

	//constructor
	public AuthTags(RRTags authRRTags, TransIDs transIDs, 
			    BigInt authRetNum) throws Exception {

		this.authRRTags = authRRTags;
		this.transIDs   = transIDs;
		this.authRetNum = authRetNum;
	}

	public AuthTags(DerInputStream din) throws Exception {
	
		din = din.getDerValue().toDerInputStream();
		
		authRRTags = new RRTags(din);
		transIDs   = new TransIDs(din);
		if(din.available()>0)
			authRetNum = new BigInt(din.getInteger().toInt());
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		authRRTags.derEncode(dout);
		transIDs.derEncode(dout);				
		if(authRetNum!=null)
			dout.putInteger(authRetNum);
		
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}	

	public String toString() {
		String s = "<AuthTags>\n";
		s += authRRTags + "\n";
		s += transIDs + "\n";
		if(authRetNum!=null)
			s += "authRetNum: " + authRetNum + "\n";
		return s;
	}		

	public static void main (String[] args) {
		try {
			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);
			byte[] lid_C = {8,1,2,3,1,7};
			TransIDs id = TransIDs.getExample();
			AuthTags au = new AuthTags(rr,id,null); //new BigInt(812));
			System.out.println(au);
			DerOutputStream dout = new DerOutputStream();
			au.encode(dout);
			au = new AuthTags(new DerInputStream(dout.toByteArray()));
			System.out.println(au);

		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
		

