package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;

/**
 *	<pre>
 *	1170 AuthValCodes ::= SEQUENCE {
 *	1171    approvalCode    [0] ApprovalCode  OPTIONAL,
 *	1172    authCharInd     [1] AuthCharInd  OPTIONAL,
 *	1173    validationCode  [2] ValidationCode  OPTIONAL,
 *	1174    marketSpec      MarketSpecDataID  OPTIONAL
 *	1175 }
 *
 *	1215 ApprovalCode ::= VisibleString (SIZE(ub-approvalCode))
 *
 *	1217 AuthCharInd ::= ENUMERATED {
 *	1218    directMarketing      (0),
 *	1219    recurringPayment     (1),
 *	1220    addressVerification  (2),
 *	1221    preferredCustomer    (3),
 *	1222    incrementalAuth      (4)
 *	1223 }
 *
 *
 *	1225 ValidationCode ::= VisibleString (SIZE(ub-validationCode))
 *
 *	1897 MarketSpecDataID ::= ENUMERATED {
 *	1898   failedEdit  (0),
 *	1899   auto        (1),
 *	1900   hotel       (2),
 *	1901   transport   (3)
 *	1902 }
 *	</pre>
 */

public class AuthValCodes implements DerEncoder {

	//fields
	public String	approvalCode;
	public BigInt	authCharInd;
	public String	validationCode;
	public BigInt	marketSpecDataID;

	//constant
	public static BigInt AuthCharInd_directMarketing = new BigInt(0);
	public static BigInt AuthCharInd_recurringPayment = new BigInt(1);
	public static BigInt AuthCharInd_addressVerification = new BigInt(2);
	public static BigInt AuthCharInd_preferedCustomer = new BigInt(3);
	public static BigInt AuthCharInd_incrementalAuth = new BigInt(4);

	public static BigInt MarketSpecDataID_failedEdit = new BigInt(0);
	public static BigInt MarketSpecDataID_auto = new BigInt(1);
	public static BigInt MarketSpecDataID_hotel = new BigInt(2);
	public static BigInt MarketSpecDataID_transport = new BigInt(3);
 
	//constructor
	public AuthValCodes(String approvalCode, BigInt authCharInd,
				  String validationCode, BigInt marketSpecDataID) throws Exception {

		this.approvalCode = approvalCode;
		this.authCharInd = authCharInd;
		this.validationCode = validationCode;
		this.marketSpecDataID = marketSpecDataID;
	}

	public AuthValCodes(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		DerValue dv;
		if((byte)din.peekByte()==(byte)0x80) {
			dv=din.getDerValue();
			dv.resetTag(DerValue.tag_PrintableString);
			approvalCode=dv.getPrintableString();
		}
		if((byte)din.peekByte()==(byte)0x81) {
			dv=din.getDerValue();
			dv.resetTag(DerValue.tag_Enumerated);
			authCharInd=dv.getEnumerated();
		}
		if((byte)din.peekByte()==(byte)0x82) {
			dv=din.getDerValue();
			dv.resetTag(DerValue.tag_PrintableString);
			validationCode=dv.getPrintableString();
		}
		if(din.available()!=0) {
			marketSpecDataID = din.getEnumerated();
		}
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		if(approvalCode!=null) {
			DerOutputStream dout1 = new DerOutputStream();
			dout1.putPrintableString(approvalCode);
			dout.writeImplicit((byte)0x80,dout1);
		}
		if(authCharInd!=null) {
			DerOutputStream dout2 = new DerOutputStream();
			dout2.putEnumerated(authCharInd);
			dout.writeImplicit((byte)0x81,dout2);
		}
		if(validationCode!=null) {
			DerOutputStream dout3 = new DerOutputStream();
			dout3.putPrintableString(validationCode);
			dout.writeImplicit((byte)0x82,dout3);
		}
		if(marketSpecDataID!=null) {
			dout.putEnumerated(marketSpecDataID);
		}

		DerOutputStream finalDout = new DerOutputStream();
		finalDout.write((byte)48,dout);
		out.write(finalDout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<AuthValCodes>\n";
		if(approvalCode!=null)
			s += "approvalCode: " + approvalCode + "\n"; 	
		if(approvalCode!=null)
			s += "authCharInd: " + authCharInd + "\n";
		if(validationCode!=null)
			s += "validationCode: " + validationCode + "\n";
		if(marketSpecDataID!=null)
			s += "marketSpecDataID: " + marketSpecDataID + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			AuthValCodes avc = new AuthValCodes("dhany", AuthValCodes.AuthCharInd_directMarketing,
									"arif", AuthValCodes.MarketSpecDataID_hotel);
			System.out.println(avc);
			DerOutputStream dout = new DerOutputStream();
			avc.encode(dout);
			avc = new AuthValCodes(new DerInputStream(dout.toByteArray()));
			System.out.println(avc);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}
			








