package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	1727 BatchDetails ::= SEQUENCE {
 *	1728    batchTotals           BatchTotals,
 *	1729    brandBatchDetailsSeq  BrandBatchDetailsSeq  OPTIONAL
 *	1730 }
 *	</pre>
 */

public class BatchDetails implements DerEncoder {

	//fields
	public BatchTotals		batchTotals;
	public BrandBatchDetailsSeq	brandBatchDetailsSeq;

	//constructor
	public BatchDetails(BatchTotals batchTotals, 
				  BrandBatchDetailsSeq brandBatchDetailsSeq) throws Exception {

		this.batchTotals = batchTotals;
		this.brandBatchDetailsSeq = brandBatchDetailsSeq;
	}	

	public BatchDetails(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		batchTotals = new BatchTotals(din);
		if(din.available()!=0) {
			brandBatchDetailsSeq = new BrandBatchDetailsSeq(din);
		}
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		batchTotals.encode(dout);
		if(brandBatchDetailsSeq!=null)
			brandBatchDetailsSeq.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<BatchDetails>\n";
		s += batchTotals;
		if(brandBatchDetailsSeq!=null)
			s += brandBatchDetailsSeq;
		return s;
	}

	public static void main(String[] args) {
		try {
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
		
			BatchDetails bd = new BatchDetails(bt,seq);
			System.out.println(bd);
			DerOutputStream dout = new DerOutputStream();
			bd.encode(dout);
			bd = new BatchDetails(new DerInputStream(dout.toByteArray()));
			System.out.println(bd);

		}
		catch(Exception e) {
			e.printStackTrace();
		}

	}
}