package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import java.util.Date;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	1718 BatchStatus ::= SEQUENCE {
 *	1719    openDateTime     Date,
 *	1720    closedWhen       [0] ClosedWhen  OPTIONAL,
 *	1721    batchDetails     BatchDetails,
 *	1722    batchExtensions  [1] MsgExtensions {{BSExtensionsIOS}} OPTIONAL
 *	1723 }
 *	</pre>
 */

public class BatchStatus implements DerEncoder {

	//fields
	public Date			openDateTime;
	public ClosedWhen 	closedWhen;
	public BatchDetails	batchDetails;

	//constructor
	public BatchStatus(Date openDateTime, ClosedWhen closedWhen, 
				 BatchDetails batchDetails) throws Exception {
		this.openDateTime = openDateTime;
		this.closedWhen = closedWhen;
		this.batchDetails = batchDetails;
	}

	public BatchStatus(DerInputStream din) throws Exception {

		din = din.getDerValue().toDerInputStream();
		openDateTime = din.getGeneralizedTime();
		
		if((byte)din.peekByte() == (byte) 0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			closedWhen = new ClosedWhen(new DerInputStream(dv.toByteArray()));
		}

		batchDetails = new BatchDetails(din);
	}

	public void  derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putGeneralizedTime(openDateTime);
		if(closedWhen!=null) {
			DerOutputStream temp = new DerOutputStream();
			closedWhen.encode(temp);
			dout.writeImplicit((byte)0xA0,temp);
		}
		batchDetails.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<BatchStatus>\n";
		s += openDateTime + "\n";
		if(closedWhen!=null)
			s += closedWhen;
		s += batchDetails;
		return s;
	}	

	public static void main(String[] args) {
		try{
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
		
			BatchDetails bd = new BatchDetails(bt,seq);

			ClosedWhen cw = new ClosedWhen(ClosedWhen.closedbyMerchant,
							     new Date());

			BatchStatus bs = new BatchStatus(new Date(),cw, bd);
			System.out.println(bs);

			DerOutputStream dout = new DerOutputStream();
			bs.encode(dout);

			bs = new BatchStatus(new DerInputStream(dout.toByteArray()));
			System.out.println(bs); 

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}