package digsec.set.payment.message.AuthReqRes;

import digsec.set.payment.component.CurrencyAmount;
import sun.security.util.*;
import java.io.*;

/**
 *	<pre>
 *	1739 BatchTotals ::= SEQUENCE {
 *	1740    transactionCountCredit     INTEGER (0..MAX),
 *	1741    transactionTotalAmtCredit  CurrencyAmount,
 *	1742    transactionCountDebit      INTEGER (0..MAX),
 *	1743    transactionTotalAmtDebit   CurrencyAmount,
 *	1744    batchTotalExtensions       [0] MsgExtensions {{BTExtensionsIOS}} OPTIONAL
 *	1745 }
 *	</pre>
 */

public class BatchTotals implements DerEncoder {

	//fields
	public BigInt		transactionCountCredit;
	public CurrencyAmount	transactionTotalAmtCredit;
	public BigInt		transactionCountDebit;
	public CurrencyAmount	transactionTotalAmtDebit;

	//constructor
	public BatchTotals(BigInt 		transactionCountCredit,
				 CurrencyAmount	transactionTotalAmtCredit,
				 BigInt		transactionCountDebit,
				 CurrencyAmount 	transactionTotalAmtDebit) throws Exception {

		this.transactionCountCredit = transactionCountCredit;
		this.transactionTotalAmtCredit = transactionTotalAmtCredit;
		this.transactionCountDebit = transactionCountDebit;
		this.transactionTotalAmtDebit = transactionTotalAmtDebit;
	}

	public BatchTotals(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(4);
		transactionCountCredit = new BigInt(adv[0].getInteger().toInt());
		transactionTotalAmtCredit = new CurrencyAmount(new DerInputStream(adv[1].toByteArray()));
		transactionCountDebit = new BigInt(adv[2].getInteger().toInt());
		transactionTotalAmtDebit = new CurrencyAmount(new DerInputStream(adv[3].toByteArray()));
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger(transactionCountCredit);
		transactionTotalAmtCredit.encode(dout);
		dout.putInteger(transactionCountDebit);
		transactionTotalAmtDebit.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());	
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<BatchTotals>\n";
		s += "transactionCountCredit: " + transactionCountCredit + "\n";
		s += "transactionTotalAmtCredit:\n" + transactionTotalAmtCredit;
		s += "transactionCountDebit: " + transactionCountDebit + "\n";
		s += "transactionTotalAmtDebit:\n" + transactionTotalAmtDebit;
		return s;
	}

	public static void main(String[] args) {
		try {
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);
			System.out.println(bt);
			DerOutputStream dout = new DerOutputStream();
			bt.encode(dout);
			bt = new BatchTotals(new DerInputStream(dout.toByteArray()));
			System.out.println(bt);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}