package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	1732 BrandBatchDetailsSeq ::= SEQUENCE SIZE(1..MAX) OF BrandBatchDetails
 *
 *	1734 BrandBatchDetails ::= SEQUENCE {
 *	1735    brandID      BrandID,
 *	1736    batchTotals  BatchTotals
 *	1737 }
 *
 *	232 BrandID ::= SETString { ub-BrandID }
 *	</pre>
 */

public class BrandBatchDetailsSeq implements DerEncoder {

	//fields
	public BrandBatchDetails[] brandBatchDetails;

	//constructor
	public BrandBatchDetailsSeq(BrandBatchDetails[] brandBatchDetails) throws Exception {
		this.brandBatchDetails = brandBatchDetails;
	}

	public BrandBatchDetailsSeq(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(16);
		brandBatchDetails = new BrandBatchDetails[adv.length];
		for(int i=0; i<brandBatchDetails.length; i++) {
			brandBatchDetails[i] = new BrandBatchDetails(new DerInputStream(adv[i].toByteArray()));
		}
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		for(int i=0; i<brandBatchDetails.length; i++) {
			brandBatchDetails[i].encode(dout);
		}
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}
	
	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {

		String s = "<BrandBatchDetailsSeq>\n";
		for(int i=0; i<brandBatchDetails.length; i++) {
			s += "[" + i + "]\n" + brandBatchDetails[i] + "\n";
		}
		return s;
	}

	public static void main(String[] args) {

		try {
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
			System.out.println(seq);
			DerOutputStream dout = new DerOutputStream();
			seq.encode(dout);
			seq = new BrandBatchDetailsSeq(new DerInputStream(dout.toByteArray()));
			System.out.println(seq);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	} 
}