package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.payment.component.CurrencyAmount;

/**
 *	<pre>
 *	1384 CapResPayload ::= SEQUENCE {
 *	1385    capCode           CapCode,
 *	1386    capAmt            CurrencyAmount,
 *	1387    batchID           [0] BatchID  OPTIONAL,
 *	1388    batchSequenceNum  [1] BatchSequenceNum  OPTIONAL,
 *	1389    cRsPayExtensions  [2] MsgExtensions {{CRsPayExtensionsIOS}} OPTIONAL
 *	1390 }
 *
 *	1394 CapCode ::= ENUMERATED {
 *	1395    success             (0),
 *	1396    unspecifiedFailure  (1),
 *	1397    duplicateRequest    (2),
 *	1398    authExpired         (3),
 *	1399    authDataMissing     (4),
 *	1400    invalidAuthData     (5),
 *	1401    capTokenMissing     (6),
 *	1402    invalidCapToken     (7),
 *	1403    batchUnknown        (8),
 *	1404    batchClosed         (9),
 *	1405    unknownXID         (10),
 *	1406    unknownLID         (11)
 *	1407 }
 *
 *	1812 BatchID ::= INTEGER (0..MAX)
 *
 *	1814 BatchSequenceNum ::= INTEGER (1..MAX)
 *	</pre>
 */

public class CapResPayload implements DerEncoder {

	//fields
	public BigInt 		capCode;
	public CurrencyAmount	capAmt;
	public BigInt		batchID;
	public BigInt		batchSequenceNum;		

	//constructor
	public CapResPayload(BigInt capCode, CurrencyAmount capAmt,
				   BigInt batchID, BigInt batchSequenceNum) throws Exception {
		this.capCode = capCode;
		this.capAmt = capAmt;
		this.batchID = batchID;
		this.batchSequenceNum = batchSequenceNum;
	}

	public CapResPayload(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		DerValue dv;

		capCode = new BigInt(din.getEnumerated().toInt());
		capAmt = new CurrencyAmount(din);
					
		if((byte)din.peekByte()==(byte)0x80) {
			dv = din.getDerValue();
			dv.resetTag((byte)0x02);
			batchID = new BigInt(dv.getInteger().toInt());
		}
	
		if((byte)din.peekByte()==(byte)0x81) {
			dv = din.getDerValue();
			dv.resetTag((byte)0x02);
			batchSequenceNum = new BigInt(dv.getInteger().toInt());
		}
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putEnumerated(capCode);
		capAmt.encode(dout);
		
		if(batchID!=null) {
			DerOutputStream temp = new DerOutputStream();
			temp.putInteger(batchID);
			dout.writeImplicit((byte)0x80,temp);
		}

		if(batchSequenceNum!=null) {
			DerOutputStream temp1 = new DerOutputStream();
			temp1.putInteger(batchSequenceNum);
			dout.writeImplicit((byte)0x81,temp1);
		}

		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<CapResPayload>\n";
		s += "capCode: " + capCode + "\n";
		s += "capAmt: " + capAmt + "\n";
		if(batchID!=null)
			s += "batchID: " + batchID + "\n";
		if(batchSequenceNum!=null)
			s += "batchSequenceNum: " + batchSequenceNum + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			CapResPayload crp = new CapResPayload(new BigInt(0),cur,new BigInt(812), new BigInt(317));
			System.out.println(crp);

			DerOutputStream dout = new DerOutputStream();
			crp.encode(dout);

			crp = new CapResPayload(new DerInputStream(dout.toByteArray()));
			System.out.println(crp);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}	