package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.core.encapsulation.*;
import digsec.set.core.pkcs.ContentInfo;
import digsec.set.payment.component.*;
import java.security.*;
import java.security.cert.*;


/**
 *	<pre>
 *	1816 CapToken ::= CHOICE {
 *	1817    encX  [0] EXPLICIT EncX { P1, P2, CapTokenData, PANToken },
 *	1818    enc   [1] EXPLICIT Enc { P1, P2, CapTokenData },
 *	1819    null  [2] EXPLICIT NULL
 *	1820 }
 *	</pre>
 */

public class CapToken implements DerEncoder {

	//fields
	public Enc			enc;
	public EncX		encX;

	public CapTokenData	capTokenData;
	public PANToken		panToken;

	
	//constructor
	public CapToken(X509Certificate s_cert, PrivateKey s_priv, X509Certificate r_cert,
			    CapTokenData capTokenData, PANToken panToken) throws Exception {
		this.capTokenData = capTokenData;
		this.panToken	= panToken;

		DerOutputStream dout = new DerOutputStream();
		capTokenData.encode(dout);
		
		if(panToken==null) {
			enc = new Enc(s_cert,s_priv,r_cert, 
					  new ContentInfo(new DerInputStream(dout.toByteArray())));
		}
		else {
			encX = new EncX(s_cert,s_priv,r_cert, 
					  new ContentInfo(new DerInputStream(dout.toByteArray())),panToken);
		}
	}

	public CapToken(PrivateKey r_priv, DerInputStream din) throws Exception {

		if((byte)din.peekByte()==(byte)0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			din = dv.toDerInputStream();
			enc = new Enc(r_priv,din);
			ContentInfo cInfo = enc.getContentInfo();
			DerOutputStream dout = new DerOutputStream();
			cInfo.encode(dout);
			capTokenData = new CapTokenData(new DerInputStream(dout.toByteArray()));
		}
		else if((byte)din.peekByte()==(byte)0xA1) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)48);
			din = dv.toDerInputStream();
			encX = new EncX(r_priv,din);
			ContentInfo cInfo = encX.getContentInfo();
			DerOutputStream dout = new DerOutputStream();
			cInfo.encode(dout);
			capTokenData = new CapTokenData(new DerInputStream(dout.toByteArray()));
			panToken = (PANToken) encX.getParameter();
		}
	}	
	
	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream dout2 = new DerOutputStream();		

		if(enc!=null) {
			enc.derEncode(dout);
			dout2.write((byte)0xA0, dout);
		}
		else {
			encX.derEncode(dout);
			dout2.write((byte)0xA1, dout);
		}
				
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<CapToken>\n";
		s += capTokenData;
		if(panToken!=null)
			s += panToken;
		return s;
	}

	public static void main(String[] args) {
		try {
			//get payment-gateway's certificate
			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate p_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get payment-gateway's private key
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\keystore\\pg.jks");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"PaymentGateway".toCharArray());
			PrivateKey p_priv = (PrivateKey) ks.getKey("PG", "PaymentGateway".toCharArray());

			byte[] authAmt = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  

			CapTokenData ctd = new CapTokenData(authAmt,cur);

			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);

			CapToken ct = new CapToken(p_cert,p_priv,p_cert,ctd,pToken);
			System.out.println(ct);
			
			DerOutputStream dout = new DerOutputStream();
			ct.encode(dout);

			ct = new CapToken(p_priv,new DerInputStream(dout.toByteArray()));
			System.out.println(ct);	

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}