package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;
import digsec.set.payment.component.*;
import digsec.set.core.crypto.FalseVerifiedException;
import digsec.set.core.pkcs.ContentInfo;

/**
 *	<pre>
 *	1835 CapTokenData ::= SEQUENCE {
 *	1836    authRRPID    RRPID,
 *	1837    authAmt      CurrencyAmount,
 *	1838    tokenOpaque  TokenOpaque
 *	1839 }
 *	</pre>
 */

public class CapTokenData implements DerEncoder {

	//fields
	public byte[]		authRRPID; //20 bytes
	public CurrencyAmount	authAmt;
	
	public static ObjectIdentifier id_set_content_CapTokenData = new ObjectIdentifier("2.23.42.0.20");	

	//constructor
	public CapTokenData(byte[] authRRPID, CurrencyAmount authAmt) throws Exception {
		this.authRRPID = authRRPID;
		this.authAmt = authAmt;
	}

	public CapTokenData(DerInputStream din) throws Exception {

		ContentInfo cInfo = new ContentInfo(din);
		if(!cInfo.getContentType().equals(id_set_content_CapTokenData))
			throw new FalseVerifiedException("CapTokenData");
		din = cInfo.getContent().toDerInputStream();
		authRRPID = din.getOctetString();
		authAmt = new CurrencyAmount(din);
	}

	public void derEncode(OutputStream out) throws IOException {

		DerOutputStream dout = new DerOutputStream();
		dout.putOctetString(authRRPID);
		authAmt.encode(dout);
		DerOutputStream dout2 = new DerOutputStream();
		dout2.write((byte)48, dout);
		
		ContentInfo cInfo = new ContentInfo(id_set_content_CapTokenData,
								new DerValue(dout2.toByteArray()));
		dout.reset();
		cInfo.encode(dout);
		out.write(dout.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<CapTokenData>\n";
		s += "authRRPID: ";
		for(int i=0 ;i<authRRPID.length; i++)
			s += Byte.toString(authRRPID[i]) + " ";
		s += "\n";
		s += "authAmt:\n" + authAmt;
		return s;
	}

	public static void main(String[] args) {
		try {
			byte[] authAmt = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  

			CapTokenData ctd = new CapTokenData(authAmt,cur);
			System.out.println(ctd);
			
			DerOutputStream dout = new DerOutputStream();
			ctd.encode(dout);

			ctd = new CapTokenData(new DerInputStream(dout.toByteArray()));
			System.out.println(ctd);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}  

}