package digsec.set.payment.message.AuthReqRes;

import digsec.set.payment.component.*;
import java.io.*;
import sun.security.util.*;
import java.util.*;

/**
 *	<pre>
 *	1010 CheckDigests ::= SEQUENCE {
 *	1011    hOIData   HOIData,
 *	1012    hod2      HOD
 *	1013 }
 *	</pre>
 */

public class CheckDigests implements DerEncoder {

	//fields
	public HOIData 	hOIData;
	public HOD	  	hod2;

	//Constructor
	public CheckDigests(HOIData hOIData, HOD hod2) throws Exception {
		this.hOIData = hOIData;
		this.hod2	 = hod2;
	}

	public CheckDigests(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);
		hOIData = new HOIData(new DerInputStream(adv[0].toByteArray()));
		hod2 = new HOD(new DerInputStream(adv[1].toByteArray()));
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		hOIData.encode(dout);
		hod2.encode(dout);
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<CheckDigests>\n";
		s += hOIData + "\n";
		s += hod2 + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			String strOD = "Ini barang-barang yang kubeli";
			byte[] od = strOD.getBytes();

			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),new BigInt(2));  
			byte[] odSalt = {1,4,33,2,1,67,88,23,12,8,1,4,33,2,1,67,88,23,12,8};

			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);

			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			HODInput hodi = new HODInput(od,cur,odSalt,install);  

			HOD hod2 = new HOD(hodi);

			//*******bikin transIDs*********
			byte[] lid_C = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};
			TransIDs transIDs = new TransIDs(lid_C,null,"Priharsanta Inc.", "Javanese");

			//*******bikin RRPID*********
			byte[] rrpid = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin chall-C*********
			byte[] challC = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin HOD*********
			//udah ada di atas: hod2

			//*******bikin chall-M*********
			byte[] challM = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20};

			//*******bikin BrandID*********
			String brandID = "Master Mind Credit Card";

			//*******bikin BIN*********
			String bin = "123456";

			//******bikin OIData*******
			OIData oidata = new OIData(transIDs,rrpid,challC,hod2,odSalt,challM,brandID,bin);

			//******baru deh bikin HOIData*******
			HOIData hoiData = new HOIData(oidata);

			//******baru juga bikin CheckDigests*****
			CheckDigests cd = new CheckDigests(hoiData,hod2);
			System.out.println(cd);

			DerOutputStream dout = new DerOutputStream();
			cd.encode(dout);
 			
			cd = new CheckDigests(new DerInputStream(dout.toByteArray()));
			System.out.println(cd);

		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}
