package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;


/**
 *	<pre>
 *	1878 MarketSpecAuthData ::= CHOICE {
 *	1879    auto-rental  [0] MarketAutoAuth,
 *	1880    hotel        [1] MarketHotelAuth,
 *	1881    transport    [2] MarketTransportAuth
 *	1882 }
 *
 *
 *	1860 MarketAutoAuth ::= SEQUENCE {
 *	1861    duration  Duration
 *	1862 }
 *
 *	1895 MarketTransportAuth ::= NULL
 *
 *	1864 MarketHotelAuth ::= SEQUENCE {
 *	1865    duration  Duration,
 *	1866    prestige  Prestige  OPTIONAL
 *	1867 }
 *
 *	1869 Duration ::= INTEGER (1..99)                           -- Number of days
 *
 *	1871 Prestige ::= ENUMERATED {
 *	1872    unknown  (0),
 *	1873    level-1  (1),  -- Transaction floor limits for each level are
 *	1874    level-2  (2),  -- defined by brand policy and may vary between
 *	1875    level-3  (3)   -- national markets.
 *	1876 }
 *	
 *	</pre>
 */

public class MarketSpecAuthData implements DerEncoder {

	//fields
	public BigInt duration;
	public BigInt prestige;

	//Constructor
	public MarketSpecAuthData(BigInt duration, BigInt prestige) throws Exception {
		this.duration = duration;
		this.prestige = prestige;
	}

	public MarketSpecAuthData(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);
		duration = new BigInt(adv[0].getInteger().toInt());
		if(adv[1].tag!=(byte)0x05)
			prestige = new BigInt(adv[1].getEnumerated().toInt());
		else
			prestige = null;
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		dout.putInteger(duration);
		if(prestige!=null)
			dout.putEnumerated(prestige);
		else
			dout.putNull();
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<MarketSpecAuthData>\n";
		s += "duration: " + duration + "\n";
		if(prestige!=null)
		s += "prestige: " + prestige + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			MarketSpecAuthData market = new MarketSpecAuthData(new BigInt(812),
											   new BigInt(317));
			System.out.println(market);
			DerOutputStream dout = new DerOutputStream();
			market.encode(dout);
			market = new MarketSpecAuthData(new DerInputStream(dout.toByteArray()));
			System.out.println(market);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
	
}