package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;

/**
 *	<pre>
 *	1049 MerchData ::= SEQUENCE {
 *	1050    merchCatCode  MerchCatCode  OPTIONAL,
 *	1051    merchGroup    MerchGroup  OPTIONAL
 *	1052 }
 *
 *	1054 MerchCatCode ::= NumericString (SIZE(ub-merType))  -- ANSI X9.10
 *	1055          -- Merchant Category Code (MCCs) are assigned by acquirer to
 *	1056          -- describe the merchant's product, service or type of business
 *
 *	1058 MerchGroup ::= ENUMERATED {
 *	1059    commercialTravel  (1),
 *	1060    lodging           (2),
 *	1061    automobileRental  (3),
 *	1062    restaurant        (4),
 *	1063    medical           (5),
 *	1064    mailOrPhoneOrder  (6),
 *	1065    riskyPurchase     (7),
 *	1066    other             (8)
 *	1067 }
 *	</pre>
 */

public class MerchData implements DerEncoder {

	//fields
	public String merchCatCode;
	public BigInt merchGroup;

	//Constructor
	public MerchData(String merchCatCode, BigInt merchGroup) throws Exception {
		this.merchCatCode = merchCatCode;
		this.merchGroup = merchGroup;
	}

	public MerchData(DerInputStream din) throws Exception {
		DerValue[] adv = din.getSequence(2);
		if(adv[0].tag!=(byte)0x05)
			merchCatCode = adv[0].getPrintableString();
		else
			merchCatCode = null;
		if(adv[1].tag!=(byte)0x05)
			merchGroup = new BigInt(adv[1].getEnumerated().toInt());
		else
			merchGroup = null;
	}

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		if(merchCatCode!=null)
			dout.putPrintableString(merchCatCode);
		else
			dout.putNull();
		if(merchGroup!=null)
			dout.putEnumerated(merchGroup);
		else
			dout.putNull();
		DerOutputStream dout1 = new DerOutputStream();
		dout1.write((byte)48,dout);
		out.write(dout1.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<MerchData>\n";
		if(merchCatCode!=null)
		s += "merchCatCode: " + merchCatCode + "\n";
		if(merchGroup!=null)
		s += "merchGroup: " + merchGroup + "\n";
		return s;
	}

	public static void main(String[] args) {
		try {
			MerchData market = new MerchData("Butique",
								   new BigInt(8));
			System.out.println(market);
			DerOutputStream dout = new DerOutputStream();
			market.encode(dout);
			FileOutputStream fo = new FileOutputStream("MerchData.dat");
			fo.write(dout.toByteArray());
			fo.close();

			FileInputStream fi = new FileInputStream("MerchData.dat");
			byte[] input = new byte[fi.available()];
			fi.read(input);
			market = new MerchData(new DerInputStream(input));
			System.out.println(market);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}	
}
