package digsec.set.payment.message.AuthReqRes;

import java.io.*;
import sun.security.util.*;

/**
 *	<pre>
 *	1162 ResponseData ::= SEQUENCE {
 *	1163    authValCodes  [0] AuthValCodes  OPTIONAL,
 *	1164    respReason    [1] RespReason  OPTIONAL,
 *	1165    cardType      CardType  OPTIONAL,
 *	1166    avsResult     [2] AVSResult  OPTIONAL,
 *	1167    logRefID      LogRefID  OPTIONAL
 *	1168 }
 *
 *	1177 RespReason ::= ENUMERATED {
 *	1178    issuer                    (0),
 *	1179    standInTimeOut            (1),
 *	1180    standInFloorLimit         (2),
 *	1181    standInSuppressInquiries  (3),
 *	1182    standInIssuerUnavailable  (4),
 *	1183    standInIssuerRequest      (5)
 *	1184 }
 *
 *	1186 CardType ::= ENUMERATED {
 *	1187    unavailable            ( 0),
 *	1188    classic                ( 1),
 *	1189    gold                   ( 2),
 *	1190    platinum               ( 3),
 *	1191    premier                ( 4),
 *	1192    debit                  ( 5),
 *	1193    pinBasedDebit          ( 6),
 *	1194    atm                    ( 7),
 *	1195    electronicOnly         ( 8),
 *	1196    unspecifiedConsumer    ( 9),
 *	1197    corporateTravel        (10),
 *	1198    purchasing             (11),
 *	1199    business               (12),
 *	1200    unspecifiedCommercial  (13),
 *	1201    privateLabel           (14),
 *	1202    proprietary            (15)
 *	1203 }
 *
 *	1205 AVSResult ::= ENUMERATED {
 *	1206    resultUnavailable    (0),
 *	1207    noMatch              (1),
 *	1208    addressMatchOnly     (2),
 *	1209    postalCodeMatchOnly  (3),
 *	1210    fullMatch            (4)
 *	1211 }
 *
 *	1213 LogRefID ::= NumericString (SIZE(1..ub-logRefID))
 *
 *	</pre>
 */

public class ResponseData implements DerEncoder {

	//fields
	public AuthValCodes 	authValCodes;
	public BigInt		respReason;
	public BigInt		cardType;
	public BigInt		avsResult;	
	public String		logRefID;

	//constructor
	public ResponseData(AuthValCodes 	authValCodes,
				  BigInt		respReason,
				  BigInt		cardType,
				  BigInt		avsResult,
				  String		logRefID) throws Exception {

		this.authValCodes = authValCodes;
		this.respReason = respReason;
		this.cardType = cardType;
		this.avsResult = avsResult;
		this.logRefID = logRefID;
	}

	public ResponseData(DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		DerValue dv;		

		if((byte)din.peekByte() == (byte)0xA0) {
			dv = din.getDerValue();
			dv.resetTag((byte)48);
			authValCodes = new AuthValCodes(new DerInputStream(dv.toByteArray()));
		}
		if((byte)din.peekByte() == (byte)0x81) {
			dv = din.getDerValue();
			dv.resetTag(DerValue.tag_Enumerated);
			respReason = dv.getEnumerated();
		}
		if((byte)din.peekByte() == DerValue.tag_Enumerated) {
			cardType = din.getEnumerated();
		}
		if((byte)din.peekByte() == (byte)0x82) {
			dv = din.getDerValue();
			dv.resetTag(DerValue.tag_Enumerated);
			avsResult = dv.getEnumerated();
		}
		if((byte)din.peekByte() == DerValue.tag_PrintableString) {
			logRefID = din.getDerValue().getPrintableString();
		}							
	}		

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream temp;
		if(authValCodes!=null) {
			temp = new DerOutputStream();
			authValCodes.encode(temp);
			dout.writeImplicit((byte)0xA0,temp);
		}
		if(respReason!=null) {
			temp = new DerOutputStream();
			temp.putEnumerated(respReason);
			dout.writeImplicit((byte)0x81,temp);
		}
		if(cardType!=null) {
			dout.putEnumerated(cardType);	
		}	
		if(avsResult!=null) {
			temp = new DerOutputStream();
			temp.putEnumerated(avsResult);
			dout.writeImplicit((byte)0x82,temp);
		}
		if(logRefID!=null) {
			dout.putPrintableString(logRefID);
		}
		temp = new DerOutputStream();
		temp.write((byte)48,dout);
		out.write(temp.toByteArray());
	}	

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<ResponseData>\n";
		if(authValCodes!=null) {
			s += "authValCodes: " + authValCodes + "\n";
		}
		if(respReason!=null) {
			s += "respReason: " + respReason + "\n";
		}
		if(cardType!=null) {
			s += "cardType: " + cardType + "\n";
		}	
		if(avsResult!=null) {
			s += "avsResult: " + avsResult + "\n";			
		}
		if(logRefID!=null) {
			s += "logRefID: " + logRefID + "\n";
		}
		return s;
	}

	public static void main(String[] args) {
		try {
			AuthValCodes avc = new AuthValCodes("dhany", AuthValCodes.AuthCharInd_directMarketing,
									"arif", AuthValCodes.MarketSpecDataID_hotel);
			
			ResponseData rd = new ResponseData(avc,new BigInt(8), new BigInt(1),
								     new BigInt(2), "accepted");
			System.out.println(rd);
			DerOutputStream dout2 = new DerOutputStream();
			rd.encode(dout2);
			rd = new ResponseData(new DerInputStream(dout2.toByteArray()));
			System.out.println(rd);		
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}