package digsec.set.payment.wrapper;

import java.io.*;
import sun.security.util.*;
import java.security.*;
import java.security.cert.*;
import digsec.set.core.pkcs.*;
import digsec.set.core.encapsulation.*;
import digsec.set.core.crypto.*;
import digsec.set.payment.component.*;
import java.util.*;
import digsec.set.payment.message.AuthReqRes.*;

/**
 *	<pre>	
 *  	43 MessageWrapper ::= SEQUENCE {
 * 	44    messageHeader  MessageHeader,
 * 	45    message        [0] EXPLICIT MESSAGE.&Type (Message),
 * 	46    mwExtensions   [1] MsgExtensions {{MWExtensionsIOS}} OPTIONAL
 * 	47 }
 *	</pre>
 */

public class MessageWrapper implements DerEncoder {

	//fields
	public MessageHeader 	messageHeader;
	public Message		message;

	public MessageWrapper (MessageHeader header, Message message) throws Exception {
		this.messageHeader = header;
		this.message = message;
	}

	public MessageWrapper (PrivateKey priv, DerInputStream din) throws Exception {
		din = din.getDerValue().toDerInputStream();
		messageHeader = new MessageHeader(din);

		if((byte)din.peekByte() == (byte)0xA0) {
			DerValue dv = din.getDerValue();
			dv.resetTag((byte)0x30);
			din = dv.toDerInputStream();
			
			if((byte)din.peekByte()==Message.tag_AuthReq)
				message = new AuthReq(priv,din);
			else if((byte)din.peekByte()==Message.tag_AuthRes)
				message = new AuthRes(priv,din);
		}
	}	

	public void derEncode(OutputStream out) throws IOException {
		DerOutputStream dout = new DerOutputStream();
		DerOutputStream dout2 = new DerOutputStream();

		messageHeader.encode(dout);
		if(message!=null) {
			message.encode(dout2);
			dout.write((byte)0xA0,dout2);
		}

		dout2.reset();
		dout2.write((byte)0x30,dout);
		out.write(dout2.toByteArray());
	}

	public void encode(DerOutputStream dout) throws IOException {
		derEncode(dout);
	}

	public String toString() {
		String s = "<MessageWrapper>\n";
		s += messageHeader;
		if(message!=null)
			s += message;
		return s;
	}

	public static void main(String[] args) {
		try {
			byte[] lidc = new byte[20];
			byte[] lidm = new byte[20];
			byte[] xid = new byte[20];
			byte[] rrpid = new byte[20];

			Random myRandom = new Random(5485076);
			myRandom.nextBytes(lidc);
			myRandom.setSeed(8642178);
			myRandom.nextBytes(lidm);
			myRandom.setSeed(7870481);
			myRandom.nextBytes(xid);
			myRandom.setSeed(5840502);
			myRandom.nextBytes(rrpid);

			MessageIDs mids = new MessageIDs(lidc,lidm,xid);

			Date date = new Date();
			String swident = "Bataknese";

			MessageHeader mh = new MessageHeader(date, mids, rrpid, swident);

			//===============================================================

			byte[] RRPID = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			MerTermIDs mti = new MerTermIDs("Arif","PCSC",
								  new BigInt(5),
								  new BigInt(6),
								  new BigInt(7));

			RRTags rr = new RRTags(RRPID,mti);

			//*******bikin transIDs*********
			byte[] lid_C = {8,1,2,3,1,7};
			byte[] lid_M = {2,1,0,3,1,7};
			Calendar calendar = Calendar.getInstance();
			calendar.set(1999,1,14);
			Date pReqDate = calendar.getTime();
			SecureRandom sr = new SecureRandom();	
			byte[] XID = new byte[20];
			sr.nextBytes(XID);

			TransIDs id = new TransIDs(lid_C,lid_M, XID, pReqDate, null, "English");

			AuthTags au = new AuthTags(rr,id,new BigInt(812));

			AuthCode ac = new AuthCode(AuthCode.approved);
			CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  
			AuthValCodes avc = new AuthValCodes("dhany", AuthValCodes.AuthCharInd_directMarketing,
									"arif", AuthValCodes.MarketSpecDataID_hotel);
			
			ResponseData rd = new ResponseData(avc,new BigInt(8), new BigInt(1),
								     new BigInt(2), "accepted");

			BatchTotals bt = new BatchTotals(new BigInt(100000), cur, new BigInt(2000000), cur);

			BrandBatchDetails bbd = new BrandBatchDetails("Dhany's",bt);
			BrandBatchDetails[] abbd = new BrandBatchDetails[3];
			abbd[0] = bbd;
			abbd[1] = bbd;
			abbd[2] = bbd;
			BrandBatchDetailsSeq seq = new BrandBatchDetailsSeq(abbd);
		
			BatchDetails bd = new BatchDetails(bt,seq);

			ClosedWhen cw = new ClosedWhen(ClosedWhen.closedbyMerchant,
							     new Date());

			BatchStatus bs = new BatchStatus(new Date(),cw, bd);

			AuthHeader ah = new AuthHeader(cur,ac,rd,bs);

			CapResPayload crp = new CapResPayload(new BigInt(0),cur,new BigInt(812), new BigInt(317));
		
			AuthResPayload arp = new AuthResPayload(ah,crp);

			AuthResData ard = new AuthResData(au,arp);

			//*************************************************
			//get payment-gateway's certificate
			InputStream inStream = new FileInputStream("d:\\digsec\\progie\\arif\\source\\digsec\\set\\application\\keystore\\pg.cert");
 			CertificateFactory cf = CertificateFactory.getInstance("X.509");
  			X509Certificate p_cert = (X509Certificate)cf.generateCertificate(inStream);
 			inStream.close();
			
			//get payment-gateway's private key
			inStream = new FileInputStream("d:\\digsec\\progie\\arif\\source\\digsec\\set\\application\\keystore\\pg.jks");
			java.security.KeyStore ks = java.security.KeyStore.getInstance("JKS", "SUN");
			ks.load(inStream,"PaymentGateway".toCharArray());
			PrivateKey p_priv = (PrivateKey) ks.getKey("PG", "PaymentGateway".toCharArray());

			byte[] authAmt = {8,1,2,3,1,7,8,1,2,3,1,7,8,1,2,3,1,7,2,3};
			//CurrencyAmount cur = new CurrencyAmount(new BigInt(120),new BigInt(150),    new BigInt(2));  

			CapTokenData ctd = new CapTokenData(authAmt,cur);

			byte[] ab = {0,1,2,3,4,5,6,7,8,9,0,1,2,3,4,5,6,7,8,9};
			PANToken pToken = new PANToken("1294", "199908", ab);

			CapToken ct = new CapToken(p_cert,p_priv,p_cert,ctd,pToken);

			//byte[] lid_C = {8,1,2,3,1,7};
			//TransIDs id = new TransIDs(lid_C,lid_C,"Priharsanta Inc.", "Javanese");
			
			Calendar cal = Calendar.getInstance();
			cal.set(1999,11,8);
			Recurring rec = new Recurring(new BigInt(28),
								cal.getTime());
			InstallRecurData install = new InstallRecurData(rec);

			AuthTokenData auth = new AuthTokenData(id,cur,"dodol812",install,
									   new BigInt(1), new Date(), 
									   null);
					
			AuthToken authToken = new AuthToken(p_priv,p_cert,p_cert,auth,pToken);

			AuthResBaggage arb = new AuthResBaggage(ct,authToken);
			
			//*******************************************

			AuthRes ar = new AuthRes(p_cert,p_priv,p_cert,ard,arb,pToken);
			Message msg = ar;

			MessageWrapper mw = new MessageWrapper(mh,msg);
			System.out.println(mw);

			DerOutputStream dout = new DerOutputStream();
			mw.encode(dout);

			mw = new MessageWrapper(p_priv, new DerInputStream(dout.toByteArray()));
			System.out.println(mw);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
}